Ext.define('App.controller.Navigation', {
    extend: 'Ext.app.Controller',
    refs: [
        {
            ref: 'userNameContainer',
            selector: '#navigationUserName'
        },
        {
            ref: 'userSettingsContainer',
            selector: '#navigationUserSettings'
        },
        {
            ref: 'userCloseContainer',
            selector: '#navigationUserClose'
        },
        {
            ref: 'userUpdatedbContainer',
            selector: '#navigationUserUpdatedb'
        },
        {
            ref: 'progressLeftDateContainer',
            selector: '#navigationProgressLeftDate'
        },
        {
            ref: 'progressRightDateContainer',
            selector: '#navigationProgressRightDate'
        },
        {
            ref: 'navigationInformerContainer',
            selector: 'navigationInformer'
        }
    ],
    init: function() {
        this.addEvents('redrawInformer');
        this.on('redrawInformer', this.onRedrawInformer, this);

        this.synchronizer = App.storage.Synchronizer;

        this.getUserSettingsContainer().getEl().on('click', this.onSettingsClick, this);
        this.getUserUpdatedbContainer().getEl().on('click', this.onUpdatedbClick, this);
        this.redrawInformer();

        this.callParent(arguments);
    },

    /**
     * Fires when user clicks "Update" link, we should start synchronization
     */
    onUpdatedbClick: function() {
        window.bootstrap.incrementalSynchronization();
    },

    /**
     * Fires when user clicks "Settings" link, we should show settings dialog.
     */
    onSettingsClick: function() {
        if (!this.settingsWindow) {
            this.settingsWindow = Ext.create('App.view.navigation.Settings', {
                period: User['period'],
                slaPeriod: User['sla_period'],
                missCallSecPeriod: User['miss_call_sec_period'],
                gradeOfAccuracy: User['grade_of_accuracy']
            });
            this.settingsWindow.on('submit', this.onSettingsSubmit, this);
        }
        this.settingsWindow.show();
    },

    /**
     * Fires when user wants to save a settings, we should show loader and send request to a server.
     */
    onSettingsSubmit: function(period, slaPeriod, missCallSecPeriod, gradeOfAccuracy) {
        this.saveSettingsPeriod = period;
        this.saveSettingsSlaPeriod = slaPeriod;
        this.saveSettingsMissCallSecPeriod = missCallSecPeriod;
        this.saveGradeOfAccuracy = gradeOfAccuracy;
        this.saveSettingsNecessary = false;
        this.saveSettings(this.saveSettingsPeriod, this.saveSettingsSlaPeriod, this.saveSettingsMissCallSecPeriod, this.saveGradeOfAccuracy);
    },

    /**
     * Saves user settings on a server.
     */
    saveSettings: function(period, slaPeriod, missCallSecPeriod, gradeOfAccuracy) {

        var params = {
            period: period,
            sla_period: slaPeriod,
            miss_call_sec_period: missCallSecPeriod,
            grade_of_accuracy: gradeOfAccuracy
        };

        Ext.Ajax.request({
            url: '../../api/v1/CDRView/Settings/',
            method: 'put',
            params: params,
            success: this.onSaveSettingsSuccess,
            failure: this.onSaveSettingsFailure,
            scope: this
        });
    },

    /**
     * Fires when request for settings change was completed successfylly, we should handle a response.
     */
    onSaveSettingsSuccess: function(response) {
        if (User['period'] != this.saveSettingsPeriod || User['sla_period'] != this.saveSettingsSlaPeriod || User['miss_call_sec_period'] != this.saveSettingsMissCallSecPeriod || User.gradeOfAccuracy != this.saveGradeOfAccuracy) {
            User['prev_period'] = User['period'];
            User['period'] = this.saveSettingsPeriod;
            User['sla_period'] = this.saveSettingsSlaPeriod;
            User['miss_call_sec_period'] = this.saveSettingsMissCallSecPeriod;
            User.gradeOfAccuracy = this.saveGradeOfAccuracy;

            window.bootstrap.incrementalSynchronization();
        }
        this.getController('Viewport').processResponse(response.responseText, this.onSaveSettingsDecoded, this.onSaveSettingsRetry, this);
    },

    /**
     * Fires when response is valid and we should handle a result.
     */
    onSaveSettingsDecoded: function(result, text) {
        this.settingsWindow.close();

        // TODO: Verify if data has for selected period if its doesn't we should sync selected period.
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond,
     * we should show a dialog and retry after a timeout.
     */
    onSaveSettingsFailure: function(response) {
        this.getController('Viewport').fireEvent('connectionError', this.onSaveSettingsRetry, this);
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onSaveSettingsRetry: function(necessary) {
        this.saveSettingsNecessary = necessary;
        this.saveSettings(this.saveSettingsPeriod, this.saveSettingsSlaPeriod, this.saveSettingsMissCallSecPeriod);
    },

    /**
     * Fires when user synchronize some events.
     */
    onRedrawInformer: function() {
        this.redrawInformer();
    },

    redrawInformer: function() {
        this.synchronizer.getEventsStatistics(this.onGetEventsStatisticsComplete, this);
    },

    onGetEventsStatisticsComplete: function(range) {
        if(App.storage.Synchronizer.getBackendMinDate() == null || App.storage.Synchronizer.getBackendMaxDate() == null) {
            return false;
        }

        this.getProgressLeftDateContainer().update(Ext.Date.dateFormat(Ext.Date.parse(App.storage.Synchronizer.getBackendMinDate(), 'Y-m-d H:i:s'), 'd/m/Y H:i'));
        this.getProgressRightDateContainer().update(Ext.Date.dateFormat(Ext.Date.parse(App.storage.Synchronizer.getBackendMaxDate(), 'Y-m-d H:i:s'), 'd/m/Y H:i'));

        this.getNavigationInformerContainer().drawBar(
            range,
            App.storage.Synchronizer.getBackendMinId(),
            App.storage.Synchronizer.getBackendMaxId()
        );
    }

} );
