Ext.define('App.controller.Report', {
    extend: 'Ext.app.Controller',
    refs: [

        // Filter refs
        {
            ref: 'filterDestinationField',
            selector: '#filterDestinationField'
        },
        {
            ref: 'filterCostField',
            selector: '#filterCostField'
        },
        {
            ref: 'filterBillField',
            selector: '#filterBillField'
        },
        {
            ref: 'filterPeriodField',
            selector: '#filterPeriodField'
        },
        {
            ref: 'filterTypesField',
            selector: '#filterTypesField'
        },
        {
            ref: 'filterTrunksField',
            selector: '#filterTrunksField'
        },
        {
            ref: 'filterPbxesField',
            selector: '#filterPbxesField'
        },
        {
            ref: 'filterTagsField',
            selector: '#filterTagsField'
        },
        {
            ref: 'filterForm',
            selector: '#filterForm'
        },
        {
            ref: 'filterContainer',
            selector: 'filter'
        },
        {
            ref: 'filterResetButton',
            selector: '#filterResetButton'
        },
        {
            ref: 'filterApplyButton',
            selector: '#filterApplyButton'
        },
        {
            ref: 'filterSaveButton',
            selector: '#viewerReportSaveButton'
        },
        {
            ref: 'reportPicker',
            selector: '#viewerReportPicker'
        }
    ],
    init: function() {
        var self = this;

        // Filter validation
        this.trunksProxy = App.storage.Storage.proxyTrunks;
        this.pbxesProxy = App.storage.Storage.proxyPbxes;

        this.getFilterResetButton().on('click', this.onFilterReset, this);
        this.getFilterApplyButton().on('click', this.onFilterApply, this);
        this.getFilterSaveButton().on('click',this.onFilterSave, this);

        this.trunksProxy.readCount(this.onTrunksCountLoaded, this);
        this.pbxesProxy.readCount(this.onPbxesCountLoaded, this);

        this.setFilterState(this.getDefaultFilterState());

        // Define filter view heights
        this.filterHeight = this.getFilterContainer().getHeight();
        this.filterCollapsedHeight = 26;

        this.getFilterForm().getHeader().el
                            .on('click', this.onFilterFormHeaderDblClick, this);
        this.getFilterForm().on('collapse', this.onFilterFormCollapse, this);
        this.getFilterForm().on('expand', this.onFilterFormExpand, this);

        // Reports events
        this.getReportPicker().on('selectReport', this.onSelectReport, this);
        this.getReportPicker().on('deleteReport', this.onDeleteReport, this);
        this.getReportPicker().on('fetchPersonalReports', this.onFetchPersonalReports, this);
        this.getReportPicker().on('fetchSharedReports', this.onFetchSharedReports, this);

        // Automatically load default data
        this.ignoreFilterCollapse = true;
        this.callParent(arguments);
    },

    // ---

    /**
     * Fires when we process results, here we should update a description
     */
    onReport: function(grouping, metric, filter) {
        if (!this.ignoreFilterCollapse && !this.getFilterForm().collapsed) {
            this.collapseFilter();
        }

        this.ignoreFilterCollapse = false;
        this.getFilterForm().setTitle(this.getGroupingDescription(grouping) + " <span style=\"color:#333; font-weight: 400;\">" + this.getFilterDescription() + "</span>");
        this.pbxesProxy.readCount(this.onPbxesCountLoaded, this);
    },

    // ---

    /**
     * Fires when user is clicked to "Save this Report" button.
     */
    onFilterSave: function() {
        this.saveDialog = Ext.create('App.view.reports.Save');
        this.saveDialog.on('submit', this.onSaveDialogSubmit, this);
        this.saveDialog.on('cancel', this.onSaveDialogCancel, this);
        this.saveDialog.show();
    },

    /**
     * Fires when user clicked to Submit button at Save Dialog Window.
     */
    onSaveDialogSubmit: function(report) {
        this.saveDialog.showSavingState();
        report['name'] = report['name'].replace(/</g, "&lt;");
        report['name'] = report['name'].replace(/>/g, "&gt;");
        report['name'] = report['name'].replace(/\"/g, "&quot;");
        report['description'] = report['description'].replace(/</g, "&lt;");
        report['description'] = report['description'].replace(/>/g, "&gt;");
        report['description'] = report['description'].replace(/\"/g, "&quot;");

        // Prepare states
        var state = new Object();

        state['filter'] = this.getFilterState();
        state['grouping'] = this.getGroupingState();
        state['metric'] = this.getMetricState();
        state['view'] = this.getViewState();
        state['chart'] = this.getChartState();
        state['table'] = this.getTableState();

        report['filter'] = Ext.encode(state);
        report['share'] = report['share'] ? 'shared' : 'private';

        this.savingReport = report;
        this.saveReport();
    },

    /**
     * Makes a request to server to save report.
     */
    saveReport: function(necessary) {
        var report = this.savingReport;

        Ext.Ajax.request({
        	url: '../../api/v1/CDRView/Filters/',
            method: 'post',
            params: {
            	name: report['name'],
            	description: report['description'],
            	shared: ((report['share'] == 'private')? 0: 1),
            	filter: report['filter']
            },
            success: this.onSaveReportSuccess,
            failure: this.onSaveReportFailure,
            scope: this
        });
    },

    /**
     * Fires when request was completed successfully, we should handle a response.
     */
    onSaveReportSuccess: function(response) {
        this.getController('Viewport').processResponse(response.responseText, this.onSaveReportDecoded, this.onSaveReportFailure, this);
    },

    /**
     * Fires when response is valid, we should invoke a reports picker that reports has been successfully loaded.
     */
    onSaveReportDecoded: function(result, text) {
        this.saveDialog.close();
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond,
     * we should show a dialog and retry after a timeout.
     */
    onSaveReportFailure: function(response) {
        this.getController('Viewport').fireEvent('connectionError', this.onSaveReportRetry, this);
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onSaveReportRetry: function(necessary) {
        this.saveReport(necessary);
    },

    /**
     * Fires when user clicks to Cancel button, we should just close a dialog.
     */
    onSaveDialogCancel: function() {
        this.saveDialog.close();
    },

    /**
     * Fires when user select a "Personal reports" category, we should mask application and waits for valid response.
     */
    onFetchPersonalReports: function(callback, scope) {
        this.fetchPersonalCallback = callback;
        this.fetchPersonalScope = scope;
        this.fetchPersonalReports();
    },

    /**
     * Makes a request to server for a personal reports.
     */
    fetchPersonalReports: function(necessary) {
        // Ajax
        Ext.Ajax.request({
            url: '../../api/v1/CDRView/Filters/?filter[shared]=0',
            method: 'get',
            success: this.onFetchPersonalReportsSuccess,
            failure: this.onFetchPersonalReportsFailure,
            scope: this
        });
    },

    /**
     * Fires when request for settings change was completed successfylly, we shouldd handle a response.
     */
    onFetchPersonalReportsSuccess: function(response) {
        this.getController('Viewport').processResponse(response.responseText, this.onFetchPersonalReportsDecoded, this.onFetchPersonalReportsFailure, this);
    },

    /**
     * Fires when response is valid, we should invoke a reports picker that reports has been successfully loaded.
     */
    onFetchPersonalReportsDecoded: function(result, text) {
        this.getReportPicker().fireEvent('fetchedPersonalReports', this.decodeReportsResult(result));
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond,
     * we should show a dialog and retry after a timeout.
     */
    onFetchPersonalReportsFailure: function(response) {
        this.getController('Viewport').fireEvent('connectionError', this.onFetchPersonalReportsRetry, this);
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onFetchPersonalReportsRetry: function(necessary) {
        this.fetchPersonalReports(necessary);
    },


    /**
     * Fires when user select a "Shared reports" category, we should mask application and waits for valid response.
     */
    onFetchSharedReports: function(callback, scope) {
        this.fetchSharedCallback = callback;
        this.fetchSharedScope = scope;
        this.fetchSharedReports();
    },

    /**
     * Makes a request to server for a Shared reports.
     */
    fetchSharedReports: function(necessary) {
        // Ajax
        Ext.Ajax.request({
        	url: '../../api/v1/CDRView/Filters/?filter[shared]=1',
            method: 'get',
            success: this.onFetchSharedReportsSuccess,
            failure: this.onFetchSharedReportsFailure,
            scope: this
        });
    },

    /**
     * Fires when request for settings change was completed successfylly, we should handle a response.
     */
    onFetchSharedReportsSuccess: function(response) {
        this.getController('Viewport').processResponse(response.responseText, this.onFetchSharedReportsDecoded, this.onFetchSharedReportsFailure, this);
    },

    /**
     * Fires when response is valid, we should invoke a reports picker that reports has been successfully loaded.
     */
    onFetchSharedReportsDecoded: function(result, text) {
        this.getReportPicker().fireEvent('fetchedSharedReports', this.decodeReportsResult(result));
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond,
     * we should show a dialog and retry after a timeout.
     */
    onFetchSharedReportsFailure: function(response) {
        this.getController('Viewport').fireEvent('connectionError', this.onFetchSharedReportsRetry, this);
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onFetchSharedReportsRetry: function(necessary) {
        this.fetchSharedReports(necessary);
    },

    /**
     * Decodes a reports that was fetched from server
     */
    decodeReportsResult: function(reports) {
        var out = new Array;

        if(reports && reports.type == 'result' && reports.result && reports.result.total > 0 && reports.result.records){
            for(var i = 0; i < reports.result.records.length; i++){
            	var report = reports.result.records[i];
                var obj = new Object;
                obj.id = report['id'];
                obj.user = report['uid'];
                obj.name = report['name'];
                obj.description = report['description'];
                obj.state = Ext.JSON.decode(report['filter']);
                obj.deletable = true;
                out.push(obj);
            }
        }

        return out;
    },

    /**
     * Fires when user select one of a prepared, shared or personal reports.
     */
    onSelectReport: function(report) {
        var s = report.get('state');

        this.setViewState(s['view']);
        this.setFilterState(s['filter']);
        this.setGroupingState(s['grouping']);
        this.setMetricState(s['metric']);

        if (s.view == 'TABLE') {
            this.setTableState(s);
        } else {
            this.setChartState(s);
        }

        this.onFilterApply();

        this.name = report.get('name');
        // not available
        // this.periodFilterName = FilterPeriodField.getTypeDescription(s.filter.period.type);
        this.periodFilterName = this.getFilterPeriodField().value;
    },

    /**
     * Fires when user deletes a shared or personal report, we should mask application and waits for valid response.
     */
    onDeleteReport: function(report) {
        this.deletingReport = report;

        Ext.Msg.confirm('Confirmation', sprintf('Delete %(reportName)s report ?', { reportName: report.get('name') }), this.onDeleteReportConfirmed, this);
    },

    /**
     * Fires when user has confirmed report delete action.
     */
    onDeleteReportConfirmed: function(button) {
        if (button == 'yes') {
            Ext.getBody().mask(_t('Please wait...')+' '+_t('Deleting...'));
            this.deleteReport();
        }
    },

    /**
     * Makes a request to server to delete a report.
     */
    deleteReport: function(necessary) {
        // Ajax
        var id = this.deletingReport.get('id');

        Ext.Ajax.request({
            url: '../../api/v1/CDRView/Filters/' + id + '/',
            method: 'delete',
            success: this.onDeleteReportSuccess,
            failure: this.onDeleteReportFailure,
            scope: this
        });
    },

    /**
     * Fires when request for settings change was completed successfully, we should handle a response.
     */
    onDeleteReportSuccess: function(response) {
        this.getController('Viewport').processResponse(response.responseText, this.onDeleteReportDecoded, this.onDeleteReportFailure, this);
    },

    /**
     * Fires when response is valid, we should hide loader.
     */
    onDeleteReportDecoded: function(result, text) {
        Ext.getBody().unmask();
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond,
     * we should show a dialog and retry after a timeout.
     */
    onDeleteReportFailure: function(response) {
        this.getController('Viewport').fireEvent('connectionError', this.onDeleteReportRetry, this);
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onDeleteReportRetry: function(necessary) {
        this.deleteReport(necessary);
    },

    // Filter events for layouting
    onFilterFormHeaderDblClick: function() {
        if(this.getFilterForm().collapsed) {
            this.expandFilter();
        } else {
            this.collapseFilter();
        }
    },

    onFilterFormCollapse: function() {
        this.getFilterContainer().setHeight(this.filterCollapsedHeight);

        // Show filters link at Description
        // this.getDescriptionContainer().showFiltersLink();
    },
    onFilterFormExpand: function() {
        this.getFilterContainer().setHeight(this.filterHeight);

        // Hide filters link at Description
        // this.getDescriptionContainer().hideFiltersLink();
    },

    collapseFilter: function() {
        this.getFilterForm().collapse();
        this.getFilterContainer().setHeight(this.filterCollapsedHeight);
    },
    expandFilter: function() {
        this.getFilterForm().expand();
        this.getFilterContainer().setHeight(this.filterHeight);
    },

    // Filter private events

    onTrunksCountLoaded: function(count) {
        if (count > 0) {
            this.getFilterTrunksField().setDisabled(false);
        }
    },

    onPbxesCountLoaded: function(count) {
        if (count > 1) {
            this.getFilterPbxesField().setDisabled(false);
        }
    },

    onFilterReset: function() {
        var mask = new Ext.LoadMask(this.getFilterForm().getEl(), { msg:_t('Please wait...')} );
            mask.show();
        var self = this;
        var callback = function() {
            this.setFilterState(this.getDefaultFilterState());
            mask.hide();
        }

        setTimeout( function() { callback.call(self) }, 500 );
    },

    onFilterApply: function() {
        // reset personal/shared report info
        this.name = '';
        this.periodFilterName = '';
        this.getController('Viewer').onFilterChange.call(this.getController('Viewer'), this.getFilterState());
    },


    // Filter methods

    setFilterState: function(s) {
        this.getFilterDestinationField().setState(s['destination']);
        this.getFilterCostField().setState(s['cost']);
        this.getFilterBillField().setState(s['bill']);
        this.getFilterPeriodField().setState(s['period']);
        this.getFilterTypesField().setState(s['types']);
        this.getFilterTrunksField().setState(s['trunks']);
        this.getFilterTagsField().setState(s['tags']);
        this.getFilterPbxesField().setState(s['pbxes']);
    },

    getDefaultFilterState: function() {
        var s = new Object();

        s['destination'] = this.getFilterDestinationField().getDefaultState();
        s['cost'] = this.getFilterCostField().getDefaultState();
        s['bill'] = this.getFilterBillField().getDefaultState();
        s['period'] = this.getFilterPeriodField().getDefaultState();
        s['types'] = this.getFilterTypesField().getDefaultState();
        s['trunks'] = this.getFilterTrunksField().getDefaultState();
        s['tags'] = this.getFilterTagsField().getDefaultState();
        s['pbxes'] = this.getFilterPbxesField().getDefaultState();

        return s;
    },

    getFilterState: function() {
        var s = new Object();

        s['destination'] = this.getFilterDestinationField().getState();
        s['cost'] = this.getFilterCostField().getState();
        s['bill'] = this.getFilterBillField().getState();
        s['period'] = this.getFilterPeriodField().getState();
        s['types'] = this.getFilterTypesField().getState();
        s['trunks'] = this.getFilterTrunksField().getState();
        s['tags'] = this.getFilterTagsField().getState();
        s['pbxes'] = this.getFilterPbxesField().getState();
        s['miss_call_sec_period'] = User['miss_call_sec_period'];

        return s;
    },

    getFilterDescription: function() {
        var extra = new Array();
        var destination = this.getFilterDestinationField().getFilterDescription();
        var cost = this.getFilterCostField().getFilterDescription();
        var bill = this.getFilterBillField().getFilterDescription();
        var period = this.getFilterPeriodField().getFilterDescription();
        var types = this.getFilterTypesField().getFilterDescription();
        var trunks = this.getFilterTrunksField().getFilterDescription();
        var tags = this.getFilterTagsField().getFilterDescription();
        var pbxes = this.getFilterPbxesField().getFilterDescription();

        if (destination) {
            extra.push(destination);
        }
        if (cost) {
            extra.push(cost);
        }
        /**
        if (duration) {
            extra.push(duration);
        }
        **/
        if (bill) {
            extra.push(bill);
        }
        if (types) {
            extra.push(types);
        }
        if (trunks) {
            extra.push(trunks);
        }
        if (tags) {
            extra.push(tags);
        }
        if (pbxes) {
            extra.push(pbxes);
        }

        if (extra.length > 0) {
            return sprintf("%(period)s; %(extra)s.", { period: period, extra: extra.join('; ') });
        }

        return sprintf("%(period)s.", { period: period });
    },
    getGroupingDescription: function(columns) {
        return _t('Filter');
    },

    // Columns state

    /**
     * Sets a Table state (columns and sorting)
     */
    setTableState: function(state) {
        return this.getController('viewer.Table').setState(state.table);
    },

    /**
     * Gets a Table state (columns and sorting)
     */
    getTableState: function() {
        return this.getController('viewer.Table').getState();
    },

    /**
     * Sets a Chart state (...)
     */
    setChartState: function(state) {
        return;
        // return this.getController('viewer.Chart').setState(state.chart);
    },

    /**
     * Gets a Chart state
     */
    getChartState: function() {
        return {};
        // return this.getController('viewer.Chart').getState();
    },

    /**
     * Sets a current View State, viewer Table or viewer Chart.
     */
    setViewState: function(state) {
        return this.getController('Viewer').setViewState(state);
    },

    /**
     * Gets a current View State, viewer Table (table) or viewer Chart (chart).
     */
    getViewState: function() {
        return this.getController('Viewer').getViewState();
    },

    setGroupingState: function(state) {
        return this.getController('Viewer').setGroupingState(state);
    },

    getGroupingState: function() {
        return this.getController('Viewer').getGroupingState();
    },

    setMetricState: function(state) {
        return this.getController('Viewer').setMetricState(state);
    },

    getMetricState: function() {
        return this.getController('Viewer').getMetricState();
    }
});
