Ext.define('App.controller.Viewer', {
    extend: 'Ext.app.Controller',
    refs: [
        {
            ref: 'viewportContainer',
            selector: '#viewport'
        },
        {
            ref: 'viewerToolbarContainer',
            selector: '#viewerToolbar'
        },
        {
            ref: 'viewerToolbarHolder',
            selector: '#viewerToolbarHolder'
        },
        {
            ref: 'viewerColumnsContainer',
            selector: '#viewerColumnsContainer'
        },
        {
            ref: 'viewerContainer',
            selector: 'viewer'
        },
        {
            ref: 'viewerHolder',
            selector: '#viewerHolder'
        },
        {
            ref: 'viewerTableContainer',
            selector: 'viewerTable'
        },
        {
            ref: 'viewerChartContainer',
            selector: 'viewerChart'
        },
        {
            ref: 'viewerErrorContainer',
            selector: 'viewerError'
        },
        {
            ref: 'typeSwitcher',
            selector: '#viewerTypeSwitcher'
        },
        {
            ref: 'chartLabelSwitcher',
            selector: '#viewerChartLabelSwitcher'
        },
        {
            ref: 'fullPageButton',
            selector: 'viewerFullPage'
        },
        {
            ref: 'exportButton',
            selector: 'viewerExportButton'
        },
        {
            ref: 'groupingButton',
            selector: 'viewerGrouping'
        },
        {
            ref: 'metricButton',
            selector: 'viewerMetric'
        }
    ],
    init: function() {
        this.panoramState = 'NORMAL';
        this.viewState = 'TABLE';
        this.viewResultsState = 'TABLE';

        this.reportId = 0;
        this.periodId = 0;
        this.periodRange = false;
        this.tableReportId = 0;

        this.filterState = false;
        this.groupingState = this.getGroupingButton().getPicker().getState();
        this.tableMetricState = this.getMetricButton().getPicker().getState();
        this.chartMetricState = [this.getMetricButton().getPicker().getLastActiveRecordName()];

        this.getViewerErrorContainer().on('action', this.onErrorContainerAction, this);
        this.getTypeSwitcher().on('change',this.onViewChange, this);
        this.getChartLabelSwitcher().on('change',this.onChartLabelChange, this);
        this.getGroupingButton().on('change', this.onGroupingChange, this);
        this.getMetricButton().on('grpElClick', this.onGroupingLinkClick, this);
        this.getMetricButton().on('change', this.onMetricChange, this);
        this.getFullPageButton().on('click', this.onPanoramSwitch, this);
        this.getExportButton().on('click', this.onExportButtonClick, this);
        this.setExportButtonEnabled(false);
        this.on('incrementalSynchronization', this.onIncrementalSynchronization, this);
        this.callParent(arguments);
    },

    /**
     * Fires when application is lauched, we should load default results.
     */
    launch: function() {
        this.filterState = this.getController('Report').getFilterState();
        this.onVerifyFilterPeriod(this.filterState);
    },

    unSynchronizedSynchronize: function() {
        this.syncDialog = Ext.create('App.view.viewer.Sync');
        this.syncDialog.on('complete', this.onFilterSyncDialogComplete, this);
        this.syncDialog.on('terminated', this.onFilterSyncDialogTerminated, this);

        this.syncDialog.show();
        this.syncDialog.sync(this.syncPeriodDateFrom, this.syncPeriodDateTo);
    },

    onIncrementalSynchronization: function() {
        this.filterState = this.getController('Report').getFilterState();
        if (!parseInt(this.filterState['period']['type']) || parseInt(this.filterState['period']['type']) == parseInt(User['prev_period'])){
            //Update filter just in case previous user settings same as filter
            //This will no affect in case Update DB button pressed
            this.filterState['period']['type'] = parseInt(User['period']);
        }
        User['prev_period'] = undefined;
        this.getController('Report').setFilterState(this.filterState);
        this.onVerifyFilterPeriod(this.filterState);
    },

    /**
     * Fires when user clicks to a filter apply button.
     */
    onFilterChange: function(filter) {
        this.filterState = filter;
        this.onVerifyFilterPeriod(this.filterState);
    },

    /**
     * Fires when period has been verified, we should display results or show message that data is not fully synced.
     */
    onVerifiedFilterPeriod: function(complete, dayFrom, dayTo, dateFrom, dateTo) {
        // Statistics dont't fully completed, we should notify user
        if (!complete) {
            // Sets a periods properties for using its later when user clicks to sync link
            this.syncPeriodDayFrom = dayFrom;
            this.syncPeriodDayTo = dayTo;
            this.syncPeriodDateFrom = dateFrom;
            this.syncPeriodDateTo = dateTo;

            //non sync period
            this.unSynchronizedSynchronize();
            return;
        }

        this.report();
    },

    /**
     * Fires when user has been changed a grouping.
     */
    onGroupingChange: function(grouping) {
        this.getMetricButton().changeStatus(!!grouping.length);
        this.groupingState = grouping;
        this.report();
    },

    onGroupingLinkClick: function() {
        this.getGroupingButton().getPicker().show();
    },
    /**
     * Fires when user has been changed a metric.
     */
    onMetricChange: function(metric) {
        this.setMetricState(metric, true);
        this.report();
    },

    onViewChange: function(state) {
        this.viewResultsState = state;
        if (state == 'TABLE') {
            this.getMetricButton().getPicker().setMode('MULTI');
            this.setMetricState(this.getMetricState());
        } else {
            this.getMetricButton().getPicker().setMode('SINGLE');
            this.setMetricState([this.getMetricButton().getPicker().getLastActiveRecordName()]);
        }

        this.report();
    },
    onChartLabelChange: function(state , isLabelVisible) {
        this.getController('viewer.Chart').isLabelVisible = isLabelVisible;
        if (this.viewResultsState != 'TABLE'){
            this.report();
        }
    },
    report: function(silent) {
        var grouping = this.groupingState,
            metric = this.getMetricState(),
            filter = this.filterState,
            view = this.viewResultsState;

        if (view == 'TABLE' && this.viewState != view) {
            this.setTableState(view);

            if (this.tableReportId == this.reportId) {
                silent = true;
            }
        } else if (view.substr(0, 5) == 'CHART' && this.viewState != view) {
            var chartErr = this.getChartError(this.groupingState, this.chartMetricState, this.filterState);
            if(chartErr) {
                this.setErrorState(chartErr);
                return;
            }

            this.setChartState(view);
        }

        if (!silent) {
            this.reportId++;
        }

        this.getController("Report").onReport(grouping, metric, filter, view);

        if (this.viewState == 'TABLE') {
            this.tableReportId = this.reportId;
            this.getController('viewer.Table').report(grouping, metric, filter, view, this.reportId);
        } else if (this.viewState.substr(0, 5) == 'CHART') {
            this.getController('viewer.Chart').report(grouping, metric, filter, view, this.reportId);
        } else if (this.viewResultsState == 'TABLE') {
            this.tableReportId = this.reportId;
            this.setTableState();
            this.getController('viewer.Table').report(grouping, metric, filter, view, this.reportId);
        } else if (this.viewResultsState.substr(0, 5) == 'CHART') {
            this.setChartState();
            this.getController('viewer.Chart').report(grouping, metric, filter, view, this.reportId);
        }
    },
    setViewState: function(state) {
        if (this.viewState != state) {
            if (state.substr(0, 5) == 'CHART') {
                this.setChartState(state);
            } else {
                this.setTableState(state);
            }
        }

        if (this.getTypeSwitcher().getActiveItem() != state) {
            this.getTypeSwitcher().setActiveItem(state);
        }

        this.viewState = state;
    },
    getViewState: function() {
        return this.viewState;
    },
    setTableState: function(state) {
        if (state) {
            this.viewState = 'TABLE';
            this.viewResultsState = 'TABLE';
        } else if (this.viewState == this.viewResultsState) {
            return;
        }

        this.getViewerContainer().getLayout().setActiveItem(this.getViewerTableContainer());
        this.setExportButtonEnabled(false);
    },
    setChartState: function(state) {
        if (state) {
            this.viewState = state;
            this.viewResultsState = state;
        } else if (this.viewState == this.viewResultsState) {
            return;
        }

        this.getViewerContainer().getLayout().setActiveItem(this.getViewerChartContainer());
        this.setExportButtonEnabled(false);
    },
    setErrorState: function(type, config) {
        // Switch viewer layout to notification view

        this.viewState = 'error';
        this.getViewerContainer().getLayout().setActiveItem(this.getViewerErrorContainer());

        if (Ext.isFunction(this.getGroupingButton) && this.getGroupingButton().getPicker().getFirstStateName()){
            config = Ext.applyIf(config||{},{groupingStateName:this.getGroupingButton().getPicker().getFirstStateName()});
        }

        this.getViewerErrorContainer().setError(type, config);
        this.setExportButtonEnabled(false);
    },
    setExportButtonEnabled: function(enabled) {
        console.log("setExportButtonEnabled", enabled, this.viewState);
        if (enabled)
        if (this.viewState.substr(0, 5) == 'CHART'){
            this.getExportButton().setText(_t('Export chart'));
        } else {
            this.getExportButton().setText(_t('Export'));
        }
        this.getExportButton().setEnabled(enabled);
    },
    onPanoramSwitch: function() {
        if (this.panoramState == 'FULL') {
            this.onExitTableFullView();
            return;
        }

        this.panoramState = 'FULL';

        var width = Ext.getBody().getWidth();
        var height = Ext.getBody().getHeight();

        if (width < 1240) {
            width = 1240;
        }

        this.panoramToolbar = Ext.create("Ext.container.Container", {
            height: 42,
            padding: '10',
            style: 'background:#ededed;',
            layout: 'fit'
        });

        this.panoramContainer = Ext.create("Ext.container.Container", {
            cls: 'full-view',
            anchor: '100%, -42',
            layout: 'fit'
        });

        this.panoramOverlay = Ext.create("Ext.container.Container", {
            floating: true,
            width: width,
            height: height,
            renderTo: Ext.getBody(),
            style: "background:#fff;z-index: 10000;",
            layout: 'anchor',
            items: [
                this.panoramToolbar,
                this.panoramContainer
            ]
        });

        Ext.EventManager.onWindowResize(this.onPanoramWindowResize, this);

        this.panoramFullPageText = this.getFullPageButton().getText();
        this.getFullPageButton().setText('<div class="w-viewer-fullscreen-collapse-button"></div>');

        this.getViewerToolbarHolder().remove(this.getViewerToolbarContainer(), false);
        this.getViewerHolder().remove(this.getViewerContainer(), false);

        this.panoramToolbar.add(this.getViewerToolbarContainer());
        this.panoramContainer.add(this.getViewerContainer());
    },

    onPanoramWindowResize: function() {
        var width = Ext.getBody().getWidth();
        var height = Ext.getBody().getHeight();

        if (width < 1240) {
            width = 1240;
        }

        if (this.panoramOverlay.getWidth() != width) {
            this.panoramOverlay.setWidth(width);
        }
        if (this.panoramOverlay.getHeight() != height) {
            this.panoramOverlay.setHeight(height);
        }
    },

    onExitTableFullView: function() {
        Ext.EventManager.removeResizeListener(this.alignEditor, this);

        this.panoramState = 'NORMAL';
        this.panoramOverlay.hide();
        this.panoramToolbar.remove(this.getViewerToolbarContainer(), false);
        this.panoramContainer.remove(this.getViewerContainer(), false);

        this.getFullPageButton().setText(this.panoramFullPageText);
        this.getViewerToolbarHolder().add(this.getViewerToolbarContainer());
        this.getViewerHolder().add(this.getViewerContainer());
    },


    onExportButtonClick: function() {
        if (this.getExportButton().disabled)
            return;
        if (this.viewResultsState == 'TABLE') {
            this.exportDialog = Ext.create("App.view.viewer.ExportDialog");
            this.exportDialog.on("export", this.onExportDialogExport, this);
            this.exportDialog.on("open", this.onExportDialogOpen, this);
            this.exportDialog.show();
        } else if (this.viewState != 'error'){
            this.getController('viewer.Chart')['export']();
        }

        console.log("onExportButtonClick", this.viewState);
    },

    /**
     * We should generate data for xls applet generation.
     */
    onExportDialogExport: function(type) {
        this.exportType = type;
        this.exportDialog.setLoading(_t('Please wait...'));
        this.getController('viewer.Table').reportPlainWithCurrentConditions(this.onExportDialogReportGenerated, this);
    },

    onExportDialogReportGenerated: function(report) {
        var reportName = this.getController('Report').name || null;
        var reportPeriodFilterName = this.getController('Report').periodFilterName || null;
        report['name'] = reportName;
        report['periodFilterName'] = reportPeriodFilterName;
        if (this.exportType === 'csv') {
            App.storage.Storage.pushCSVGeneration(report, this.onExportDialogReportFileGenerated, this);
        } else {
            App.storage.Storage.pushXMLGeneration(report, this.onExportDialogReportFileGenerated, this);
        }
    },

    onExportDialogReportFileGenerated: function(result) {
        this.exportDialog.setLoading(false);
        this.exportDialog.close();
        return;
        this.exportDialogFilePath = result.file;
        this.exportDialog.switchToDownloadState(this.exportDialogFilePath);
    },

    onExportDialogOpen: function() {
        this.exportDialog.setLoading(_t('Please wait...'));
        App.storage.Storage.pushOpenFile(this.exportDialogFilePath, this.onExportDialogOpened, this);
    },

    onExportDialogOpened: function() {
        this.exportDialog.setLoading(false);
    },

    /**
     * Fires when user clicks to a apply button on a filter, we should verify statistics in database
     * and if it's complete we should show results.
     */
    onVerifyFilterPeriod: function(filter) {
        var self = this;
        var range = this.getFilterPeriodRange(filter);

        // Backend have events for selected period, we should verify is all statistics is complete
        var dayFrom = range['dayFrom'];
        var dayTo = range['dayTo'];
        var dateFrom = range['dateFrom'];
        var dateTo = range['dateTo'];

        var controller = function(statistics) {
            self.onVerifyFilterPeriodStatisticsFetched(statistics, dayFrom, dayTo, dateFrom, dateTo, self.onVerifiedFilterPeriod, self);
        }

        App.proxy.Synchronization.readStatisticsProper(dayFrom, dayTo, controller);
    },

    /**
     * Fires when statistics for filter period has been fetched from database, we should define if it's is complete.
     */
    onVerifyFilterPeriodStatisticsFetched: function(statistics, dayFrom, dayTo, dateFrom, dateTo, callback, scope) {
        scope = scope || this;

        // Verify if statistics is complete at all
        var i, day, a;
        var complete = true;

        for (day=dayFrom; day <= dayTo; day++) {
            a = false;

            for (i=0; i < statistics.length; i++) {
                if (day == parseInt(statistics[i]['DAY'])) {
                    if (statistics[i]['COMPLETE'] == '1') {
                        a = true;
                    }
                    break;
                }
            }

            if (!a) {
                complete = false;
                break;
            }
        }

        callback.call(scope, complete, dayFrom, dayTo, dateFrom, dateTo);
    },

    /**
     * Fires when user clicks on link in error widget.
     * @param error String
     * @param action String
     * @param param Object
     */
    onErrorContainerAction: function(error, action, param) {
        switch (action){
            case 'groupingclick':
                this.setGroupingState(param);
                this.onGroupingChange(param);
                break;
            case 'metricclick':
                this.setMetricState(param);
                this.onMetricChange(param);
                break;
        }
    },

    /**
     * Fires when synchronization is comleted, we should show results and redraw an informer.
     */
    onFilterSyncDialogComplete: function() {
        this.getController('Navigation').fireEvent('redrawInformer');
        this.report();
    },

    /**
     * Fires when user has terminated a synchronization, we should to redraw an informer.
     */
    onFilterSyncDialogTerminated: function() {
        this.getController('Navigation').fireEvent('redrawInformer');
    },

    getChartError: function(grouping, metric, filter) {
        if (grouping == undefined || grouping.length === 0) {
            //this.setErrorState('NO_GROUPING');
            return 'NO_GROUPING';
        }
        if (grouping.length > 1 || grouping[0] === 'USERS' || grouping[0] === 'GROUPS') {
            //this.setErrorState('NOT_SUPPORTED_GROUPING');
            return 'NOT_SUPPORTED_GROUPING';
        }
        return false;
    },

    setGroupingState: function(state) {
        this.groupingState = state;

        this.getMetricButton().changeStatus(!!this.groupingState.length);
        this.getGroupingButton().setState(state);
    },

    getGroupingState: function() {
        return this.groupingState;
    },

    setMetricState: function(state, silent) {
        if(this.viewResultsState == 'TABLE') {
            this.tableMetricState = state;
        } else {
            this.chartMetricState = state;
        }
        if(!silent) {
            this.getMetricButton().getPicker().setState(state);
        }
    },

    getMetricState: function() {
        if(this.viewResultsState == 'TABLE') {
            return this.tableMetricState;
        } else {
            return this.chartMetricState;
        }
    },

    getFilterPeriodRange: function(filter) {
        var dateFrom = false;
        var dateTo = false;
        var dateUtility = App.utility.Date;

        switch (filter['period']['type']) {
            case App.utility.Date.TODAY:
                //today
                dateFrom = dateUtility.getToday();
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.YESTERDAY:
                //yesterday
                dateFrom = dateUtility.getYesterday();
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.WEEK:
                //This week
                dateFrom = dateUtility.getThisWeek();
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.LAST_2_DAYS:
                //Last 2 days
                dateFrom = dateUtility.getLastNDays(2);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.LAST_7_DAYS:
                //Last 7 days
                dateFrom = dateUtility.getLastNDays(7);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.PREV_WEEK:
                //Prev week
                dateFrom = dateUtility.getPrevWeek();
                dateTo = dateUtility.getThisWeek();
                break;
            case App.utility.Date.LAST_2_WEEKS:
                //Last 2 weeks
                dateFrom = dateUtility.getToday();
                dateTo = dateUtility.getPrevWeek();
                break;
            case App.utility.Date.MONTH:
                //This month
                dateFrom = dateUtility.getThisMonth();
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.LAST_30_DAYS:
                //Last 30 days
                dateFrom = dateUtility.getLastNDays(30);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.PREV_MONTH:
                //Prev month
                dateFrom = dateUtility.getPrevNMonth(1);
                dateTo = dateUtility.getThisMonth();
                break;
            case App.utility.Date.LAST_60_DAYS:
                //Last 60 days
                dateFrom = dateUtility.getLastNDays(60);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.LAST_3_MONTH:
                //Last 3 months
                dateFrom = dateUtility.getPrevNMonth(3);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.LAST_6_MONTH:
                //Last 3 months
                dateFrom = dateUtility.getPrevNMonth(6);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.LAST_12_MONTH:
                //Last 12 months
                dateFrom = dateUtility.getPrevNMonth(12);
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.YEAR:
                //This year
                dateFrom = dateUtility.getThisYear();
                dateTo = dateUtility.getToday();
                break;
            case App.utility.Date.PREV_YEAR:
                //Prev year
                dateFrom = dateUtility.getPrevNYears(1);
                dateTo = dateUtility.getThisYear();
                break;
            case App.utility.Date.CUSTOM:
                //Custom
                dateFrom = filter['period']['fromDate'] + ' ' + filter['period']['fromTime']
                dateTo = filter['period']['toDate'] + ' ' + filter['period']['toTime']
                break;
        }

        var dayFromDate = Ext.Date.parse(dateFrom.substr(0, 10), "Y-m-d");
        var dayToDate = Ext.Date.parse(dateTo.substr(0, 10), "Y-m-d");
        var dayFromTime = parseInt(Ext.Date.format(dayFromDate, "U")) + parseInt(Ext.Date.format(dayFromDate, "Z"));
        var dayToTime = parseInt(Ext.Date.format(dayToDate, "U")) + parseInt(Ext.Date.format(dayToDate, "Z"));
        var dayFrom = dayFromTime / 86400;
        var dayTo = dayToTime / 86400;
        var range = {
            dateFrom: dateFrom,
            dateTo: dateTo,
            dayFromDate: dayFromDate,
            dayToDate: dayToDate,
            dayFrom: dayFrom,
            dayTo: dayTo
        }

        return range;
    },

    getBuilderColumnsByIdenties: function(identies) {
        var columns = new Array();
        var i, c, b;

        for (i=0; i < identies.length; i++) {
            b = this.getBuilderColumnsByIdenty(identies[i]);

            for (c=0; c < b.length; c++) {
                columns.push(b[c]);
            }
        }

        return columns;
    },

    getBuilderColumnsByIdenty: function(identy) {
        var columns = new Array();

        switch (identy) {
            // Simple Columns
            case 'DATE':
                columns.push({name: 'COLUMN_DATE', as: 'DATE'});
                break;
            case 'DURATION':
                columns.push({name: 'COLUMN_DURATION', as: 'DURATION'});
                break;
            case 'FROM':
                columns.push({name: 'COLUMN_FROM', as: 'FROM'}, {name: 'COLUMN_FROM_USER', as: 'FROM_USER'}, {name: 'COLUMN_FROM_NAME', as: 'FROM_NAME'});
                break;
            case 'FROM_GROUP':
                columns.push({name: 'COLUMN_FROM_GROUP', as: 'FROM_GROUP'});
                break;
            case 'TO':
                columns.push({name: 'COLUMN_TO', as: 'TO'}, {name: 'COLUMN_TO_USER', as: 'TO_USER'}, {name: 'COLUMN_TO_NAME', as: 'TO_NAME'});
                break;
            case 'TO_GROUP':
                columns.push({name: 'COLUMN_TO_GROUP', as: 'TO_GROUP'});
                break;
            case 'AREA':
                columns.push({name: 'COLUMN_COUNTRY', as: 'COUNTRY'}, {name: 'COLUMN_COUNTRY_AREA', as: 'COUNTRY_AREA'});
                break;
            case 'CLASS':
                columns.push({name: 'COLUMN_CLASS', as: 'CLASS'});
                break;
            case 'COST':
                columns.push({name: 'COLUMN_COST', as: 'COST'});
                break;
            case 'DIRECTION':
                columns.push({name: 'COLUMN_DIRECTION', as: 'DIRECTION'});
                break;
            case 'STATUS':
                // TODO When support for checks
                // columns.push({name: 'COLUMN_BILL', as: 'BILL'}, {name: 'COLUMN_DURATION', as: 'DURATION'});
                break;
            case 'BILL':
                columns.push({name: 'COLUMN_BILL', as: 'BILL'});
                break;
            case 'RING':
                columns.push({name: 'COLUMN_RING', as: 'RING'});
                break;
            case 'TRUNK':
                columns.push({name: 'COLUMN_TRUNK', as: 'TRUNK'});
                break;
            case 'PBX':
                columns.push({name: 'COLUMN_PBX', as: 'PBX'});
                break;
            case 'TAG':
                columns.push({name: 'COLUMN_TAG', as: 'TAG'});
                break;
            case 'SERVICE':
                columns.push({name: 'COLUMN_SERVICE', as: 'SERVICE'});
                break;
            case 'USERFIELD':
                columns.push({name: 'COLUMN_USERFIELD', as: 'USERFIELD'});
                break;
            case 'FAXDATA':
                columns.push({name: 'COLUMN_FAXDATA', as: 'FAXDATA'});
                break;

            // Grouped Columns
            case 'TOTAL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS', as: 'TOTAL_EVENTS'});
                break;
            case 'TOTAL_CALL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS_CALL', as: 'TOTAL_CALL_EVENTS'});
                break;
            case 'TOTAL_FAX_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS_FAX', as: 'TOTAL_FAX_EVENTS'});
                break;
            case 'TOTAL_VOICEMAIL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS_VOICEMAIL', as: 'TOTAL_VOICEMAIL_EVENTS'});
                break;
            case 'TOTAL_LOCAL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_LOCAL_EVENTS', as: 'TOTAL_LOCAL_EVENTS'});
                break;
            case 'TOTAL_INCOMING_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_INCOMING_EVENTS', as: 'TOTAL_INCOMING_EVENTS'});
                break;
            case 'TOTAL_OUTGOING_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_OUTGOING_EVENTS', as: 'TOTAL_OUTGOING_EVENTS'});
                break;
            case 'TOTAL_ANSWERED_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_ANSWERED_EVENTS', as: 'TOTAL_ANSWERED_EVENTS'});
                break;
            case 'TOTAL_ANSWERED_WITHIN_XX_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_ANSWERED_WITHIN_XX_EVENTS', as: 'TOTAL_ANSWERED_WITHIN_XX_EVENTS'});
                break;
            case 'TOTAL_ANSWERED_OVER_XX_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_ANSWERED_OVER_XX_EVENTS', as: 'TOTAL_ANSWERED_OVER_XX_EVENTS'});
                break;
            case 'TOTAL_NOT_ANSWERED_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_NOT_ANSWERED_EVENTS', as: 'TOTAL_NOT_ANSWERED_EVENTS'});
                break;
            case 'TOTAL_DURATION':
                columns.push({name: 'COLUMN_TOTAL_DURATION', as: 'TOTAL_DURATION'});
                break;
            case 'AVG_DURATION':
                columns.push({name: 'COLUMN_AVG_DURATION', as: 'AVG_DURATION'});
                break;
            case 'TOTAL_LOCAL_DURATION':
                columns.push({name: 'COLUMN_TOTAL_LOCAL_DURATION', as: 'TOTAL_LOCAL_DURATION'});
                break;
            case 'AVG_LOCAL_DURATION':
                columns.push({name: 'COLUMN_AVG_LOCAL_DURATION', as: 'AVG_LOCAL_DURATION'});
                break;
            case 'TOTAL_INCOMING_DURATION':
                columns.push({name: 'COLUMN_TOTAL_INCOMING_DURATION', as: 'TOTAL_INCOMING_DURATION'});
                break;
            case 'AVG_INCOMING_DURATION':
                columns.push({name: 'COLUMN_AVG_INCOMING_DURATION', as: 'AVG_INCOMING_DURATION'});
                break;
            case 'TOTAL_OUTGOING_DURATION':
                columns.push({name: 'COLUMN_TOTAL_OUTGOING_DURATION', as: 'TOTAL_OUTGOING_DURATION'});
                break;
            case 'TOTAL_BILL':
                columns.push({name: 'COLUMN_TOTAL_BILL', as: 'TOTAL_BILL'});
                break;
            case 'AVG_BILL':
                columns.push({name: 'COLUMN_AVG_BILL', as: 'AVG_BILL'});
                break;
            case 'TOTAL_LOCAL_BILL':
                columns.push({name: 'COLUMN_TOTAL_LOCAL_BILL', as: 'TOTAL_LOCAL_BILL'});
                break;
            case 'AVG_LOCAL_BILL':
                columns.push({name: 'COLUMN_AVG_LOCAL_BILL', as: 'AVG_LOCAL_BILL'});
                break;
            case 'TOTAL_INCOMING_BILL':
                columns.push({name: 'COLUMN_TOTAL_INCOMING_BILL', as: 'TOTAL_INCOMING_BILL'});
                break;
            case 'AVG_INCOMING_BILL':
                columns.push({name: 'COLUMN_AVG_INCOMING_BILL', as: 'AVG_INCOMING_BILL'});
                break;
            case 'TOTAL_OUTGOING_BILL':
                columns.push({name: 'COLUMN_TOTAL_OUTGOING_BILL', as: 'TOTAL_OUTGOING_BILL'});
                break;
            case 'AVG_OUTGOING_BILL':
                columns.push({name: 'COLUMN_AVG_OUTGOING_BILL', as: 'AVG_OUTGOING_BILL'});
                break;
            case 'TOTAL_RING':
                columns.push({name: 'COLUMN_TOTAL_RING', as: 'TOTAL_RING'});
                break;
            case 'AVG_RING':
                columns.push({name: 'COLUMN_AVG_RING', as: 'AVG_RING'});
                break;
            case 'TOTAL_COST':
                columns.push({name: 'COLUMN_TOTAL_COST', as: 'TOTAL_COST'});
                break;
            case 'TOTAL_CALL_COST':
                columns.push({name: 'COLUMN_TOTAL_COST_CALL', as: 'TOTAL_CALL_COST'});
                break;
            case 'TOTAL_FAX_COST':
                columns.push({name: 'COLUMN_TOTAL_COST_FAX', as: 'TOTAL_FAX_COST'});
                break;
            case 'AVG_COST':
                columns.push({name: 'COLUMN_AVG_COST', as: 'AVG_COST'});
                break;
            case 'AVG_CALL_COST':
                columns.push({name: 'COLUMN_AVG_COST_CALL', as: 'AVG_CALL_COST'});
                break;
            case 'AVG_FAX_COST':
                columns.push({name: 'COLUMN_AVG_COST_FAX', as: 'AVG_FAX_COST'});
                break;
        }

        return columns;
    },

    getBuilderGroupingsByIdenties: function(identies) {
        var grouping = new Array();
        var i, c, b;

        for (i=0; i < identies.length; i++) {
            b = this.getBuilderGroupingsByIdenty(identies[i]);

            for (c=0; c < b.length; c++) {
                grouping.push(b[c]);
            }
        }

        return grouping;
    },

    getBuilderGroupingsByIdenty: function(identy) {
        var grouping = new Array();

        switch (identy) {
            case 'GROUP_HOUR':
                grouping.push({name: 'GROUP_BY_HOUR', as: 'HOUR'});
                break;
            case 'GROUP_HOUR_OF_DAY':
                grouping.push({name: 'GROUP_BY_HOUR_OF_DAY', as: 'HOUR_OF_DAY'});
                break;
            case 'GROUP_DAY':
                grouping.push({name: 'GROUP_BY_DAY', as: 'DAY'});
                break;
            case 'GROUP_DAY_OF_WEEK':
                grouping.push({name: 'GROUP_BY_DAY_OF_WEEK', as: 'DAY_OF_WEEK'});
                break;
            case 'GROUP_WEEK':
                grouping.push({name: 'GROUP_BY_WEEK', as: 'WEEK'});
                break;
            case 'GROUP_MONTH':
                grouping.push({name: 'GROUP_BY_MONTH', as: 'MONTH'});
                break;
            case 'GROUP_MONTH_OF_YEAR':
                grouping.push({name: 'GROUP_BY_MONTH_OF_YEAR', as: 'MONTH_OF_YEAR'});
                break;
            case 'GROUP_YEAR':
                grouping.push({name: 'GROUP_BY_YEAR', as: 'YEAR'});
                break;
            case 'GROUP_USER':
                grouping.push({name: 'GROUP_BY_USER', as: 'USER'});
                break;
            case 'GROUP_GROUP':
                grouping.push({name: 'GROUP_BY_GROUP', as: 'USER_GROUP'});
                break;
            case 'GROUP_CLIENT':
                grouping.push({name: 'GROUP_BY_CLIENT', as: 'CLIENT'});
                break;
            case 'GROUP_COMPANY':
                grouping.push({name: 'GROUP_BY_COMPANY', as: 'COMPANY'});
                break;
            case 'GROUP_NUMBER':
                grouping.push({name: 'GROUP_BY_NUMBER', as: 'NUMBER'});
                break;
            case 'GROUP_SERVICE':
                grouping.push({name: 'GROUP_BY_SERVICE', as: 'SERVICE'});
                break;
            case 'GROUP_DIRECTION':
                grouping.push({name: 'GROUP_BY_DIRECTION', as: 'DIRECTION'});
                break;
            case 'GROUP_TRUNK':
                grouping.push({name: 'GROUP_BY_TRUNK', as: 'TRUNK'});
                break;
            case 'GROUP_TAG':
                grouping.push({name: 'GROUP_BY_TAG', as: 'TAG'});
                break;
            case 'GROUP_PBX':
                grouping.push({name: 'GROUP_BY_PBX', as: 'PBX'});
                break;
            case 'GROUP_COUNTRY':
                grouping.push({name: 'GROUP_BY_COUNTRY', as: 'COUNTRY'});
                break;
            case 'GROUP_CLASS':
                grouping.push({name: 'GROUP_BY_CLASS', as: 'CLASS'});
                break;
            // Hack from Communication of users (don't care about group_user_comm_from identy)
            case 'GROUP_USER_COMM_FROM':
                grouping.push({name: 'GROUP_BY_COMMUNICATION_OF_USERS', as: { from: 'USER_FROM', to: 'USER_TO'}});
                break;
            // Hack from Communication of groups (don't care about group_group_comm_from identy)
            case 'GROUP_GROUP_COMM_FROM':
                grouping.push({name: 'GROUP_BY_COMMUNICATION_OF_GROUPS', as: { from: 'GROUP_FROM', to: 'GROUP_TO'}});
                break;
        }

        return grouping;
    },

    getBuilderDefaultSorters: function() {
        return [{field: 'DATE', dest: 'DESC'}];
    },

    getBuilderGroupingsSortersByIdenties: function(identies) {
        var sorters = new Array();
        var i, c, b;

        for (i=0; i < identies.length; i++) {
            b = this.getBuilderGroupingsSortersByIdenty(identies[i]);

            for (c=0; c < b.length; c++) {
                sorters.push(b[c]);
            }
        }

        return sorters;
    },

    getBuilderGroupingsSortersByIdenty: function(identy) {
        var sorters = new Array();

        switch (identy) {
            case 'GROUP_HOUR':
                sorters.push({field: 'HOUR', dest: 'ASC'});
                break;
            case 'GROUP_HOUR_OF_DAY':
                sorters.push({field: 'HOUR_OF_DAY', dest: 'ASC'});
                break;
            case 'GROUP_DAY':
                sorters.push({field: 'DAY', dest: 'ASC'});
                break;
            case 'GROUP_DAY_OF_WEEK':
                sorters.push({field: 'DAY_OF_WEEK', dest: 'ASC'});
                break;
            case 'GROUP_WEEK':
                sorters.push({field: 'WEEK', dest: 'ASC'});
                break;
            case 'GROUP_MONTH':
                sorters.push({field: 'MONTH', dest: 'ASC'});
                break;
            case 'GROUP_MONTH_OF_YEAR':
                sorters.push({field: 'MONTH_OF_YEAR', dest: 'ASC'});
                break;
            case 'GROUP_YEAR':
                sorters.push({field: 'YEAR', dest: 'ASC'});
                break;
            case 'GROUP_USER':
                sorters.push({field: 'USER_NAME', dest: 'ASC'});
                break;
            case 'GROUP_GROUP':
                sorters.push({field: 'USER_GROUP_NAME', dest: 'ASC'});
                break;
            case 'GROUP_CLIENT':
                sorters.push({field: 'CLIENT_NAME', dest: 'ASC'});
                break;
            case 'GROUP_COMPANY':
                sorters.push({field: 'COMPANY_NAME', dest: 'ASC'});
                break;
            case 'GROUP_NUMBER':
                sorters.push({field: 'NUMBER', dest: 'ASC'});
                break;
            case 'GROUP_SERVICE':
                sorters.push({field: 'SERVICE', dest: 'ASC'});
                break;
            case 'GROUP_DIRECTION':
                sorters.push({field: 'DIRECTION', dest: 'ASC'});
                break;
            case 'GROUP_TRUNK':
                sorters.push({field: 'TRUNK', dest: 'ASC'});
                break;
            case 'GROUP_TAG':
                sorters.push({field: 'TAG', dest: 'ASC'});
                break;
            case 'GROUP_PBX':
                sorters.push({field: 'PBX', dest: 'ASC'});
                break;
            case 'GROUP_COUNTRY':
                sorters.push({field: 'COUNTRY', dest: 'ASC'});
                break;
            case 'GROUP_CLASS':
                sorters.push({field: 'CLASS', dest: 'ASC'});
                break;
            case 'GROUP_USER_COMM_FROM':
                sorters.push({field: 'USER_FROM_NAME', dest: 'ASC'});
                break;
            case 'GROUP_USER_COMM_TO':
                sorters.push({field: 'USER_TO_NAME', dest: 'ASC'});
                break;
            case 'GROUP_GROUP_COMM_FROM':
                sorters.push({field: 'GROUP_FROM_NAME', dest: 'ASC'});
                break;
            case 'GROUP_GROUP_COMM_TO':
                sorters.push({field: 'GROUP_TO_NAME', dest: 'ASC'});
                break;
        }

        return sorters;
    },


    getBuilderSummariesByIdenties: function(identies) {
        var columns = new Array();
        var i, c, b;

        for (i=0; i < identies.length; i++) {
            b = this.getBuilderSummariesByIdenty(identies[i]);

            for (c=0; c < b.length; c++) {
                columns.push(b[c]);
            }
        }

        return columns;
    },

    getBuilderSummariesByIdenty: function(identy) {
        var columns = new Array();

        switch (identy) {
            // Simple columns
            case 'DURATION':
                columns.push({name: 'COLUMN_TOTAL_DURATION', as: 'TOTAL_DURATION'});
                break;
            case 'COST':
                columns.push({name: 'COLUMN_TOTAL_COST', as: 'TOTAL_COST'});
                break;
            case 'BILL':
                columns.push({name: 'COLUMN_TOTAL_BILL', as: 'TOTAL_BILL'});
                break;
            case 'RING':
                columns.push({name: 'COLUMN_TOTAL_RING', as: 'TOTAL_RING'});
                break;

            // Grouped columns
            case 'TOTAL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS', as: 'TOTAL_EVENTS'});
                break;
            case 'TOTAL_CALL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS_CALL', as: 'TOTAL_CALL_EVENTS'});
                break;
            case 'TOTAL_FAX_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS_FAX', as: 'TOTAL_FAX_EVENTS'});
                break;
            case 'TOTAL_VOICEMAIL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_EVENTS_VOICEMAIL', as: 'TOTAL_VOICEMAIL_EVENTS'});
                break;
            case 'TOTAL_LOCAL_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_LOCAL_EVENTS', as: 'TOTAL_LOCAL_EVENTS'});
                break;
            case 'TOTAL_INCOMING_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_INCOMING_EVENTS', as: 'TOTAL_INCOMING_EVENTS'});
                break;
            case 'TOTAL_OUTGOING_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_OUTGOING_EVENTS', as: 'TOTAL_OUTGOING_EVENTS'});
                break;
            case 'TOTAL_ANSWERED_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_ANSWERED_EVENTS', as: 'TOTAL_ANSWERED_EVENTS'});
                break;
            case 'TOTAL_ANSWERED_WITHIN_XX_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_ANSWERED_WITHIN_XX_EVENTS', as: 'TOTAL_ANSWERED_WITHIN_XX_EVENTS'});
                break;
            case 'TOTAL_ANSWERED_OVER_XX_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_ANSWERED_OVER_XX_EVENTS', as: 'TOTAL_ANSWERED_OVER_XX_EVENTS'});
                break;
            case 'TOTAL_NOT_ANSWERED_EVENTS':
                columns.push({name: 'COLUMN_TOTAL_NOT_ANSWERED_EVENTS', as: 'TOTAL_NOT_ANSWERED_EVENTS'});
                break;
            case 'TOTAL_DURATION':
                columns.push({name: 'COLUMN_TOTAL_DURATION', as: 'TOTAL_DURATION'});
                break;
            case 'AVG_DURATION':
                columns.push({name: 'COLUMN_AVG_DURATION', as: 'AVG_DURATION'});
                break;
            case 'TOTAL_LOCAL_DURATION':
                columns.push({name: 'COLUMN_TOTAL_LOCAL_DURATION', as: 'TOTAL_LOCAL_DURATION'});
                break;
            case 'AVG_LOCAL_DURATION':
                columns.push({name: 'COLUMN_AVG_LOCAL_DURATION', as: 'AVG_LOCAL_DURATION'});
                break;
            case 'TOTAL_INCOMING_DURATION':
                columns.push({name: 'COLUMN_TOTAL_INCOMING_DURATION', as: 'TOTAL_INCOMING_DURATION'});
                break;
            case 'AVG_INCOMING_DURATION':
                columns.push({name: 'COLUMN_AVG_INCOMING_DURATION', as: 'AVG_INCOMING_DURATION'});
                break;
            case 'TOTAL_OUTGOING_DURATION':
                columns.push({name: 'COLUMN_TOTAL_OUTGOING_DURATION', as: 'TOTAL_OUTGOING_DURATION'});
                break;
            case 'TOTAL_BILL':
                columns.push({name: 'COLUMN_TOTAL_BILL', as: 'TOTAL_BILL'});
                break;
            case 'AVG_BILL':
                columns.push({name: 'COLUMN_AVG_BILL', as: 'AVG_BILL'});
                break;
            case 'TOTAL_LOCAL_BILL':
                columns.push({name: 'COLUMN_TOTAL_LOCAL_BILL', as: 'TOTAL_LOCAL_BILL'});
                break;
            case 'AVG_LOCAL_BILL':
                columns.push({name: 'COLUMN_AVG_LOCAL_BILL', as: 'AVG_LOCAL_BILL'});
                break;
            case 'TOTAL_INCOMING_BILL':
                columns.push({name: 'COLUMN_TOTAL_INCOMING_BILL', as: 'TOTAL_INCOMING_BILL'});
                break;
            case 'AVG_INCOMING_BILL':
                columns.push({name: 'COLUMN_AVG_INCOMING_BILL', as: 'AVG_INCOMING_BILL'});
                break;
            case 'TOTAL_OUTGOING_BILL':
                columns.push({name: 'COLUMN_TOTAL_OUTGOING_BILL', as: 'TOTAL_OUTGOING_BILL'});
                break;
            case 'AVG_OUTGOING_BILL':
                columns.push({name: 'COLUMN_AVG_OUTGOING_BILL', as: 'AVG_OUTGOING_BILL'});
                break;
            case 'TOTAL_RING':
                columns.push({name: 'COLUMN_TOTAL_RING', as: 'TOTAL_RING'});
                break;
            case 'AVG_RING':
                columns.push({name: 'COLUMN_AVG_RING', as: 'AVG_RING'});
                break;
            case 'TOTAL_COST':
                columns.push({name: 'COLUMN_TOTAL_COST', as: 'TOTAL_COST'});
                break;
            case 'TOTAL_CALL_COST':
                columns.push({name: 'COLUMN_TOTAL_COST_CALL', as: 'TOTAL_CALL_COST'});
                break;
            case 'TOTAL_FAX_COST':
                columns.push({name: 'COLUMN_TOTAL_COST_FAX', as: 'TOTAL_FAX_COST'});
                break;
            case 'AVG_COST':
                columns.push({name: 'COLUMN_AVG_COST', as: 'AVG_COST'});
                break;
            case 'AVG_CALL_COST':
                columns.push({name: 'COLUMN_AVG_COST_CALL', as: 'AVG_CALL_COST'});
                break;
            case 'AVG_FAX_COST':
                columns.push({name: 'COLUMN_AVG_COST_FAX', as: 'AVG_FAX_COST'});
                break;
        }

        return columns;
    },


    getGroupedMetricColumns: function(grouping, metric) {
        var columns = new Array();

        for (var i=0; i < grouping.length; i++) {
            switch(grouping[i]) {
                case 'HOUR':
                case 'HOUR_OF_DAY':
                case 'DAY':
                case 'DAY_OF_WEEK':
                case 'WEEK':
                case 'MONTH':
                case 'MONTH_OF_YEAR':
                case 'YEAR':
                case 'USER':
                case 'GROUP':
                case 'CLIENT':
                case 'COMPANY':
                case 'NUMBER':
                case 'TRUNK':
                case 'PBX':
                case 'TAG':
                case 'COUNTRY':
                case 'CLASS':
                    columns.push("GROUP_" + grouping[i]);
                    break;
                case 'USERS':
                    columns.push('GROUP_USER_COMM_FROM', 'GROUP_USER_COMM_TO');
                    break;
                case 'GROUPS':
                    columns.push('GROUP_GROUP_COMM_FROM', 'GROUP_GROUP_COMM_TO');
                    break;
            }
        }

        for (var i=0; i < metric.length; i++) {
            switch(metric[i]) {
                case 'COUNT':
                    columns.push('TOTAL_EVENTS');
                    break;
                case 'COUNT_BY_SERVICE':
                    columns.push('TOTAL_EVENTS', 'TOTAL_CALL_EVENTS', 'TOTAL_FAX_EVENTS', 'TOTAL_VOICEMAIL_EVENTS');
                    break;
                case 'COUNT_BY_STATUS':
                    columns.push('TOTAL_EVENTS', 'TOTAL_ANSWERED_EVENTS', 'TOTAL_NOT_ANSWERED_EVENTS');
                    break;
                case 'COUNT_BY_DIRECTION':
                    columns.push('TOTAL_EVENTS', 'TOTAL_LOCAL_EVENTS', 'TOTAL_INCOMING_EVENTS', 'TOTAL_OUTGOING_EVENTS');
                    break;
                case 'COUNT_BY_SLA':
                    columns.push('TOTAL_EVENTS', 'TOTAL_ANSWERED_WITHIN_XX_EVENTS', 'TOTAL_ANSWERED_OVER_XX_EVENTS', 'TOTAL_NOT_ANSWERED_EVENTS');
                    break;
                case 'COST':
                    columns.push('TOTAL_COST');
                    break;
                case 'COST_BY_SERVICE':
                    columns.push('TOTAL_COST', 'TOTAL_CALL_COST', 'TOTAL_FAX_COST');
                    break;
                case 'COST_AVG_BY_SERVICE':
                    columns.push('AVG_COST', 'AVG_CALL_COST', 'AVG_FAX_COST');
                    break;
                case 'BILL':
                    columns.push('TOTAL_BILL', 'TOTAL_RING');
                    break;
                case 'BILL_BY_DIRECTION':
                    columns.push('TOTAL_BILL', 'TOTAL_RING', 'TOTAL_LOCAL_BILL', 'TOTAL_INCOMING_BILL', 'TOTAL_OUTGOING_BILL');
                    break;
                case 'BILL_AVG_BY_DIRECTION':
                    columns.push('AVG_BILL', 'AVG_RING', 'AVG_LOCAL_BILL', 'AVG_INCOMING_BILL', 'AVG_OUTGOING_BILL');
                    break;
            }
        }

        return this.getOrderedColumns(columns);
    },

    getOrderedColumns: function(columns) {
        var orders = [
            'GROUP_HOUR',
            'GROUP_HOUR_OF_DAY',
            'GROUP_DAY',
            'GROUP_DAY_OF_WEEK',
            'GROUP_WEEK',
            'GROUP_MONTH',
            'GROUP_MONTH_OF_YEAR',
            'GROUP_YEAR',
            'GROUP_USER',
            'GROUP_GROUP',
            'GROUP_CLIENT',
            'GROUP_COMPANY',
            'GROUP_NUMBER',
            'GROUP_TRUNK',
            'GROUP_PBX',
            'GROUP_TAG',
            'GROUP_COUNTRY',
            'GROUP_CLASS',
            'GROUP_USER_COMM_FROM', 'GROUP_USER_COMM_TO',
            'GROUP_GROUP_COMM_FROM', 'GROUP_GROUP_COMM_TO',
            'TOTAL_EVENTS',
            'TOTAL_CALL_EVENTS', 'TOTAL_FAX_EVENTS', 'TOTAL_VOICEMAIL_EVENTS',
            'TOTAL_ANSWERED_EVENTS', 'TOTAL_ANSWERED_WITHIN_XX_EVENTS', 'TOTAL_ANSWERED_OVER_XX_EVENTS', 'TOTAL_NOT_ANSWERED_EVENTS',
            'TOTAL_LOCAL_EVENTS', 'TOTAL_INCOMING_EVENTS', 'TOTAL_OUTGOING_EVENTS',
            'TOTAL_COST', 'AVG_COST',
            'TOTAL_CALL_COST', 'AVG_CALL_COST',
            'TOTAL_FAX_COST', 'AVG_FAX_COST',
            'TOTAL_BILL', 'AVG_BILL',
            'TOTAL_LOCAL_BILL', 'AVG_LOCAL_BILL',
            'TOTAL_INCOMING_BILL', 'AVG_INCOMING_BILL',
            'TOTAL_OUTGOING_BILL', 'AVG_OUTGOING_BILL',
            'TOTAL_RING', 'AVG_RING'
        ];
        var sorter = function(a, b) {
            var ai = Ext.Array.indexOf(orders, a);
            var ab = Ext.Array.indexOf(orders, b);

            return ai - ab;
        }

        return columns.sort(sorter);
    },

    getSimpleMetricColumns: function() {
        return [
            'DATE',
            'FROM',
            'FROM_GROUP',
            'TO',
            'TO_GROUP',
            'AREA',
            'CLASS',
            'COST',
            'DIRECTION',
            'STATUS',
            'BILL',
            'RING',
            'PBX',
            'TRUNK',
            'TAG',
            'SERVICE',
            'USERFIELD',
            'FAXDATA'
        ];
    }
} );
