/**
 * Charts controller.
 * @method report : executable method for generating charts
 *
 * User: Artyom Trityak
 * Date: 1/4/12
 * Time: 1:39 PM
 */
Ext.define('App.controller.viewer.Chart', {
    mixins: {
        observable: 'Ext.util.Observable'
    },
    requires: [
        'App.controller.viewer.chart.Transformer',
        'App.view.viewer.Chart'
    ],
    refs: [
        {
            ref: 'viewerChartMetricSwitcher',
            selector: '#viewerChartMetricSwitcher'
        },
        {
            ref: 'viewerChartContainerChart',
            selector: '#viewerChartContainerChart'
        }

    ],

    options: {
        mask: false
    },
    isLabelVisible: false,

    extend: 'Ext.app.Controller',

    init: function() {
        this.options.mask = new Ext.LoadMask(this.getViewerChartContainerChart().getEl(), {msg: _t('Please wait...')});
        this.getController("viewer.chart.Transformer").on('chartDataTransformed', this._createView, this);
        this.getController("viewer.chart.Transformer").on('error', this._createErrorView, this);

        this.callParent(arguments);
    },

    report: function(groupingState, metricState, filterState, activeChart, reportId, activeMetric) {
        activeMetric = activeMetric === undefined ? 0 : activeMetric;
        console.log(arguments);

        this.options.mask.show();
        var options = {
            reportId: reportId,
            filters: filterState,
            grouping: groupingState,
            metric: metricState,
            activeChart: activeChart,
            activeMetric: activeMetric
        };

        this.getController("viewer.chart.Transformer").transform(options);
    },

    _createView: function(options) {
        options = options || {chartType: 'CHART_COLUMN', data: []};
        options.isLabelVisible = this.isLabelVisible;
        console.log("VIEW:");
        console.log(options);
        this.getViewerChartContainerChart().removeAll();
        var chartInstance = App.view.viewer.chart.ChartView.create(options);
        if (chartInstance.canDraw() === true) {
            this.getViewerChartContainerChart().add(chartInstance);
            this.getController('Viewer').setExportButtonEnabled(true);
        } else {
            this._createErrorView(chartInstance.getReason());
            delete chartInstance;
        }
        this.options.mask.hide();
    },

    _createErrorView: function(type) {
        this.getController("Viewer").setErrorState(type);
        this.options.mask.hide();
    },
    'export': function(){
        var chartCmp = this.getViewerChartContainerChart().down('chart');
        if (chartCmp && chartCmp.surface){
            App.utility.ImageExporter.self.generate({
                type: 'image/png',
                onerror: function(config){
                    Ext.create('App.view.bootstrap.ErrorConnection',{
                      initComponent: function() {
                            this.renderTo = Ext.getBody();
                            this.renderTpl = [
                                sprintf('<span class="message">%(message)s %(url)s</span>', {
                                  message : _t('Unable to load url'),
                                  url:config.url
                                })
                            ];
                            this.renderSelectors = {
                                messageEl: 'span.message'
                            }
                            this.layoutOnShow = Ext.create('Ext.util.MixedCollection');
                            this.callParent(arguments);
                        },
                        afterRender: function() {
                            var me = this;
                            setTimeout(function(){me.hide();},5000);
                            me.el.on('click',function(){me.hide();});
                        }

                    }).show();
                }
            }, chartCmp.surface);
        }
    }

});