/**
 * Class gets data from DB and transforms it to data for charts
 *
 * User: Artyom Trityak
 * Date: 1/4/12
 * Time: 2:45 PM
 */
Ext.define('App.controller.viewer.chart.Transformer', {
    extend: 'Ext.app.Controller',
    mixins: {
        observable: 'Ext.util.Observable'
    },

    options: {
        currentReportId: false,
        transformedChartsData: {
            CHART_LINE: false,
            CHART_AREA: false,
            CHART_PIE: false,
            CHART_BAR: false,
            CHART_COLUMN: false
        }
    },

    init: function() {
        this.addEvents(
            'chartDataTransformed',
            '_chartDataReceived',
            'error'
        );
        this.on('_chartDataReceived', this._transformData, this);

        this.callParent(arguments);
    },

    transform: function(options) {
        if (!options.hasOwnProperty('grouping') || options.grouping.length === 0) {
            this.fireEvent('error', 'NO_GROUPING');
            return false;
        }
        if (options.grouping.length > 1 || options.grouping[0] === 'USERS' || options.grouping[0] === 'GROUPS') {
            this.fireEvent('error', 'NOT_SUPPORTED_GROUPING');
            return false;
        }

        if (this.options.currentReportId !== options.reportId) {
            for (var key in this.options.transformedChartsData) {
                this.options.transformedChartsData[key] = false;
            }
        }
        if (this.options.transformedChartsData[options.activeChart] === false ||
            this.options.transformedChartsData[options.activeChart][options.activeMetric] === undefined) {

            this._getData(options);
        } else {
            this.fireEvent('chartDataTransformed', {
                data: this.options.transformedChartsData[options.activeChart][options.activeMetric],
                chartType: options.activeChart
            });
        }
    },

    _getData: function(options) {
        var query = this._createQuery(options);
        if (query === false) {
            return false;
        }

        App.storage.Storage.fetchAsync(query, function(data, queryId) {
            console.log(data);
            if (data.length === 0) {
                this.fireEvent('error', 'NO_DATA');
            } else if(data.length < 1000) {
                this.fireEvent('_chartDataReceived', {
                    reportId: options.reportId,
                    data: data,
                    activeChart: options.activeChart,
                    activeMetric: options.activeMetric
                });
            } else {
                this.fireEvent('error', 'TOO_MANY_RECORDS');
            }
        }, this);
    },

    _transformData: function(options) {
        /*...Profit!*/
        var transformedData = [];

        //TODO: transform data to needed format and delete =
        transformedData = options.data;

        if (this.options.transformedChartsData[options.activeChart] === false) {
            this.options.transformedChartsData[options.activeChart] = {};
        }
        this.options.transformedChartsData[options.activeChart][options.activeMetric] = transformedData;
        this.options.currentReportId = options.reportId;

        //TODO: remove testDataToCheck
        this.fireEvent('chartDataTransformed', {
            data: this.options.transformedChartsData[options.activeChart][options.activeMetric],
            chartType: options.activeChart,
            testDataToCheck: this.options.transformedChartsData
        });
    },

    _createQuery: function(options) {
        var currentMetric = options.metric[options.activeMetric];

        var columnsAndGrouping = this.getGroupedMetricColumns(options.grouping, [currentMetric]);

        if (options.activeChart == 'CHART_PIE' && currentMetric == 'BILL') {
            Ext.Array.remove(columnsAndGrouping, 'TOTAL_RING')
        }

        var builderColumns = this.getBuilderColumnsByIdenties(columnsAndGrouping);
        var builderGroupings = this.getBuilderGroupingsByIdenties(columnsAndGrouping);
        var builderSorters = this.getBuilderGroupingsSortersByIdenties(columnsAndGrouping);


        var columns = [];
        for (var i in builderColumns) {
            columns.push({
                name: builderColumns[i]['name'],
                as: builderColumns[i]['as'] + '_CHART_VAL'
            });
        }

        var grouping = [];
        var sort = [];
        for (var i in builderGroupings) {
            grouping.push({
                name: builderGroupings[i]['name'],
                as: 'GROUP_' + builderGroupings[i]['as'] + "_CHART_GROUPING"
            });
            sort.push({
                field: 'GROUP_' + builderGroupings[i]['as'] + "_CHART_GROUPING",
                dest: 'ASC'
            });
        }


        /*for (var index in columnsAndGrouping) {
            if (columnsAndGrouping[index].indexOf('GROUP_') !== -1) {
                grouping.push({
                    name: columnsAndGrouping[index].replace(/^GROUP_/, 'GROUP_BY_'),
                    as: columnsAndGrouping[index] + "_CHART_GROUPING"
                });
                sort.push({
                    field: columnsAndGrouping[index] + "_CHART_GROUPING",
                    dest: 'ASC'
                });
            } else {
                columns.push({
                    name: 'COLUMN_' + columnsAndGrouping[index],
                    as: columnsAndGrouping[index] + '_CHART_VAL'
                });
            }
        }*/

        if (columns.length === 0) {
            columns = undefined;
        }
        if (sort.length === 0) {
            sort = undefined;
        }

        var queryObject ={
            filters: options.filters,
            getTotalRows: false,
            columns: columns,
            grouping: grouping,
            sort: sort
        };
        console.log(queryObject);

        var query = App.storage.Storage.getBuilder().build(queryObject);

        console.log(query);
        return query;
    },

    getBuilderColumnsByIdenties: function(identies) {
        return this.getViewerController().getBuilderColumnsByIdenties(identies);
    },

    getBuilderGroupingsByIdenties: function(identies) {
        return this.getViewerController().getBuilderGroupingsByIdenties(identies);
    },

    getBuilderDefaultSorters: function() {
        return this.getViewerController().getBuilderDefaultSorters();
    },

    getBuilderGroupingsSortersByIdenties: function(identies) {
        return this.getViewerController().getBuilderGroupingsSortersByIdenties(identies);
    },

    getBuilderSummariesByIdenties: function(identies) {
        return this.getViewerController().getBuilderSummariesByIdenties(identies);
    },

    getGroupedMetricColumns: function(grouping, metric) {
        return this.getViewerController().getGroupedMetricColumns(grouping, metric);
    },

    getSimpleMetricColumns: function() {
        return this.getViewerController().getSimpleMetricColumns();
    },

    getViewerController: function() {
        return this.getController("Viewer");
    }
});
