Ext.define('App.proxy.AbstractEvents', {
    extend: 'App.proxy.AbstractProxy',
    alias: 'proxy.events',
    read: function(operation, callback, scope) {
        if (this.getTargetStore().getBuilderType() == 1) {
            return this.readSimple(operation, callback, scope);
        } else {
            return this.readGrouped(operation, callback, scope);
        }
	},
    readSimple: function(operation, callback, scope) {
        var self = this;
        var limit = operation.limit;
        var start = operation.start;

        // build results query
        var resultsQuery = {
            filters: this.getTargetStore().getBuilderFilters(),
            columns: this.getTargetStore().getBuilderColumns(),
            sort: this.getTargetStore().getBuilderSorters(),
            limit: {
                limit: limit,
                offset: start
            }
        }
        var resultsSql = App.storage.Storage.getBuilder().build(resultsQuery);

        // here we allready have summaries, and should just laod results
        if (this.reportId == this.getTargetStore().getReportId()) {
            this.getStorage().batch(
                [App.storage.Storage.METHOD_FETCH, resultsSql],
                function(results) {
                    self.onReadedSimpleWithoutSummaries.call(self, results, operation, callback, scope);
                }
            );
            return;
        }

        // build summary query
        var sumQuery = {
            filters: this.getTargetStore().getBuilderFilters(),
            columns: this.getTargetStore().getBuilderSummaries()
        }
        var sumSql = App.storage.Storage.getBuilder().build(sumQuery);

        // executing
        this.getStorage().batch(
            [App.storage.Storage.METHOD_FETCH, resultsSql],
            [App.storage.Storage.METHOD_FETCHROW, sumSql],
            function(results, summaries) {
                self.onReadedSimple.call(self, results, summaries, operation, callback, scope);
            }
        );

        this.reportId = this.getTargetStore().getReportId();
    },
    onReadedSimple: function(results, summaries, operation, callback, scope) {
        var total, result, reader;

        this.simplePreviosSummaries = summaries;
        this.getTargetStore().setSummaries(summaries);

        if (summaries['TOTAL_EVENTS']) {
            total = parseInt(summaries['TOTAL_EVENTS']);
        }

        reader = this.getReader();
        result = reader.read(results);
        result.total = total;
        result.count = results.length;

        operation.setCompleted();
        operation.setSuccessful();

        Ext.apply(operation, { resultSet: result } );
        Ext.callback(callback, scope || this, [operation]);
    },
    onReadedSimpleWithoutSummaries: function(results, operation, callback, scope) {
        var total, result, reader, summaries = this.simplePreviosSummaries;

        if (summaries['TOTAL_EVENTS']) {
            total = parseInt(summaries['TOTAL_EVENTS']);
        }

        reader = this.getReader();
        result = reader.read(results);
        result.total = total;
        result.count = results.length;

        operation.setCompleted();
        operation.setSuccessful();

        Ext.apply(operation, { resultSet: result } );
        Ext.callback(callback, scope || this, [operation]);
    },
    readGrouped: function(operation, callback, scope) {
        var self = this;
        var limit = operation.limit;
        var start = operation.start;

        // build results query
        var resultsQuery = {
            filters: this.getTargetStore().getBuilderFilters(),
            columns: this.getTargetStore().getBuilderColumns(),
            grouping: this.getTargetStore().getBuilderGroupings(),
            sort: this.getTargetStore().getBuilderSorters(),
            limit: {
                limit: limit,
                offset: start
            }
        }
        var resultsSql = App.storage.Storage.getBuilder().build(resultsQuery);

        // here we allready have summaries, and should just laod results
        if (this.reportId == this.getTargetStore().getReportId()) {
            this.getStorage().batch(
                [App.storage.Storage.METHOD_FETCH, resultsSql],
                function(results) {
                    self.onReadedGroupedWithoutSummaries.call(self, results, operation, callback, scope);
                }
            );
            return;
        }

        // build total rows query
        var totalQuery = {
            filters: this.getTargetStore().getBuilderFilters(),
            grouping: this.getTargetStore().getBuilderGroupings(),
            getTotalRows: true
        }
        var totalSql = App.storage.Storage.getBuilder().build(totalQuery);

        // build summary query
        var sumQuery = {
            filters: this.getTargetStore().getBuilderFilters(),
            columns: this.getTargetStore().getBuilderSummaries()
        }

        if (this.getTargetStore().getBuilderSummariesType() == 'extended') {
            sumQuery['grouping'] = this.getTargetStore().getBuilderGroupings();
            sumQuery['getSumRows'] = true;
        }

        var sumSql = App.storage.Storage.getBuilder().build(sumQuery);

        // executing
        this.getStorage().batch(
            [App.storage.Storage.METHOD_FETCH, resultsSql],
            [App.storage.Storage.METHOD_FETCHONE, totalSql],
            [App.storage.Storage.METHOD_FETCHROW, sumSql],
            function(results, total, summaries) {
                self.onReadedGrouped.call(self, results, total, summaries, operation, callback, scope);
            }
        );

        this.reportId = this.getTargetStore().getReportId();
    },
    onReadedGrouped: function(results, total, summaries, operation, callback, scope) {
        var result, reader;

        this.groupedPreviosSummaries = summaries;
        this.groupedPreviosTotal = total;
        this.getTargetStore().setSummaries(summaries);

        total = parseInt(total);
        reader = this.getReader();
        result = reader.read(results);
        result.total = total;
        result.count = results.length;

        operation.setCompleted();
        operation.setSuccessful();

        Ext.apply(operation, { resultSet: result } );
        Ext.callback(callback, scope || this, [operation]);
    },
    onReadedGroupedWithoutSummaries: function(results, operation, callback, scope) {
        var result, reader, summaries = this.groupedPreviosSummaries, total = this.groupedPreviosTotal;

        this.getTargetStore().setSummaries(summaries);

        total = parseInt(total);
        reader = this.getReader();
        result = reader.read(results);
        result.total = total;
        result.count = results.length;

        operation.setCompleted();
        operation.setSuccessful();

        Ext.apply(operation, { resultSet: result } );
        Ext.callback(callback, scope || this, [operation]);
    }
});
