Ext.define('App.proxy.Users', {
	extend: 'App.proxy.AbstractProxy',
	alias: 'proxy.users',
	constructor	: function(config) {
		this.callParent(arguments);
	},
	read: function(operation, callback, scope) {
        console.log(['create', operation]);
	},
	readCount: function() {
        var s = App.storage.Storage;
        var c = s.featchOne("SELECT COUNT(*) FROM `CDR`.`USERS`");

        return c;
	},

    _insertedUsersIds: [],
    /**
     * Synchronize users to local db (insertUsers applet analog)
     * @param array data Array of objects {id,name,type,number,mobile_number,fax_number,office,home,home_mobile,phone,organization}
     * @param callable callback
     */
    pushUsers: function(users, isFirstTimeSync, callback){
        var self = this,
            esc = self.escape,
            storage = self.getStorage();
        if (users.length==0)
            callback();
        Promise.resolve()
            .then(function(){
                if (storage.proxyGroups._dbGroups == null)
                    return storage.fetchPromise('SELECT ID,UID,NAME,NAME_LOWER,TYPE,`IGNORE` FROM GROUPS');
                else
                    return storage.proxyGroups._dbGroups;
            })
            .then(function(groups){
                storage.proxyGroups._dbGroups = groups;
                //console.debug('pushUsers db groups', groups);
                var operations = [];
                for (var i=0; i < users.length; i++) {
                    var organizationLower = (users[i].organization||'').toLowerCase();
                    if (organizationLower == '')
                        continue;
                    var presentInDb = false;
                    for (var g=0; g < groups.length; g++) {
                        if (groups[g]['TYPE'] == 2 && organizationLower == groups[g]['NAME_LOWER']){
                            if (groups[g]['IGNORE'] == 1){
                                operations.push({type:'disIgnore',data:{ID:groups[g]['ID']}});
                            }
                            presentInDb = true;
                            break;
                        }
                    }
                    if (!presentInDb) {
                        groups.push({ID:null,UID:'-',NAME:users[i].organization,NAME_LOWER:organizationLower,TYPE:2,IGNORE:0});
                        operations.push({type:'insert',data:{ID:null,UID:'-',NAME:users[i].organization,NAME_LOWER:organizationLower,TYPE:2,IGNORE:0}});
                    }
                }

                return storage.proxyGroups.pushGroupsPromises(operations);
            })
            .then(function(){
                if (isFirstTimeSync){
                    var isNeedToDelete = false;
                    for (var i=0; i < users.length; i++)
                        if (self._insertedUsersIds.indexOf(users[i].id)!==-1){
                            isNeedToDelete = true;
                            break;
                        }
                    if (!isNeedToDelete)
                        return null;
                }
                //Here we delete all users with specified IDs
                var usersIds = [];
                for (var i=0; i < users.length; i++)
                    usersIds.push(users[i].id);
                return storage.executePromise('DELETE FROM USERS WHERE ID IN ('+(usersIds.join(','))+')');
            })
            .then(function(){
                if (storage.proxyGroups._dbGroups == null)
                    return storage.fetchPromise('SELECT ID,UID,NAME,NAME_LOWER,TYPE,`IGNORE` FROM GROUPS');
                else
                    return storage.proxyGroups._dbGroups;
            })
            .then(function(groups){
                storage.proxyGroups._dbGroups = groups;
                //Here we ensured that all groups which have users in db and we can get IDs
                //prepare insert data
                var insertRows = [];
                for (var i=0; i < users.length; i++) {
                    var d = users[i];
                    var organizationLower = (d.organization||'').toLowerCase();
                    if (organizationLower)
                        for (var g=0; g < groups.length; g++)
                            if (groups[g]['TYPE'] == 2 && organizationLower == groups[g]['NAME_LOWER']){
                                d.groupId = groups[g]['ID'];
                                break;
                            }
                    var nameLower = (d.name||'').toLowerCase();
                    var search = nameLower+'|'+d.number+'|'+d.mobile_number
                               +'|'+d.fax_number +'|'+d.office+'|'+d.home
                               +'|'+d.home_mobile +'|'+d.phone;

                    insertRows.push({
                        ID: d.id,
                        GROUP_ID: d.groupId,
                        NAME: d.name,
                        NAME_LOWER: nameLower,
                        NAME_SEARCH: search,
                        'TYPE': d['type'],
                        NUMBER: d.number,
                        MOBILE_NUMBER: d.mobile_number,
                        FAX_NUMBER: d.fax_number,
                        OFFICE_NUMBER: d.office,
                        HOME_NUMBER: d.home,
                        HOME_MOBILE_NUMBER: d.home_mobile,
                        PHONE_NUMBER: d.phone
                    });
                }
                //multiple insert
                return storage.bulkInsertPromise('USERS',insertRows).then(function(){
                    //When update button pressed new entity of users have to be merged with existen in eventsProxy
                    if (storage.proxyEvents._dirrectoryUsers.length>0)
                        for (var i=0;i<insertRows.length;i++){
                            storage.proxyEvents.addUserModel(insertRows[i]);
                        }
                });
            })
            .then(function(){
                for (var i=0; i < users.length; i++)
                    self._insertedUsersIds.push(users[i].id);
            })
            .then(callback)
            .catch(function(e){console.debug('ON  * error', e , arguments);});
    }
});
