Ext.define('App.storage.Bootstrap', {
    requires: [
        'App.proxy.AbstractProxy',
        'App.proxy.Synchronization',
        'App.storage.Storage',
        'App.storage.Synchronizer',
        'App.storage.QueryConstructor',
        'App.storage.QueryConstructorClickHouse',
        'App.view.bootstrap.Notification',
        'App.view.bootstrap.Sync',
        'App.view.bootstrap.ErrorLocked',
        'App.view.bootstrap.ErrorConnected',
        'App.view.bootstrap.ErrorConnection',
        'App.view.bootstrap.ErrorUnSupportedBrowser',
        'App.view.bootstrap.ErrorNotAllowed',
        'App.utility.Date',
        'App.utility.NamesFormatter',
        'App.utility.ImageExporter',
        'App.view.bootstrap.ErrorWISNotAppropriateVersion',
        'App.view.bootstrap.ErrorOSNotSupported',
        'App.view.bootstrap.ErrorDownloadStarted',
        'App.view.bootstrap.ErrorWISNotResponding',
        'App.view.bootstrap.ErrorWISNotInstalled'
    ],
    mixins: {
        observable: 'Ext.util.Observable'
    },
    constructor: function() {
        this.bootMask = Ext.get('boot-mask');
        this.bootLoader = Ext.get('boot-loader');



        if (!User.haveLicense){
            return Ext.create('App.view.bootstrap.ErrorNotAllowed', { renderTo: this.bootMask, reason: 'No CDRView license' });
        }
        if (!User.canUse){
            return Ext.create('App.view.bootstrap.ErrorNotAllowed', { renderTo: this.bootMask, reason: 'Access denied' });
        }

        this.storage = App.storage.Storage;
        this.storage.addListener('init', this.onInit, this);

        this.synchronizer = App.storage.Synchronizer;
        this.bindSynchronizerEvents();

        if (!this.storage.isOSSupported()) {
            // Shows a error message and stops any execution
            Ext.create('App.view.bootstrap.ErrorOSNotSupported', { renderTo: this.bootMask });
            this.hideLoader();
            return;
        } else if (!this.storage.isBrowserSupported() && Ext.util.Cookies.get('iWantThisBrowser')!=='1') {
            this.unSupportedErrorDialog = Ext.create('App.view.bootstrap.ErrorUnSupportedBrowser', { renderTo: this.bootMask });
            this.unSupportedErrorDialog.on('try', this.onTryBrowser, this);
            this.hideLoader();
        } else {
            //this.initDialog = Ext.create('App.view.bootstrap.Init', { renderTo: this.bootMask });
            this.storage.lookup();
        }

        this.callParent(arguments);
    },

    bindSynchronizerEvents: function(){
        this.synchronizer.on('syncUsersChange', this.onSyncUsersChange, this);
        this.synchronizer.on('syncUsersComplete', this.onSyncUsersComplete, this);
        this.synchronizer.on('syncExtraComplete', this.onSyncExtraComplete, this);
        this.synchronizer.on('syncBackendComplete', this.onSyncBackendComplete, this);
        this.synchronizer.on('syncEventsChange', this.onSyncEventsChange, this);
        this.synchronizer.on('syncEventsComplete', this.onSyncEventsComplete, this);
        this.synchronizer.on('syncTerminated', this.onSyncTerminated, this);
        this.synchronizer.on('syncTerminate', this.onSyncTerminate, this);

        this.synchronizer.on('exceptionError', this.onSyncExceptionError, this);
        this.synchronizer.on('connectionError', this.onSyncConnectionError, this);
        this.synchronizer.on('connectionRestored', this.onSyncConnectionRestored, this);
    },
    unBindSynchronizerEvents: function(){
        this.synchronizer.un('syncUsersChange', this.onSyncUsersChange, this);
        this.synchronizer.un('syncUsersComplete', this.onSyncUsersComplete, this);
        this.synchronizer.un('syncExtraComplete', this.onSyncExtraComplete, this);
        this.synchronizer.un('syncEventsChange', this.onSyncEventsChange, this);
        this.synchronizer.un('syncEventsComplete', this.onSyncEventsComplete, this);
        this.synchronizer.un('syncTerminated', this.onSyncTerminated, this);
        this.synchronizer.un('syncTerminate', this.onSyncTerminate, this);
        this.synchronizer.un('syncBackendComplete', this.onSyncBackendComplete, this);

        this.synchronizer.un('exceptionError', this.onSyncExceptionError, this);
        this.synchronizer.un('connectionError', this.onSyncConnectionError, this);
        this.synchronizer.un('connectionRestored', this.onSyncConnectionRestored, this);
    },
    /**
     * Fetches new data from PBX and redraw chart and tables
     */
    incrementalSynchronization: function(){

        var view = this.application.controllers.get('Viewport').view.down('#viewport');
        if (!view.mask){
            view.mask = new Ext.LoadMask(Ext.getBody(), {msg:false});
            view.mask.show();
        }
        //this.initDialog = Ext.create('App.view.bootstrap.Init', { renderTo: this.bootMask });
        this.bindSynchronizerEvents();
        this.storage.lookup();
        //
        this.synchronizer.on('syncEventsComplete', this.incrementalSynchronizationComplete, this);
    },
    incrementalSynchronizationComplete: function(){
        this.application.controllers.get('Viewer').fireEvent('incrementalSynchronization');
        this.synchronizer.un('syncEventsComplete', this.incrementalSynchronizationComplete, this);
    },
    /**
     * Fires when user clicked to Try Current Browser (Unsupported) button.
     */
    onTryBrowser: function() {
        Ext.destroy(this.unSupportedErrorDialog);

        //this.initDialog = Ext.create('App.view.bootstrap.Init', { renderTo: this.bootMask });
        this.storage.lookup();
    },
    onInit: function() {
        //Ext.destroy(this.initDialog);
        this.hideLoader();

        if (!this.storage.isInstalled()) {
            // Shows a error message and stops any execution
            this.errorDialog = Ext.create('App.view.bootstrap.ErrorWISNotInstalled', { renderTo: this.bootMask, installUrl:this.storage.getWIServiceUrl(),
                onDownloadStarted: Ext.bind(this.onDownloadStarted, this)
            });
            this.storage.close();
            return;
        } else if (!this.storage.isVersionReceived()) {
            // Shows a error message and stops any execution
            this.errorDialog = Ext.create('App.view.bootstrap.ErrorWISNotResponding', { renderTo: this.bootMask, installUrl:this.storage.getWIServiceUrl(),
                onDownloadStarted: Ext.bind(this.onDownloadStarted, this)
            });
            this.storage.close();
            return;
        } else if (!this.storage.isAppropriateVersion()) {
            // Shows a error message and stops any execution
            this.errorDialog = Ext.create('App.view.bootstrap.ErrorWISNotAppropriateVersion', { renderTo: this.bootMask, installUrl:this.storage.getWIServiceUrl(),
                onDownloadStarted: Ext.bind(this.onDownloadStarted, this)
            });
            this.storage.close();
            return;
        } else if (!this.storage.isConnected()) {
            // Shows a error message and stops any execution
            this.errorDialog = Ext.create('App.view.bootstrap.ErrorConnected', { renderTo: this.bootMask });
            this.storage.close();
            return;
        } else if (this.storage.isLocked()) {
            // Shows a error message and stops any execution
            this.errorDialog = Ext.create('App.view.bootstrap.ErrorLocked', { renderTo: this.bootMask });
            this.storage.close();
            return;
        } else {
            this.syncDialog = Ext.create('App.view.bootstrap.Sync', { renderTo: this.bootMask });
        }

        this.syncDialog.on('сancelButtonClick', this.onCancelButtonClick, this);
        if (isRemoteMode()) {
            console.log('_________ REMOTE ');
            this.synchronizer.fireEvent('syncEventsComplete');
        } else {
            console.log('_________ LOCAL');
            this.syncDialog.markUsersSync();
            this.synchronizer.fireEvent('syncUsers');
        }
    },

    onDownloadStarted: function(){
        this.closeErrorDialog();
        this.errorDialog = Ext.create('App.view.bootstrap.ErrorDownloadStarted', { renderTo: this.bootMask });
    },
    closeErrorDialog: function(){
        if (this.errorDialog)
            this.errorDialog.hide();
            this.errorDialog.destroy();
            this.errorDialog = null;
    },

    onSyncUsersChange: function(percent, timeleft) {
        this.syncDialog.markUsersSync(percent, timeleft);
    },
    /**
     * Fires when sync of users was completed successfully.
     */
    onSyncUsersComplete: function() {
        console.log('onSyncUsersComplete', arguments);

        this.syncDialog.markExtraSync();
        this.synchronizer.fireEvent('syncExtra');
    },

    /**
     * Fires when sync of extra information was completed successfully.
     */
    onSyncExtraComplete: function() {
        console.log('onSyncExtraComplete', arguments);

        this.syncDialog.markBackendSync();
        this.synchronizer.fireEvent('syncBackend');
    },

    /**
     * Fires when sync of backend information was completed successfully, we should go to events sync step.
     */
    onSyncBackendComplete: function() {
        console.log('onSyncBackendComplete', arguments);

        // Define a period for events synchronization.
        var period = 0;

        if (User.hasOwnProperty('period')) {
            period = User['period'];
        }

        var ranges = this.getEventsPeriodByType(period);
        var dateFrom = ranges[0];
        var dateTo = ranges[1];

        this.syncDialog.markEventsSync();
        this.synchronizer.fireEvent('syncEvents', dateFrom, dateTo, false);
    },

    /**
     * Fires when sync of users has been changed.
     */
    onSyncEventsChange: function(percent, timeleft) {
        this.syncDialog.markEventsSync(percent, timeleft);
    },

    /**
     * Fires when sync of users was completed successfully,
     */
    onSyncEventsComplete: function() {
        this.syncDialog.markEventsSync(100, 0);
        this.onSyncComplete();
    },

    /**
     * Fires when sync was completed successfully, we should initialize application.
     */
    onSyncComplete: function() {
        var self = this;
        var viewport;

        this.bootMask.setStyle('z-index', 10000);
        this.bootLoader.setStyle('z-index', 10010);
        this.syncDialog.hide(true);
        this.bootLoader.show(true);

        this.unBindSynchronizerEvents();
        if (!this.application)
            this.application = Ext.create('Application');


        var checker = function() {
            viewport = self.application.controllers.get('Viewport');

            if( self.application.controllers.get('Viewport') == undefined ) {
                setTimeout(checker, 200 );
                return false;
            }

            if( viewport.view == undefined || viewport.view.rendered == false ) {
                setTimeout(checker, 200 );
                return false;
            }

            self.bootLoader.hide(true);
            self.bootMask.hide(true);
            self.syncDialog.hide();

            Ext.destroy(self.syncDialog);
            var view = viewport.view.down('#viewport');
            if (view.mask){
                Ext.destroy(view.mask);
                view.mask = null;
            }

        }

        setTimeout( checker, 100 );
    },

    /**
     * Fires when synchronizer found an exception (stop working), we should
     * show this exception.
     */
    onSyncExceptionError: function(message) {
        if (this.syncDialog) {
            this.syncDialog.hide();
        }

        if (this.connErrorDialog) {
            this.connErrorDialog.hide();
        }

        this.errorDialog = Ext.create('App.view.bootstrap.ErrorCritical', {
            response: message
        });
        this.errorDialog.show();
    },

    /**
     * Fires when synchronizer found a connection problem, we should show notification
     * about this at top of page.
     */
    onSyncConnectionError: function(callback, scope) {
        if (!this.connErrorDialog) {
            this.connErrorDialog = Ext.create('App.view.bootstrap.ErrorConnection');
        }

        this.connErrorCallback = callback;
        this.connErrorScope = scope || this;

        this.connErrorDialog.handle(30, this.onSyncConnectionErrorCallback, this);
    },

    /**
     * Fires when synchronizer was been normally restored a connection to pbx, we should show a notification
     * about connection problems.
     */
    onSyncConnectionRestored: function() {
        if (this.connErrorDialog) {
            this.connErrorDialog.hide();
        }
    },

    /**
     * Fires when we should repeat failure request to a server
     */
    onSyncConnectionErrorCallback: function() {
        this.connErrorCallback.call(this.connErrorScope);
    },

    onSyncTerminate: function() {
        this.syncDialog.markTerminating();
    },

    onSyncTerminated: function() {
        this.syncDialog.markTerminated();
        this.onSyncComplete();
    },

    onCancelButtonClick: function() {
        this.synchronizer.fireEvent('syncTerminate');
    },
    /**
     * Gets a period ranges as array by specified type.
     */
    getEventsPeriodByType: function(type) {
        var dateTo = Ext.Date.format(Ext.Date.add(get_current_date(), Ext.Date.DAY,1), "Y-m-d");
        var dateFrom;

        switch(parseInt(type)) {
            case App.utility.Date.MONTH:     // 1 month
                dateFrom = App.utility.Date.getThisMonth();
                //dateFrom = Ext.Date.format(Ext.Date.add(get_current_date(), Ext.Date.MONTH, -1), "Y-m-d");
                break;
            case App.utility.Date.LAST_3_MONTH:      // 3 months
                dateFrom = App.utility.Date.getPrevNMonth(3);
                //dateFrom = Ext.Date.format(Ext.Date.add(get_current_date(), Ext.Date.MONTH, -3), "Y-m-d");
                break;
            case App.utility.Date.LAST_6_MONTH:      // 6 months
                dateFrom = App.utility.Date.getPrevNMonth(6);
                //dateFrom = Ext.Date.format(Ext.Date.add(get_current_date(), Ext.Date.MONTH, -6), "Y-m-d");
                break;
            case App.utility.Date.LAST_12_MONTH:      // 1 year
                dateFrom = App.utility.Date.getPrevNMonth(12);
                //dateFrom = Ext.Date.format(Ext.Date.add(get_current_date(), Ext.Date.MONTH, -12), "Y-m-d");
                break;
            default: // App.utility.Date.LAST_2_WEEKS        // 2 weeks
                dateFrom = App.utility.Date.getPrevWeek();
                //dateFrom = Ext.Date.format(Ext.Date.add(get_current_date(), Ext.Date.DAY, -14), "Y-m-d");
            break;
        }
        dateFrom = dateFrom.split(' ')[0];
        return [dateFrom, dateTo];
    },
    hideLoader: function() {
        Ext.get('boot-loader').hide(false);
    },
    showLoader: function() {
        Ext.get('boot-loader').show(false);
    },
    hideMask: function() {
        Ext.get('boot-mask').hide(false);
    },
    showMask: function() {
        Ext.get('boot-mask').show(false);
    }
} );
