Ext.define('App.storage.QueryConstructorClickHouse', {
    singleton: true,
    requires: [
        'App.utility.Date'
    ],

    dateUtility: false,
    _tmpTableNumber: 0,

    constructor: function() {
        this.dateUtility = App.utility.Date;

        this.tmpTableName = function() {
            this._tmpTableNumber++;
            return 'tmp_' + this._tmpTableNumber;
        };
    },

    /*
    * Exapmle object {
    *     filters: {
    *         bill,
    *         cost,
    *         destination,
    *         duration,
    *         period,
    *         trunks,
    *         tags,
    *         types
    *     },
    *     grouping: ['DATE_DAY', 'SERVICE', 'PBX'] or empty array,
    *     sort: {
    *         field: 'FIELD_NAME',
    *         order: 'ASC / DESC'
    *     },
    *     columns: [
    *       {
    *           name: '',
    *           as: ''
    *       }
    *     ],
    *     limit: {
    *         limit: num,
    *         offset: num
    *     }
    * }
    */
    build: function(queryObject) {
        var queryBuilder = new this.QueryBuilder(this);

        var filters = queryObject['filters'];
        for (var key in filters) {
            if (this.decorators.hasOwnProperty(key)) {
                queryBuilder.decorate(key, filters[key]);
            }
        }

        var columns = queryObject['columns'];
        for (var key in columns) {
            if (this.decorators.hasOwnProperty(columns[key].name)) {
                queryBuilder.decorate(columns[key].name, columns[key]);
            }
        }

        var grouping = queryObject['grouping'];
        for (var key in grouping) {
            if (this.decorators.hasOwnProperty(grouping[key].name)) {
                queryBuilder.decorate(grouping[key].name, Object.assign(grouping[key], {filters: filters}));
            }
        }

        if (queryObject['limit'] !== undefined) {
            queryBuilder.decorate('limits', queryObject['limit']);
        }

        if (queryObject['sort'] !== undefined) {
            queryBuilder.decorate('sort', queryObject['sort']);
        }

        return queryBuilder.make(queryObject['getTotalRows'], queryObject['getSumRows']);
    },

    /*Query builder. Pattern: Decorator*/
    QueryBuilder: function(scope) {
        var decoratorsList = [];
        var parent = scope;
        var lazyDecorators = [];

        var queryData = {
            where: [],
            fields: [],
            unionFields: [],
            from: ['`EVENTS` as events_main'],
            joins: [],
            order: '',
            grouping: [],
            limit: '',
            assocForSuperSelect: []
        }

        var decorate = function(decoratorName, data) {
            decoratorsList.push({
                name: decoratorName,
                data: data
            });
        }

        var make = function(getTotalRows, getSumRows) {
            for (var i in decoratorsList) {
                if (decoratorsList[i].name === 'GROUP_BY_USER' || decoratorsList[i].name === 'GROUP_BY_CLIENT' || decoratorsList[i].name === 'GROUP_BY_NUMBER' || decoratorsList[i].name === 'GROUP_BY_GROUP'
                    || decoratorsList[i].name === 'GROUP_BY_COMMUNICATION_OF_USERS' || decoratorsList[i].name === 'GROUP_BY_COMMUNICATION_OF_GROUPS') {

                    lazyDecorators.push(decoratorsList[i]);
                    continue;
                }
                queryData = parent.decorators[decoratorsList[i].name].call(parent, decoratorsList[i].data, queryData);
            }

            var resultQuery = '';

            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT " + queryData.fields.join(',');
            resultQuery += " FROM " + queryData.from.join(',');
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (queryData.grouping.length > 0) {
                resultQuery += " GROUP BY " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += " ) " + parent.tmpTableName() + ' ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }
            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            for (var i in lazyDecorators) {
                resultQuery = parent.decorators[lazyDecorators[i].name].call(parent, lazyDecorators[i].data, queryData, getTotalRows, getSumRows);
            }
            return resultQuery;
        }

        return {
            decorate: decorate,
            make: make
        };
    },

    /*Decorators*/
    decorators: {
        /*Filter decorators */
        bill: function(data, queryData) {
            var query = [];

            if (data.leftEnabled === true && data.rightEnabled === true && data.leftValue === data.rightValue) {
                query.push(" events_main.BILL = " + data.leftValue);
            } else {
                if (data.leftEnabled === true) {
                    query.push(" events_main.BILL > " + data.leftValue);
                }
                if (data.rightEnabled === true) {
                    query.push(" events_main.BILL < " + data.rightValue + " ");
                }
            }
            if (query.length > 0) {
                query = "(" + query.join(' AND ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        cost: function(data, queryData) {
            var query = [];

            if (data.leftEnabled === true && data.rightEnabled === true && data.leftValue === data.rightValue) {
                query.push(" events_main.COST = " + data.leftValue);
            } else {
                if (data.leftEnabled === true) {
                    query.push(" events_main.COST > " + data.leftValue);
                }
                if (data.rightEnabled === true) {
                    query.push(" events_main.COST < " + data.rightValue + " ");
                }
            }
            if (query.length > 0) {
                query = "(" + query.join(' AND ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        duration: function(data, queryData) {
            var query = [];

            if (data.leftEnabled === true && data.rightEnabled === true && data.leftValue === data.rightValue) {
                query.push(" events_main.DURATION = " + data.leftValue);
            } else {
                if (data.leftEnabled === true) {
                    query.push(" events_main.DURATION > " + data.leftValue);
                }
                if (data.rightEnabled === true) {
                    query.push(" events_main.DURATION < " + data.rightValue + " ");
                }
            }
            if (query.length > 0) {
                query = "(" + query.join(' AND ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        destination: function(data, queryData) {
            var query = [];

            var subFilters = [
                {
                    keyCheck: 'extToIntEnabled',
                    from: 'extToIntFrom',
                    to: 'extToIntTo',
                    direction: 2
                },
                {
                    keyCheck: 'intToExtEnabled',
                    from: 'intToExtFrom',
                    to: 'intToExtTo',
                    direction: 3
                },
                {
                    keyCheck: 'intToIntEnabled',
                    from: 'intToIntFrom',
                    to: 'intToIntTo',
                    direction: 4
                }
            ];

            for (var i in subFilters) {
                if (!data.hasOwnProperty(subFilters[i].keyCheck) || data[subFilters[i].keyCheck] !== true) {
                    continue;
                }
                var rowQuery = [];
                rowQuery.push(
                    " events_main.DIRECTION = " + subFilters[i].direction + " "
                );

                if (data.hasOwnProperty(subFilters[i].from) && data[subFilters[i].from].length > 0) {
                    var elementsQueryFrom = [];
                    for (var j in data[subFilters[i].from]) {
                        //Generate from query element
                        switch (parseInt(data[subFilters[i].from][j].type, 10)) {
                            case 1:
                                //custom phone number
                                var customNumber = data[subFilters[i].from][j].name;
                                elementsQueryFrom.push(
                                    " events_main.`FROM` " +
                                    ( customNumber && customNumber.indexOf('%')>-1? "like" : "=" ) +
                                    " '" + customNumber + "' "
                                );
                                break;
                            case 2:
                                //call class
                                elementsQueryFrom.push(
                                    " events_main.`CLASS` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 3:
                                //group
                                elementsQueryFrom.push(
                                    " events_main.`FROM_GROUP` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 4:
                                //local user
                                //TODO: optimization: add type of user
                                elementsQueryFrom.push(
                                    " events_main.`FROM_USER` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 5:
                                //phonebook user
                                //TODO: optimization: add type of user
                                elementsQueryFrom.push(
                                    " events_main.`FROM_USER` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                            case 6:
                                //phonebook company
                                elementsQueryFrom.push(
                                    " events_main.`FROM_GROUP` = " + data[subFilters[i].from][j].uid + " "
                                );
                                break;
                        }
                    }
                    //Push to array subqueries
                    if (elementsQueryFrom.length > 0) {
                        rowQuery.push(
                            " ( " + elementsQueryFrom.join(" OR ") + " ) "
                        );
                    }
                }

                if (data.hasOwnProperty(subFilters[i].to) && data[subFilters[i].to].length > 0) {
                    var elementsQueryTo = [];
                    for (var j in data[subFilters[i].to]) {
                        //Generate to query element
                        switch (parseInt(data[subFilters[i].to][j].type, 10)) {
                            case 1:
                                //custom phone number
                                var customToNumber = data[subFilters[i].to][j].name;
                                elementsQueryTo.push(
                                    " events_main.`TO` " +
                                    ( customToNumber && customToNumber.indexOf('%')>-1? "like" : "=" ) +
                                    " '" + customToNumber + "' "
                                );
                                break;
                            case 2:
                                //call class
                                elementsQueryTo.push(
                                    " events_main.`CLASS` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 3:
                                //group
                                elementsQueryTo.push(
                                    " events_main.`TO_GROUP` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 4:
                                //local user
                                //TODO: optimization: add type of user
                                elementsQueryTo.push(
                                    " events_main.`TO_USER` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 5:
                                //phonebook user
                                //TODO: optimization: add type of user
                                elementsQueryTo.push(
                                    " events_main.`TO_USER` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                            case 6:
                                // phonebook company
                                elementsQueryTo.push(
                                    " events_main.`TO_GROUP` = " + data[subFilters[i].to][j].uid + " "
                                );
                                break;
                        }
                    }
                    //Push to array subqueries
                    if (elementsQueryTo.length > 0) {
                        rowQuery.push(
                            " ( " + elementsQueryTo.join(" OR ") + " ) "
                        );
                    }
                }
                query.push(
                    " ( " + rowQuery.join(" AND ") + " ) "
                );
            }

            if (query.length > 0) {
                query = "(" + query.join(' OR ') + ")";
                queryData.where.push(query);
            }

            return queryData;
        },

        period: function(data, queryData) {
            var query = [];

            switch (parseInt(data.type, 10)) {
                case App.utility.Date.TODAY:
                    //today
                    var today = this.dateUtility.getToday();
                    query.push(" events_main.`DATE` >= '" + today + " 00:00:00' ");
                    break;
                case App.utility.Date.YESTERDAY:
                    //yesterday
                    today = this.dateUtility.getToday();
                    var yesterday = this.dateUtility.getYesterday();
                    query.push(" events_main.`DATE` >= '" + yesterday + " 00:00:00' AND events_main.`DATE` <= '" + today + " 00:00:00' ");
                    break;
                case App.utility.Date.WEEK:
                    //This week
                    var thisWeek = this.dateUtility.getThisWeek();
                    query.push(" events_main.`DATE` >= '" + thisWeek + " 00:00:00' ");
                    break;
                case App.utility.Date.LAST_2_DAYS:
                    //Last 2 days
                    var last2Days = this.dateUtility.getLastNDays(2);
                    query.push(" events_main.`DATE` >= '" + last2Days + " 00:00:00' ");
                    break;
                case App.utility.Date.LAST_7_DAYS:
                    //Last 7 days
                    var last7Days = this.dateUtility.getLastNDays(7);
                    query.push(" events_main.`DATE` >= '" + last7Days + " 00:00:00' ");
                    break;
                case App.utility.Date.PREV_WEEK:
                    //Prev week
                    var prevWeek = this.dateUtility.getPrevWeek();
                    thisWeek = this.dateUtility.getThisWeek();
                    query.push(" events_main.`DATE` >= '" + prevWeek + " 00:00:00' AND events_main.`DATE` < '" + thisWeek + " 00:00:00'");
                    break;
                case App.utility.Date.LAST_2_WEEKS:              // LAST_2_WEEKS
                    //Last 2 weeks
                    var last2Weeks = this.dateUtility.getPrevWeek();
                    today = this.dateUtility.getToday();
                    query.push(" events_main.`DATE` >= '" + last2Weeks + " 00:00:00' AND events_main.`DATE` <= '" + today + " 23:59:59'");
                    break;
                case App.utility.Date.MONTH:
                    //This month
                    var thisMonth = this.dateUtility.getThisMonth();
                    query.push(" events_main.`DATE` >= '" + thisMonth + " 00:00:00' ");
                    break;
                case App.utility.Date.LAST_30_DAYS:
                    //Last 30 days
                    var last30Days = this.dateUtility.getLastNDays(30);
                    query.push(" events_main.`DATE` >= '" + last30Days + " 00:00:00' ");
                    break;
                case App.utility.Date.PREV_MONTH:
                    //Prev month
                    thisMonth = this.dateUtility.getThisMonth();
                    var prevMonth = this.dateUtility.getPrevNMonth(1);
                    query.push(" events_main.`DATE` >= '" + prevMonth + " 00:00:00' AND events_main.`DATE` < '" + thisMonth + " 00:00:00'");
                    break;
                case App.utility.Date.LAST_60_DAYS:
                    //Last 60 days
                    var last60Days = this.dateUtility.getLastNDays(60);
                    query.push(" events_main.`DATE` >= '" + last60Days + " 00:00:00' ");
                    break;
                case App.utility.Date.LAST_3_MONTH:
                    //Last 3 months
                    var last3Months = this.dateUtility.getPrevNMonth(3);
                    query.push(" events_main.`DATE` >= '" + last3Months + " 00:00:00' ");
                    break;
                case App.utility.Date.LAST_6_MONTH:      // LAST_6_MONTH
                    //Last 6 months
                    var last6Months = this.dateUtility.getPrevNMonth(6);
                    query.push(" events_main.`DATE` >= '" + last6Months + " 00:00:00' ");
                    break;
                case App.utility.Date.LAST_12_MONTH:
                    //Last 12 months
                    var last12Months = this.dateUtility.getPrevNMonth(12);
                    query.push(" events_main.`DATE` >= '" + last12Months + " 00:00:00' ");
                    break;
                case App.utility.Date.YEAR:
                    //This year
                    var thisYear = this.dateUtility.getThisYear();
                    query.push(" events_main.`DATE` >= '" + thisYear + " 00:00:00' ");
                    break;
                case App.utility.Date.PREV_YEAR:
                    //Prev year
                    thisYear = this.dateUtility.getThisYear();
                    var prevYear = this.dateUtility.getPrevNYears(1);
                    query.push(" events_main.`DATE` >= '" + prevYear + " 00:00:00' AND events_main.`DATE` <= '" + thisYear + " 23:59:59' ");
                    break;
                case App.utility.Date.CUSTOM:
                    //Custom
                    query.push(" events_main.`DATE` >= '" + data['fromDate'] + " " + data['fromTime'] + "' AND events_main.`DATE` <= '" + data['toDate'] + " " + data['toTime'] + "'  ");
                    break;
            }

            queryData.where.push("(" + query.join(' AND ') + ")");

            return queryData;
        },

        trunks: function(data, queryData) {
            if (!data.hasOwnProperty('trunks') || data.trunks.length === 0) {
                return queryData;
            }
            var query = [];
            for (var i in data.trunks) {
                query.push(
                    " events_main.TRUNK = " + data.trunks[i].id +  " "
                );
            }
            queryData.where.push(" ( " + query.join(" OR ") + " ) ");
            return queryData;
        },

        pbxes: function(data, queryData) {
            if (!data.hasOwnProperty('pbxes') || data.pbxes.length === 0) {
                return queryData;
            }
            var query = [];
            for (var i in data.pbxes) {
                query.push(
                    " events_main.PBX = " + data.pbxes[i].id +  " "
                );
            }
            queryData.where.push(" ( " + query.join(" OR ") + " ) ");
            return queryData;
        },

        tags: function(data, queryData) {
            if (!data.hasOwnProperty('tags') || data.tags.length === 0) {
                return queryData;
            }

            var queryArr = [];
            for (var i in data.tags) {
                queryArr.push('events_main.TAGS_SEARCH LIKE (\'%|' + data.tags[i].name +'|%\')');
            }
            queryData.where.push('(' + queryArr.join(' OR ')+ ')');

            return queryData;
        },

        types: function(data, queryData) {
            if (data.call === false && data.fax === false && data.voicemail === false) {
                return queryData;
            }
            var query = [];
            if (data.call === true) {
                query.push(" events_main.TYPE = 1 ")
            }
            if (data.fax === true) {
                query.push(" events_main.TYPE = 2 ")
            }
            if (data.voicemail === true) {
                query.push(" events_main.TYPE = 3 ")
            }
            queryData.where.push(" ( " + query.join(" OR ") + " ) ");
            return queryData;
        },

        miss_call_sec_period: function(data, queryData) {
            if (parseInt( data )>0) {
                queryData.where.push(" ( events_main.BILL != 0 or events_main.RING > "+ parseInt( data ) +" ) ");
            }
            return queryData;
        },

        /*Columns decorators*/
        COLUMN_ID: function(data, queryData) {
            queryData.fields.push(
                " events_main.`ID` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_RING: function(data, queryData) {
            queryData.fields.push(
                " (events_main.`RING`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_DATE: function(data, queryData) {
            queryData.fields.push(
                " events_main.`DATE` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DURATION: function(data, queryData) {
            queryData.fields.push(
                " events_main.`DURATION` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM: function(data, queryData) {
            queryData.fields.push(
                " events_main.`FROM` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM_USER: function(data, queryData) {
            queryData.fields.push(
                " events_main.`FROM_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO: function(data, queryData) {

            queryData.fields.push(
                " events_main.`TO` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO_USER: function(data, queryData) {

            queryData.fields.push(
                " events_main.`TO_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COUNTRY: function(data, queryData) {

            queryData.fields.push(
                " events_main.`COUNTRY_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COUNTRY_AREA: function(data, queryData) {

            queryData.fields.push(
                " events_main.`COUNTRY_AREA_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_CLASS: function(data, queryData) {

            queryData.fields.push(
                " events_main.`CLASS` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DEST_TYPE: function(data, queryData) {

            queryData.fields.push(
                " events_main.`CLASS` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_BILL: function(data, queryData) {

            queryData.fields.push(
                " events_main.`BILL` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COST: function(data, queryData) {

            queryData.fields.push(
                " events_main.`COST` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COST_CALL: function(data, queryData) {

            queryData.fields.push(
                " events_main.`COST_CALL` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_COST_FAX: function(data, queryData) {

            queryData.fields.push(
                " events_main.`COST_FAX` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TRUNK: function(data, queryData) {

            queryData.fields.push(
                " events_main.`TRUNK_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_PBX: function(data, queryData) {
            queryData.fields.push(
                " events_main.`PBX_NAME` as `" + data.as + "` "
            );

            return queryData;
        },


        COLUMN_TAG: function(data, queryData) {
//            var tagsEventsTable = 'LEFT JOIN `EVENTS_TAGS` as events_tags_join ON events_tags_join.event_id = events_main.id';
//            var tagsTable = 'LEFT JOIN `TAGS` as tags_join ON tags_join.id = events_tags_join.tag_id';
//
//            if (Ext.Array.indexOf(queryData.joins, tagsEventsTable) === -1) {
//                queryData.joins.push(tagsEventsTable);
//            }
//            if (Ext.Array.indexOf(queryData.joins, tagsTable) === -1) {
//                queryData.joins.push(tagsTable);
//            }

            queryData.fields.push(
                " events_main.`TAGS` as `" + data.as + "` "
            );

//            queryData.fields.push(
//                " (SELECT GROUP_CONCAT(`tg`.`name`) FROM PUBLIC.EVENTS_TAGS as `et` INNER JOIN PUBLIC.TAGS as `tg` ON `et`.`tag_id` = `tg`.`id` WHERE `et`.`event_id` = `events_main`.`id`) `" + data.as + "` "
//            );
//            queryData.fields.push(
//                " GROUP_CONCAT(`tags_join`.`name`) `" + data.as + "` "
//            );
//            queryData.grouping.push(
//                "`events_main`.`id`"
//            );

            return queryData;
        },

        COLUMN_SERVICE: function(data, queryData) {

            queryData.fields.push(
                " events_main.`TYPE` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_USERFIELD: function(data, queryData) {

            queryData.fields.push(
                " events_main.`USERFIELD` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FAXDATA: function(data, queryData) {

            queryData.fields.push(
                " events_main.`FAXDATA` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_FROM_GROUP: function(data, queryData) {

            queryData.fields.push(
                " events_main.`FROM_GROUP_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TO_GROUP: function(data, queryData) {

            queryData.fields.push(
                " events_main.`TO_GROUP_NAME` as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_HOUR: function(data, queryData) {

            queryData.fields.push(
                " events_main.`HOUR` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_HOUR_OF_DAY: function(data, queryData) {

            queryData.fields.push(
                " events_main.`HOUR_OF_DAY` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DAY: function(data, queryData) {

            queryData.fields.push(
                " events_main.`DAY` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_DAY_OF_WEEK: function(data, queryData) {

            queryData.fields.push(
                " events_main.`DAY_OF_WEEK` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_WEEK: function(data, queryData) {

            queryData.fields.push(
                " floor((events_main.`DAY`-4) / 7) * 7 + 4 as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_MONTH: function(data, queryData) {

            queryData.fields.push(
                " events_main.`MONTH` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_MONTH_OF_YEAR: function(data, queryData) {

            queryData.fields.push(
                " events_main.`MONTH_OF_YEAR` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_YEAR: function(data, queryData) {

            queryData.fields.push(
                " events_main.`YEAR` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                " `" + data.as + "` "
            );

            return queryData;
        },


        COLUMN_DIRECTION: function(data, queryData) {

            queryData.fields.push(
                " events_main.`DIRECTION` as `" + data.as + "` "
            );

            return queryData;
        },

        /*Custom fields*/
        COLUMN_TOTAL_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " count(events_main.ID) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_LOCAL_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.EVENT_LOCAL) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_LOCAL_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.EVENT_LOCAL) * 100 / count(events_main.ID) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_INCOMING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.EVENT_INCOMING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_INCOMING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.EVENT_INCOMING) * 100 / count(events_main.ID) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_OUTGOING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.EVENT_OUTGOING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_OUTGOING_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.EVENT_OUTGOING) * 100 / count(events_main.ID) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.EVENT_ANSWERED) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_ANSWERED_WITHIN_XX_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(case when events_main.RING < "+User.sla_period+" then events_main.EVENT_ANSWERED end) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_ANSWERED_OVER_XX_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(case when events_main.RING >= "+User.sla_period+" then events_main.EVENT_ANSWERED end) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.EVENT_ANSWERED) * 100 / count(events_main.ID) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_NOT_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.EVENT_UNANSWERED) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_PERCENT_NOT_ANSWERED_EVENTS: function(data, queryData) {

            queryData.fields.push(
                " (SUM(events_main.EVENT_UNANSWERED) * 100 / count(events_main.ID) ) as `" + data.as + "` "
            );

            return queryData;
        },

        COLUMN_TOTAL_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.DURATION) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.DURATION),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_LOCAL_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.DURATION_LOCAL) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_LOCAL_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.DURATION_LOCAL),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_INCOMING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.DURATION_INCOMING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_INCOMING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.DURATION_INCOMING),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_OUTGOING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.DURATION_OUTGOING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_OUTGOING_DURATION: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.DURATION_OUTGOING),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_RING: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.RING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_RING: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.RING),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },


        COLUMN_TOTAL_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.BILL) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.BILL),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_LOCAL_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.BILL_LOCAL) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_LOCAL_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.BILL_LOCAL),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_INCOMING_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.BILL_INCOMING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_INCOMING_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.BILL_INCOMING),2) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_OUTGOING_BILL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.BILL_OUTGOING) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_OUTGOING_BILL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.BILL_OUTGOING),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_COST: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.COST) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_COST: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.COST),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_COST_CALL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`COST_CALL`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_COST_CALL: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.`COST_CALL`),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_COST_FAX: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`COST_FAX`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_AVG_COST_FAX: function(data, queryData) {

            queryData.fields.push(
                " ROUND(AVG(events_main.`COST_FAX`),2) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_EVENTS_CALL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`EVENT_CALL`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_EVENTS_VOICEMAIL: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`EVENT_VOICEMAIL`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },

        COLUMN_TOTAL_EVENTS_FAX: function(data, queryData) {

            queryData.fields.push(
                " SUM(events_main.`EVENT_FAX`) as `" + data.as + "` "
            );

            queryData.assocForSuperSelect.push(
                "SUM(`" + data.as + "`) as `" + data.as + "`"
            );

            return queryData;
        },


        /*Limit decorator*/
        limits: function(data, queryData) {
            queryData.limit = " LIMIT " + data.offset + ", " + data.limit + " ";
            return queryData;
        },

        /*Grouping decorators */
        GROUP_BY_CUSTOM_EVENTS_FIELD: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_HOUR: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`HOUR` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_HOUR_OF_DAY: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`HOUR_OF_DAY` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_DAY: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`DAY` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_DAY_OF_WEEK: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`DAY_OF_WEEK` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_WEEK: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " floor((events_main.`DAY`-4) / 7) * 7 + 4 as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_MONTH: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`MONTH` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_MONTH_OF_YEAR: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`MONTH_OF_YEAR` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_YEAR: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " events_main.`YEAR` as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_SERVICE: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`TYPE`,'' ) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_DIRECTION: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`DIRECTION`,'' ) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_TRUNK: function(data, queryData) {
            // var tableName = 'LEFT JOIN `TRUNKS` as sys_trunks ON sys_trunks.ID = events_main.`TRUNK`';
            // queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "`, `" + data.as + "_NAME` "
            );

            queryData.fields.push(
                " ifnull(events_main.`TRUNK`, 0 ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(events_main.`TRUNK_NAME`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_PBX: function(data, queryData) {
            // var tableName = 'LEFT JOIN `PBXES` as sys_pbxes ON sys_pbxes.ID = events_main.`PBX`';
            // queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "`, `" + data.as + "_NAME` "
            );

            queryData.fields.push(
                " ifnull(events_main.`PBX`, 0 ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(events_main.`PBX_NAME`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_TAG: function(data, queryData) {
            queryData.joins.push('LEFT JOIN `EVENTS_TAGS` as sys_events_tags ON sys_events_tags.EVENT_ID = events_main.ID');
            queryData.joins.push('LEFT JOIN `TAGS` as sys_tags ON sys_tags.ID = sys_events_tags.TAG_ID');

            queryData.grouping.push(
                " `" + data.as + "`, `" + data.as + "_NAME` "
            );

            queryData.fields.push(
                "  ifnull( sys_tags.`ID`,'' ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull( sys_tags.`NAME`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_COUNTRY: function(data, queryData) {
            // var tableName = 'LEFT JOIN `COUNTRIES` as sys_country ON sys_country.ID = events_main.COUNTRY';
            // queryData.joins.push(tableName);

            queryData.grouping.push(
                " `" + data.as + "`, `" + data.as + "_NAME` "
            );

            queryData.fields.push(
                " ifnull(events_main.`COUNTRY`, 0 ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(events_main.`COUNTRY_NAME`,'' ) as `" + data.as + "_NAME` "
            );

            queryData.assocForSuperSelect.push("`" + data.as + "`");
            queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");

            return queryData;
        },

        GROUP_BY_CLASS: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "` "
            );

            queryData.fields.push(
                " ifnull(events_main.`CLASS`,'' ) as `" + data.as + "` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            return queryData;
        },

        GROUP_BY_COMPANY: function(data, queryData) {
            queryData.grouping.push(
                " `" + data.as + "`, `" + data.as + "_NAME` "
            );

            queryData.fields.push(
                " ifnull(events_main.`FROM_GROUP`, 0 ) as `" + data.as + "` "
            );
            queryData.fields.push(
                " ifnull(events_main.`FROM_GROUP_NAME`,'' ) as `" + data.as + "_NAME` "
            );
            queryData.assocForSuperSelect.push("`" + data.as + "`");

            queryData.where.push("events_main.`FROM_GROUP_TYPE` = 2");

            return queryData;
        },

        GROUP_BY_USER: function(data, queryData, getTotalRows, getSumRows, userType) {
            var filteredByFromUser = Boolean(data.filters.destination['intToIntFrom'].length
                || data.filters.destination['extToIntFrom'].length
                || data.filters.destination['intToExtFrom'].length);

            var filteredByToUser = Boolean(data.filters.destination['intToIntTo'].length
                || data.filters.destination['extToIntTo'].length
                || data.filters.destination['intToExtTo'].length);

            if (userType == undefined) {
                userType = 1;
            }

            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as + "`");
                queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");
            }


            var whereFrom = queryData.where.slice(0)
            whereFrom.push("events_main.`FROM_USER_TYPE` = " + userType);

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( ";

            var populateSubquery = function (field) {
                if (field === 'TO') {
                    var whereTo = queryData.where.slice(0)
                    whereTo.push("events_main.`TO_USER_TYPE` = " + userType);
                }

                resultQuery += getSumRows !== true ? "SELECT events_main.`" + field + "_USER` as `" + data.as + "`, events_main.`" + field + "_NAME` as `" + data.as + "_NAME` " : "SELECT ";
                if (queryData.fields.length > 0) {
                    resultQuery += getSumRows !== true ? ',' : '';
                    resultQuery += " " + queryData.fields.join(',');
                }
                resultQuery += " FROM " + queryData.from.join(',');
                // resultQuery += " INNER JOIN `USERS` as sys_usr ON sys_usr.ID = events_main.`FROM_USER` ";
                resultQuery += " " + queryData.joins.join(' ');
                resultQuery += " WHERE " + whereFrom.join(' AND ');
                if (getSumRows !== true) {
                    resultQuery += " GROUP BY `" + data.as + "`, `" + data.as + "_NAME` ";
                }
                if (queryData.grouping.length > 0) {
                    resultQuery += " , " + queryData.grouping.join(',') + " ";
                }
            }

            if (filteredByToUser === filteredByFromUser) {
                populateSubquery('FROM');
                resultQuery += " UNION ALL ";
                populateSubquery('TO');
            } else if (filteredByFromUser) {
                populateSubquery('FROM');
            } else if (filteredByToUser) {
                populateSubquery('TO');
            }

            resultQuery += " ) " + this.tmpTableName() + ' ';

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "`, `" + data.as + "_NAME` ";
                if (queryData.grouping.length > 0) {
                    resultQuery += " , " + queryData.grouping.join(',') + " ";
                }
            }

            if (getTotalRows === true) {
                resultQuery += ' ) as ' + this.tmpTableName() + ' ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },

        GROUP_BY_CLIENT: function(data, queryData, getTotalRows, getSumRows) {
            return this.decorators.GROUP_BY_USER.call(this, data, queryData, getTotalRows, getSumRows, 2);
        },

        GROUP_BY_GROUP: function(data, queryData, getTotalRows, getSumRows) {
            var filteredByFromUser = Boolean(data.filters.destination['intToIntFrom'].length
                || data.filters.destination['extToIntFrom'].length
                || data.filters.destination['intToExtFrom'].length);

            var filteredByToUser = Boolean(data.filters.destination['intToIntTo'].length
                || data.filters.destination['extToIntTo'].length
                || data.filters.destination['intToExtTo'].length);

            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as + "`");
                queryData.assocForSuperSelect.push("`" + data.as + "_NAME`");
            }

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( ";

            var populateSubquery = function (field) {
                resultQuery += getSumRows !== true ? " SELECT events_main.`" + field + "` as `" + data.as + "`, events_main.`" + field + "_NAME` as `" + data.as + "_NAME` " : " SELECT ";
                if (queryData.fields.length > 0) {
                    resultQuery += getSumRows !== true ? "," : "";
                    resultQuery += " " + queryData.fields.join(',');
                }
                resultQuery += " FROM " + queryData.from.join(',');
                resultQuery += " " + queryData.joins.join(' ');
                resultQuery += " WHERE " + queryData.where.join(' AND ');
                if (getSumRows !== true) {
                    resultQuery += " GROUP BY `" + data.as + "`, `" + data.as + "_NAME` ";
                }
                if (queryData.grouping.length > 0) {
                    resultQuery += " , " + queryData.grouping.join(',') + " ";
                }
            }

            if (filteredByToUser === filteredByFromUser) {
                populateSubquery('FROM_GROUP');
                resultQuery += " UNION ALL ";
                populateSubquery('TO_GROUP');
            } else if (filteredByFromUser) {
                populateSubquery('FROM_GROUP');
            } else if (filteredByToUser) {
                populateSubquery('TO_GROUP');
            }

            resultQuery += " ) " + this.tmpTableName() + ' ';

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "`, `" + data.as + "_NAME` ";
            }

            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ' + this.tmpTableName() + ' ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },
        GROUP_BY_NUMBER: function(data, queryData, getTotalRows, getSumRows) {
            var filteredByFromUser = Boolean(data.filters.destination['intToIntFrom'].length
                || data.filters.destination['extToIntFrom'].length
                || data.filters.destination['intToExtFrom'].length);

            var filteredByToUser = Boolean(data.filters.destination['intToIntTo'].length
                || data.filters.destination['extToIntTo'].length
                || data.filters.destination['intToExtTo'].length);

            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as + "`");
            }

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( ";

            var populateSubquery = function (field) {
                resultQuery += getSumRows !== true ? " SELECT events_main.`" + field + "` as `" + data.as + "` " : "SELECT ";
                if (queryData.fields.length > 0) {
                    resultQuery += getSumRows !== true ? "," : "";
                    resultQuery += " " + queryData.fields.join(',');
                }
                resultQuery += " FROM " + queryData.from.join(',');
                resultQuery += " " + queryData.joins.join(' ');
                resultQuery += " WHERE " + queryData.where.join(' AND ');
                if (getSumRows !== true) {
                    resultQuery += " GROUP BY `" + data.as + "` ";
                }
                if (queryData.grouping.length > 0) {
                    resultQuery += " , " + queryData.grouping.join(',') + " ";
                }
            }

            if (filteredByToUser === filteredByFromUser) {
                populateSubquery('FROM');
                resultQuery += " UNION ALL ";
                populateSubquery('TO');
            } else if (filteredByFromUser) {
                populateSubquery('FROM');
            } else if (filteredByToUser) {
                populateSubquery('TO');
            }

            resultQuery += " ) " + this.tmpTableName() + ' ';

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as + "` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ' + this.tmpTableName() + ' ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;

        },

        GROUP_BY_COMMUNICATION_OF_USERS: function(data, queryData, getTotalRows, getSumRows) {
            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "_NAME`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "_NAME`");
            }
            queryData.where.push("events_main.`FROM_USER_TYPE` = 1");
            queryData.where.push("events_main.`TO_USER_TYPE` = 1");


            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`FROM_USER` as `" + data.as['from'] + "`, events_main.`TO_USER` as `" + data.as['to'] + "`, "
                            + " events_main.FROM_NAME as `" + data.as['from'] + "_NAME`, events_main.TO_NAME as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += " SELECT ";
            }
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            // resultQuery += " INNER JOIN `USERS` as sys_usr1 ON sys_usr1.ID = events_main.`FROM_USER` ";
            // resultQuery += " INNER JOIN `USERS` as sys_usr2 ON sys_usr2.ID = events_main.`TO_USER` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['from'] + "_NAME`, `" + data.as['to'] + "`, `" + data.as['to'] + "_NAME` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ALL ";

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`TO_USER` as `" + data.as['from'] + "`, events_main.`FROM_USER` as `" + data.as['to'] + "`, "
                            + " events_main.TO_NAME as `" + data.as['from'] + "_NAME`, events_main.FROM_NAME as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += " SELECT ";
            }
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            // resultQuery += " INNER JOIN `USERS` as sys_usr1 ON sys_usr1.ID = events_main.`TO_USER` ";
            // resultQuery += " INNER JOIN `USERS` as sys_usr2 ON sys_usr2.ID = events_main.`FROM_USER` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['from'] + "_NAME`, `" + data.as['to'] + "`, `" + data.as['to'] + "_NAME` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) " + this.tmpTableName() + ' ';

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['from'] + "_NAME`, `" + data.as['to'] + "`, `" + data.as['to'] + "_NAME` ";
            }

            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ' + this.tmpTableName() + ' ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },

        GROUP_BY_COMMUNICATION_OF_GROUPS: function(data, queryData, getTotalRows, getSumRows) {
            if (getSumRows !== true) {
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "`");
                queryData.assocForSuperSelect.push("`" + data.as['from'] + "_NAME`");
                queryData.assocForSuperSelect.push("`" + data.as['to'] + "_NAME`");
            }

            var resultQuery = '';
            if (getTotalRows === true) {
                resultQuery += ' SELECT COUNT(*) as TOTAL_ROWS FROM ( ';
            }

            resultQuery += " SELECT  " + queryData.assocForSuperSelect.join(',') + " FROM ( "

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`FROM_GROUP` as `" + data.as['from'] + "`, events_main.`TO_GROUP` as `" + data.as['to'] + "`, ";
                resultQuery += " events_main.FROM_GROUP_NAME as `" + data.as['from'] + "_NAME`, events_main.TO_GROUP_NAME as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += "SELECT ";
            }

            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            // resultQuery += " INNER JOIN `GROUPS` as sys_grp1 ON sys_grp1.ID = events_main.`FROM_GROUP` ";
            // resultQuery += " INNER JOIN `GROUPS` as sys_grp2 ON sys_grp2.ID = events_main.`TO_GROUP` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['from'] + "_NAME`, `" + data.as['to'] + "`, `" + data.as['to'] + "_NAME` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " UNION ALL ";

            if (getSumRows !== true) {
                resultQuery += " SELECT events_main.`TO_GROUP` as `" + data.as['from'] + "`, events_main.`FROM_GROUP` as `" + data.as['to'] + "`, ";
                resultQuery += " events_main.TO_GROUP_NAME as `" + data.as['from'] + "_NAME`, events_main.FROM_GROUP_NAME as `" + data.as['to'] + "_NAME` ";
            } else {
                resultQuery += "SELECT ";
            }
            if (queryData.fields.length > 0) {
                resultQuery += getSumRows !== true ? "," : "";
                resultQuery += " " + queryData.fields.join(',');
            }
            resultQuery += " FROM " + queryData.from.join(',');
            // resultQuery += " INNER JOIN `GROUPS` as sys_grp1 ON sys_grp1.ID = events_main.`TO_GROUP` ";
            // resultQuery += " INNER JOIN `GROUPS` as sys_grp2 ON sys_grp2.ID = events_main.`FROM_GROUP` ";
            resultQuery += " " + queryData.joins.join(' ');
            resultQuery += " WHERE " + queryData.where.join(' AND ');
            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['from'] + "_NAME`, `" + data.as['to'] + "`, `" + data.as['to'] + "_NAME` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            resultQuery += " ) " + this.tmpTableName() + ' ';

            if (getSumRows !== true) {
                resultQuery += " GROUP BY `" + data.as['from'] + "`, `" + data.as['from'] + "_NAME`, `" + data.as['to'] + "`, `" + data.as['to'] + "_NAME` ";
            }
            if (queryData.grouping.length > 0) {
                resultQuery += " , " + queryData.grouping.join(',') + " ";
            }

            if (getTotalRows === true) {
                resultQuery += ' ) ' + this.tmpTableName() + ' ';
            }

            if (queryData.order !== '') {
                resultQuery += " " + queryData.order + " ";
            }

            if (queryData.limit !== '') {
                resultQuery += " " + queryData.limit + " ";
            }

            return resultQuery;
        },

        /*Sort decorators */
        sort: function(data, queryData) {
            if (Ext.isArray(data)) {
                var orders = new Array();

                for (var i=0; i < data.length; i++) {
                    if (data[i]['field'] === 'ID') {
                        orders.push("events_main.`ID` " + data[i]['dest'] + " ");
                    } else {
                        orders.push("`" + data[i]['field'] + "` " + data[i]['dest'] +" ");
                    }
                }

                queryData.order = " ORDER BY " + orders.join(', ') +" ";
            } else if (Ext.isObject(data)) {
                queryData.order = " ORDER BY `" + data['field'] + "` " + data['dest'] +" ";
            }

            return queryData;
        }

    }





});
