String.prototype.decodeEscapeSequence = function() {
  try {
    return decodeURIComponent(this.replace(/[0-9a-f]{2}/gi, '%$&'));
  } catch(e){
    var decoded = '';
    //at least part of the string will be decoded
    if (this) {
    	var splited = this.match(/.{2}/g);
        if (splited && splited.length>0){
            for (var i = 0; i<splited.length; i++){
		try{
		   decoded += decodeURIComponent('%'+splited[i]+'');
                } catch(e){}
            }
        }
    }
    return decoded;
  }
};

Ext.define('App.storage.Synchronizer', {
    singleton: true,
    requires: ['App.storage.Storage'],
    mixins: {
        observable: 'Ext.util.Observable'
    },
    contactsLimit: 1000,
    backendEventsLimit: 3000,

    constructor: function() {
        this.storage = App.storage.Storage;
        this.proxy = App.proxy.Synchronization;

        this.addEvents(
            'syncUsers',
            'syncUsersChange',
            'syncUsersComplete',
            'syncBackend',
            'syncBackendComplete',
            'syncExtra',
            'syncExtraComplete',
            'syncEvents',
            'syncEventsAuto',
            'syncEventsChange',
            'syncEventsComplete',
            'syncTerminate',
            'syncTerminated',

            'connectionError', // Should be fired when connection problem occurs
            'connectionRestored', // Should be fired when connection was restored
            'exceptionError'   // Should be fires when exception occurs
        );
        this.on('syncUsers', this.syncUsers, this);
        this.on('syncExtra', this.syncExtra, this);
        this.on('syncBackend', this.syncBackEnd, this);
        this.on('syncEvents', this.syncEvents, this);
        this.on('syncEventsAuto', this.syncEventsAuto, this);
        this.on('syncTerminate', this.syncTerminate, this);

        this.on('connectionError', this.onConnectionError, this);

        this.callParent(arguments);
    },

    isWorking: function() {
        return this.synchronizing;
    },

    // Function for Users sync
    syncUsers: function() {
        if (this.synchronizing) {
            return;
        }

        this.synchronizing = true;
        this.syncUsersIgnoreTimes = 0;
        this.syncUsersUpdateStep = null;
        this.syncUsersUpdateDate = null;
        this._startUserSyncTime = Date.now();

        this.proxy.getUsersSettings(this.onSyncUsersSettingsFetched, this);
    },

    isFirstTimeUsersSunc: function(){
        return this.syncUsersUpdateDate == '2000-01-01 00:00';
    },

    /**
     * Removes users from database that was deleted in WMS but exist in local cache.
     * Should be called only in syncUsers procedure sequence.
     */
    syncRemovedUsers: function() {
        this.proxy.getUsersIdentifiers(this.onSyncRemovedUsersFetchedIdentifiers, this);
    },

    getContacts: function (params, success, failure, scope) {
        params = params || {};
        params.start = parseInt(params.start) || 0;
        params.count = this.contactsLimit;
        if (params.start>0)
            params['withTotal'] = 'false';

        //Put filter to fetch just PBX users, not contacts in phonebooks
        //because our query for removing has filter  ... "SELECT `id` FROM `USERS` WHERE `type` = 1";
        //look in getUsersIdentifiers function
        for (var id in this._phonebooksMapping ){
            if (App.storage.Synchronizer._phonebooksMapping.hasOwnProperty(id)){
                if (App.storage.Synchronizer._phonebooksMapping[id] == 3){
                    params['filter[phonebook_id]'] = id;
                }
            }
        }

        Ext.Ajax.request({
            url: '../../api/v1/Contacts/',
            sort: 'id',
            timeout: 120*1000,
            method: 'GET',
            params: params,
            success: success,
            failure: failure,
            scope: scope
        });
    },
    onSyncRemovedUsersFetchedIdentifiers: function(result) {
        this.syncUsersLocalIdentifiers = result;

        var total = 0,
            contacts = [],
            params = { fields : 'id' },
            success = function (response) {
                var data = Ext.JSON.decode(response.responseText);

                if (!(data && data.type && data.type == 'result')) {
                    failure.apply(this, arguments);
                    return;
                }

                var result = data.result || {},
                    records = result.records || [];

                total = total || parseInt(result.total) || 0;
                records.forEach(function (record) { contacts.push(record.id); });

                // Fire change event
                if (total) {
                    var left = total - contacts.length;
                    var time = this.getEventsTimeLeft(left, 0.4/*8*/, 1000);
                    var percent  = this.getEventsLoadPercent(total, left);

                    this.fireEvent('syncUsersChange', percent, time);
                }

                if (records.length /*&& contacts.length < total*/) {
                    this.getContacts(Ext.Object.merge(params, { start: contacts.length }), success, failure, this);
                } else {
                    this.onSyncRemovedUsersFetchedIdentifiersSuccess(contacts);
                }
            },
            failure = this.onServerUsersFetchedFailure;

        this.getContacts(params, success, failure, this);
        // Retrieve all identifiers of users from server (e.g. colleagues).
        /*Ext.Ajax.request({
            url: '../../api/v1/Contacts/',
            method: 'GET',
            params: {
                fields: 'id'
            },
            success: this.onSyncRemovedUsersFetchedIdentifiersSuccess,
            failure: this.onServerUsersFetchedFailure,
            scope: this
        });*/
    },

    onSyncRemovedUsersFetchedIdentifiersSuccess: function(serverIdentifiers) {
        var localIdentifiers = this.syncUsersLocalIdentifiers;
        var mismatches = new Array();
        this.syncUsersLocalIdentifiers = null;

        for (var i=0; i < localIdentifiers.length; i++) {
            var present = false;
            for (var j = 0; j< serverIdentifiers.length; j++){
                if (localIdentifiers[i] == serverIdentifiers[j]){
                    present = true;
                }
            }
            if (!present){
                mismatches.push(localIdentifiers[i]);
            }
        }

        if (mismatches.length > 0) {
            console.log('Remove local users with Ids: ',mismatches);
            this.proxy.removeUsers(mismatches,function(){
                this.synchronizing = false;
                this.fireEvent('syncUsersComplete');
            },this);
        } else {
            this.synchronizing = false;
            this.fireEvent('syncUsersComplete');
        }
        console.log('SyncUsers Time ',((Date.now() - this._startUserSyncTime)/1000) + ' sec (after removing)');
    },

    /**
     *
     */
    onSyncUsersSettingsFetched: function(result) {
        for (var i in result) {
            if (result[i]['KEY'] === 'LAST_USR_SYNC') {
                this.syncUsersUpdateDate = result[i]['VALUE'];
            } else if (result[i]['KEY'] === 'LAST_USR_SYNC_PART') {
                this.syncUsersUpdateStep = parseInt(result[i]['VALUE'], 10);
            }
        }

        this.fetchServerUsersByPart(this.syncUsersUpdateDate, this.syncUsersUpdateStep);
    },

    /**
     * Fetches users part from a server
     */
    fetchServerUsersByPart: function() {
        var fetchContacts = function () {
            var params = {
                fields: 'id,name,extension,mobile,fax,office,home,home_mobile,phone,organization,phonebook_id',
                sort: 'id',
                start: this.syncUsersUpdateStep * this.contactsLimit,
                count: this.contactsLimit
            };
            if (!this.isFirstTimeUsersSunc()){
                params['filter[last_update][from]'] = this.syncUsersUpdateDate;
            }

            if (this._userToFetchTotal > 0)
                params['withTotal'] = 'false';

            console.log('fetchServerUsersByPart',params);


            Ext.Ajax.request({
                url: '../../api/v1/Contacts/',
                timeout: 120*1000,
                method: 'GET',
                params: params,
                success: this.onServerUsersFetchedSuccess,
                failure: this.onServerUsersFetchedFailure,
                scope: this
            });
        }.bind(this);

        if (!this._phonebooksMapping) {
            Ext.Ajax.request({
                url: '../../api/v1/Phonebooks/',
                method: 'GET',
                params: {
                    fields: 'id,type',
                    count: 200
                },
                success: function (response) {
                    var data = Ext.JSON.decode(response.responseText);

                    if (!(data && data.type && data.type == 'result')) {
                        this.fireEvent('exceptionError', response.responseText);
                    }

                    this._phonebooksMapping = {};
                    data.result.records.forEach(function (el) {
                        this._phonebooksMapping[el['id']] = el['type'];
                    }.bind(this));

                    fetchContacts();
                },
                failure: function (response) {
                    this.fireEvent('exceptionError', response.responseText);
                },
                scope: this
            });
        } else {
            fetchContacts();
        }
    },

    /**
     * Fires when users was successfully fetched from server, we should insert them to database, and repeat request
     * for another users part.
     */
    onServerUsersFetchedSuccess: function(response) {
        var self = this, i;
        var user, users = new Array();
        var contacts, data;

        try {
            if (JSON && JSON.parse){
                data = JSON.parse(response.responseText);
            } else {
                data = Ext.JSON.decode(response.responseText);
            }
        } catch(e){
            console.error('Ext.JSON.decode cannot decode string!');
        }

        // Verification for invalid data, we should repeat request after 2 seconds
        if (!(data && data.type && data.type == 'result')) {
            this.syncUsersIgnoreTimes++;

            if (this.syncUsersIgnoreTimes > 5) {
                this.fireEvent('exceptionError', response.responseText);
            } else {
                setTimeout(function() { self.fetchServerUsersByPart.call(self); }, 2000);
            }

            return;
        }

        if (data.result.total)
            this._userToFetchTotal = data.result.total;

        contacts = data.result.records;

        // Verify if server returns empty contacts
        if (contacts.length === 0) {
            this._userToFetchTotal = 0;
            this.onSyncUsersCompleted();
            return;
        }

        for (i=0; i < contacts.length; i++) {
            var contact = contacts[i];

            user = Object.create(null);
            user['id'] = contact.id; // id
            user['name'] = contact.name; // name
            user['type'] = this._phonebooksMapping[contact.phonebook_id] == 3 ? 1 : 2; // type
            user['number'] = contact.extension; // number
            user['mobile_number'] = contact.mobile; // mobile_number
            user['fax_number'] = contact.fax; // fax_number

            user['office'] = contact.office;
            user['home'] = contact.home;
            user['home_mobile'] = contact.home_mobile;
            user['phone'] = contact.phone;

            user['organization'] = contact.organization; // organization
            users.push(user);
        }

        // Fire change event
        if (this._userToFetchTotal) {
            var left = this._userToFetchTotal - this.syncUsersUpdateStep * this.contactsLimit;
            var time = this.getEventsTimeLeft(left, 0.4/*8*/, 1000);
            var percent  = this.getEventsLoadPercent(this._userToFetchTotal, left);

            this.fireEvent('syncUsersChange', percent, time);
        }

        this.storage.pushUsers(users, this.isFirstTimeUsersSunc(), this.onSyncUsersPushed, this);

        if (this.connProblems) {
            this.connProblems = false;
            this.fireEvent('connectionRestored');
        }
    },

    /**
     * Fires when connection problems was found while fetching users
     */
    onServerUsersFetchedFailure: function() {
        this.fireEvent('connectionError', this.onServerUsersFetchedFailureRetry, this);
    },

    /**
     * Fires when we should try to fetch users after a connection problem
     */
    onServerUsersFetchedFailureRetry: function() {
        this.fetchServerUsersByPart();
    },

    /**
     * Fires when Applet has been inserted users to database, we should repeat request to server for another part of users
     */
    onSyncUsersPushed: function() {
        this.syncUsersUpdateStep++;
        // Fetch another part of users after set users part will be updated
        this.proxy.setUserSettingsPart(this.syncUsersUpdateStep, Ext.bind(this.fetchServerUsersByPart,this));
    },

    /**
     * Fires when synchronization was completed successfylly, we should save update step, date and fire complete event.
     */
    onSyncUsersCompleted: function() {
        console.log('SyncUsers Time ',((Date.now() - this._startUserSyncTime)/1000) + ' sec (without removing)');
        //Set part 0 for next sync
        this.syncUsersUpdateStep = 0;
        this.proxy.setUserSettingsPart(this.syncUsersUpdateStep,Ext.bind(function(){
            var isCheckForRemovedNeeded = !this.isFirstTimeUsersSunc();
            //Set currect date as last update date
            this.syncUsersUpdateDate = Ext.Date.format(get_current_date(), 'Y-m-d H:i');
            this.proxy.setUserSettingsLastUpdateDate(this.syncUsersUpdateDate, Ext.bind(function(){
                if (isCheckForRemovedNeeded){
                    this.syncRemovedUsers();
                } else {
                    //Not check for deleted users for the first time
                    this.synchronizing = false;
                    this.fireEvent('syncUsersComplete');
                }
            } ,this));
        },this));

    },

    // Function for Extra synch

    syncExtra: function() {
        if (this.synchronizing) {
            return;
        }

        this.synchronizing = true;
        this.fetchServerExtra();
    },

    fetchServerExtra: function(necessary) {
        Ext.Ajax.request({
            url: '../../api/v1/CDRView/ExtraProperties/',
            method: 'get',
            success: this.onExtraFetchedSuccess,
            failure: this.onExtraFetchedFailure,
            scope: this
        });
    },

    /**
     * Fires when request was successfull, we should process response for exceptions.
     */
    onExtraFetchedSuccess: function(response) {

        var result = Ext.decode(response.responseText).result;

        this.storage.pushExtra(result, this.onExtraPushed, this);
        // this.storage.pushExtra(text, this.onExtraPushed, this);
        // this.process(response.responseText, this.onExtraDecoded, this.onExtraRetry)
    },

    /**
     * Fires when request was failure, another than 200 http code was returned.
     */
    onExtraFetchedFailure: function(response) {
        this.fireEvent('connectionError', this.onExtraRetry, this);
    },

    /**
     * Fires when request is normal, we should repeat request to a server.
     * TODO: Using that logic
     */
    onExtraDecoded: function(result, text) {

    },

    /**
     * Fires when request has been skipped, we should repeat request to a server.
     * TODO: Using that logic
     */
    onExtraRetry: function(necessary) {
        this.fetchServerExtra(necessary);
    },

    /**
     * Fires when applet has been successfylly stored extra to database
     */
    onExtraPushed: function() {
        this.synchronizing = false;
        this.fireEvent('syncExtraComplete');
    },

    // Synch Backend methods

    /**
     * Sync Local and Server backend, if they are different we should truncate events and update backend property in database.
     */
    syncBackEnd: function() {
        if (this.synchronizing) {
            return;
        }

        this.synchronizing = true;
        this.proxy.readBackend(this.onLocalBackEndFetched, this);
    },

    /**
     * Fires when database has been fetched a backend property from database.
     */
    onLocalBackEndFetched: function(backend) {
        this.backendName = backend;
        this.fetchServerBackEnd(backend);
    },

    /**
     * Makes a request to a server to fetch backend.
     */
    fetchServerBackEnd: function(necessary) {
        Ext.Ajax.request({
            url: '../../api/v1/CDRView/InfoProperties/',
            method: 'get',
            success: this.onBackEndFetched,
            failure: this.onBackEndFetchedFailure,
            scope: this
        });
    },

    /**
     * Fires when request was successfull, we should process response for exceptions.
     */
    onBackEndFetched: function(response) {
        var result = Ext.encode(Ext.decode(response.responseText).result);
        this.process(result, this.onBackEndDecoded, this.onBackEndRetry, this)
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond.
     */
    onBackEndFetchedFailure: function() {
        this.fireEvent('connectionError', this.onBackEndRetry, this);
    },

    /**
     * Fires when response is valid.
     */
    onBackEndDecoded: function(result, text) {
        this.backendMinId = result['min_id'];
        this.backendMaxId = result['max_id'];
        this.backendMinDate = result['min_date'];
        this.backendMaxDate = result['max_date'];
        this.backendEventsLimit = result['events_limit'];

        if (this.backendName != result['backend']) {
            // Server and Local Backend is different, we should update Local Backend.
            this.backendName = result['backend'];
            this.proxy.updateBackend(this.backendName, this.onServerBackendUpdated, this);
            return;
        }

        this.synchronizing = false;
        this.fireEvent('syncBackendComplete');
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onBackEndRetry: function(necessary) {
        this.fetchServerBackEnd(necessary);
    },

    /**
     * Fires when backend has been successfully stored in database.
     */
    onServerBackendUpdated: function() {
        this.synchronizing = false;
        this.fireEvent('syncBackendComplete');
    },

    // Misc events methods

    getEventsStatistics: function(callback, scope){
        scope = scope || this;
        var me = this;

        var controller = function(stats) {
            var i, j, curMinId, curMaxId,
                curMinDate, curMaxDate,
                statistics = new Array();
            for (var i=0; i < stats.length; i++) {
                statistics.push( { id: parseInt(stats[i]['ID']), date: stats[i]['DATE'], min_id: parseInt(stats[i]['MIN_ID']), max_id: parseInt(stats[i]['MAX_ID']), complete:  parseInt(stats[i]['COMPLETE']) } );
            }

            // -------
            var range = me.getEventsExcludeRanges(statistics);

            for(i=0; i < range.length; i++) {
                curMinId = range[i][0];
                curMaxId = range[i][1];

                curMinDate = ''; curMaxDate = '';
                for(j=0; j < statistics.length; j++) {
                    if(statistics[j]['min_id'] == curMinId){
                        curMinDate = statistics[j]['date'];
                    }
                    if(statistics[j]['max_id'] ==  curMaxId){
                        curMaxDate = statistics[j]['date'];
                    }
                }
                range[i] = {
                    minId: curMinId,
                    minDate: curMinDate,
                    maxId: curMaxId,
                    maxDate: curMaxDate
                };
            }

            Ext.callback(callback, scope, [range]);
        };

        this.proxy.readAllStatistics(controller, scope);
    },

    // Events methods

    /**
     * Synchronize events from server for specified period.
     */
    syncEvents: function(minDate, maxDate, ignoreCurrDay) {
        if (this.synchronizing) {
            return;
        }

        this.synchronizing = true;
        this.terminated = false;
        this.syncEventsAutoFlag = false;
        this.syncEventsMinId = false;
        this.syncEventsMaxId = false;
        this.syncEventsMinDate = minDate;
        this.syncEventsMaxDate = maxDate;
        this.syncEventsSuccessRequests = 0;
        this.syncEventsSkippedRequests = 0;
        this.syncEventsFailureRequests = 0;
        this.syncEventsDelay = false;
        this.syncEventsTotalWant = false;
        this.syncEventsDays = new Array();
        this.syncEventsIgnoreCurrDay = ignoreCurrDay;
        this.syncEventsNecessary =  false;

        var i = false;
        var day = Ext.Date.parse(minDate, "Y-m-d");
        var date = minDate;

        while (true) {
            if (i) {
                day = Ext.Date.add(day, Ext.Date.DAY, 1);
                date = Ext.Date.format(day, "Y-m-d");
            } else {
                i = true;
            }

            this.syncEventsDays.push(date);

            if (date == maxDate) {
                break;
            }
        }

        this.syncEventsPartly();
    },

    /**
     * Synchronize new events from server (if they has been added since last sync).
     */
    syncEventsAuto: function() {
        if (this.synchronizing) {
            return;
        }

        this.synchronizing = true;
        this.terminated = false;
        this.syncEventsAutoFlag = true;
        this.syncEventsMinId = false;
        this.syncEventsMaxId = false;
        this.syncEventsMinDate = false;
        this.syncEventsMaxDate = false;
        this.syncEventsDelay = false;
        this.syncEventsNecessary =  false;

        //
        this.syncEventsPartly();
    },

    /**
     * Synchronize events by part.
     */
    syncEventsPartly: function() {
        var minDate = this.syncEventsMinDate;
        var maxDate = this.syncEventsMaxDate;

        if (this.syncEventsAutoFlag) {
            this.proxy.readLastStatisticsRow(this.onLocalStatisticsMaxIdFetched, this);
        } else {
            this.proxy.readStatistics(minDate, maxDate, this.onLocalStatisticsFetched, this);
        }
    },

    /**
     * Fires when local statistics was fetched from database, we should generate a params for request.
     */
    onLocalStatisticsFetched: function(stats) {
        var self = this;
        var i, c;
        var statistics = [];
        var minId = this.syncEventsMinId;
        var maxId = this.syncEventsMaxId;
        var minDate = this.syncEventsMinDate;
        var maxDate = this.syncEventsMaxDate;

        // Concatenate ranges of ids for exclusion
        for (i=0; i < stats.length; i++) {
            statistics.push( { id: parseInt(stats[i]['ID']), date: stats[i]['DATE'], min_id: parseInt(stats[i]['MIN_ID']), max_id: parseInt(stats[i]['MAX_ID']), complete:  parseInt(stats[i]['COMPLETE']) } );
        }

        var exclude = this.getEventsExcludeRanges(statistics);

        // Prepare maximum id for period that available in database
        var maxExcludeId = 0;

        for (i=0; i < exclude.length; i++) {
            if (exclude[i][1] > maxExcludeId) {
                maxExcludeId = exclude[i][1];
            }
        }

        // Verify statistics by days
        var complete = false;
        var days = this.syncEventsDays;
        var currDay = get_current_day();

        for (i=0; i < days.length; i++) {
            complete = false;

            for (c=0; c < statistics.length; c++) {
                if (statistics[c]['date'] == days[i]) {
                    if (statistics[c]['complete'] == 1) {
                        complete = true;
                    }

                    if (statistics[c]['date'] == currDay && this.syncEventsIgnoreCurrDay == false) {
                        complete = false;
                    }

                    break;
                }
            }

            if (!complete) {
                break;
            }
        }

        if (complete) {
            this.synchronizing = false;
            this.fireEvent('syncEventsComplete');
            return;
        }

        var params = {};
        var filter = {};
        params['count'] = this.backendEventsLimit;
        params['sort'] = 'rowid';
        params['dir'] = 'ASC';
        var left = false;

        if (minId > 0 && maxId > 0) {
            params['filter[id][from]'] = minId;
            params['filter[id][to]'] = maxId;

            left = this.getEventsTotalLeft(minId, maxId, exclude);

            if (left == 0) {
                this.storage.pushEvents({events:[], minDate:minDate, maxDate:maxDate, minId:minId, maxId:maxId}, this.onEventsPushedComplete, this);
                return;
            }
        } else {
            params['filter[date][from]'] = minDate;
            params['filter[date][to]'] = maxDate;
        }
        for (i=0; i < exclude.length; i++) {
            params['filter[exclude]['+ i +'][from]'] = exclude[i][0];
            params['filter[exclude]['+ i +'][to]']   = exclude[i][1];
        }


        if (this.terminated) {
            console.log('sync events', 'terminated');

            this.synchronizing = false;

            if (this.connProblems) {
                this.connProblems = false;
                this.fireEvent('connectionRestored');
            }

            console.log('synchronizer', 'syncTerminated');
            this.fireEvent('syncTerminated');
            return;
        }

        // Fire change event
        if (left) {
            if (this.syncEventsTotalWant === false) {
                this.syncEventsTotalWant = left;
            }

            var time = this.getEventsTimeLeft(left, 1.3/*8*/, 1000);
            var percent  = this.getEventsLoadPercent(this.syncEventsTotalWant, left);

            this.fireEvent('syncEventsChange', percent, time);
        }

        // Wait n seconds to next request
        if (this.syncEventsDelay > 0) {
            setTimeout(function() { self.fetchServerEvents.call(self, params); }, this.syncEventsDelay);

            this.syncEventsDelay = false;
            return;
        }

        this.fetchServerEvents(params);
    },

    /**
     * Fires when max id of local statistics was fetched from database, we should generate a params for request.
     */
    onLocalStatisticsMaxIdFetched: function(stat) {
        if (stat === undefined){
            this.terminated = true;
        } else {
            // Set a valid data
            this.syncEventsMinId = parseInt(stat['MAX_ID']);
            this.syncEventsMinDate = stat['DATE'];
        }

        if (this.terminated) {
            this.synchronizing = false;

            if (this.connProblems) {
                this.connProblems = false;
                this.fireEvent('connectionRestored');
            }

            this.fireEvent('syncTerminated');
            return;
        }

        // Make a query
        var params = new Object();

        params['count'] = this.backendEventsLimit;
        params['sort'] = 'rowid';
        params['dir'] = 'ASC';
        params['filter[id][from]'] = this.syncEventsMinId + 1;
        params['filter[id][to]'] = this.syncEventsMinId + 1001;
        params['backend'] = this.backendName;
        delete params['filter[exclude]'];

        this.fetchServerEvents(params);
    },


    /**
     * Fetches an events from a server.
     */
    fetchServerEvents: function(params) {
        if (isRemoteMode()) {
            return;
        }

        console.log('fetch events, filter = ', params);

        Ext.Ajax.request({
            url: '../../api/v1/CDRView/CallsHistory/',
            method: 'get',
            params: params,
            success: this.onEventsFetched,
            failure: this.onEventsFetchedFailure,
            scope: this
        });
    },

    /**
     * Fires when request was successfull, we should process response for exceptions.
     */
    onEventsFetched: function(response) {
        var result = Ext.encode(Ext.decode(response.responseText).result)
        this.process(result, this.onEventsDecoded, this.onEventsRetry, this)
    },

    /**
     * Fires when response is valid and we should handle a result.
     */
    onEventsDecoded: function(result, text) {
        var self = this;
        var events = result['events'];
        var limit = result['limit'];
        var timeout = parseInt(result['timeout']);
        var minId = this.syncEventsMinId;
        var minDate = this.syncEventsMinDate;
        var maxId = this.syncEventsMaxId;
        var maxDate = this.syncEventsMaxDate;

        if (result['min_id'] != undefined && result['max_id'] != undefined) {
            this.syncEventsMinId = parseInt(result['min_id']);
            this.syncEventsMaxId = parseInt(result['max_id']);
        }

        if (!minId || !maxId) {
            minId = 0;
            maxId = 0;
        }


        if (this.syncEventsAutoFlag) {
            if (events.length == 0) {
                console.log('syncEventsComplete 3');

                this.synchronizing = false;
                this.fireEvent('syncEventsComplete');
                return;
            }

            this.backendMaxId = parseInt(events[events.length - 1]['id']);
            this.backendMaxDate = events[events.length - 1]['date'];

            minId = this.syncEventsMinId;
            minDate = this.syncEventsMinDate;
            maxId = this.backendMaxId;
            maxDate = get_current_day();
        }

        // Push Events to Database
        this.syncEventsSuccessRequests++;
        this.syncEventsTimeout = timeout;

        console.log('synchronizer push events: ' + events.length);
        if (!events.length)
            this.syncEventsAutoFlag = true;
        this.storage.pushEvents({events:events, minDate:minDate, maxDate:maxDate, minId:minId, maxId:maxId}, this.onEventsPushed, this);
    },

    removeKeysFromEvents: function(events){
        var eventsWithoutKeys = [];
        for (var i=0; i<events.length; i++){
            eventsWithoutKeys.push([
                events[i].id,
                events[i].from,
                events[i].fromChannel,
                events[i].to,
                events[i].toChannel,
                events[i].type,
                events[i]['class'],
                events[i].direction,
                events[i].date,
                events[i].duration,
                events[i].billsec,
                events[i].cost,
                events[i].tags
            ]);
        }
        return eventsWithoutKeys;
    },

    /**
     * Fires when request has been fail, another than 200 http code was respond.
     */
    onEventsFetchedFailure: function() {
        this.fireEvent('connectionError', this.onEventsRetry, this);
    },

    /**
     * Fires when response has try_again_later exception, we should repeat request to a server.
     */
    onEventsRetry: function(necessary) {
        this.syncEventsNecessary = necessary;
        this.syncEventsPartly();
    },

    /**
     * Fires when we successfully stored events to a database.
     */
    onEventsPushed: function() {
        this.syncEventsDelay = this.syncEventsTimeout * 1000;
        this.syncEventsPartly();
    },

    onEventsPushedComplete: function() {
        console.log('SyncEvents Time ',((Date.now() - this._startUserSyncTime)/1000) + ' sec');
        this.synchronizing = false;
        this.fireEvent('syncEventsComplete');
    },

    /**
     * Gets an exclude ranges by a statistics
     */
    getEventsExcludeRanges: function(statistics) {
        var i;
        var min_id, max_id;
        var ranges = [];
        var excludes = [];

        for (i=0; i < statistics.length; i++) {
            min_id = parseInt(statistics[i]['min_id']);
            max_id = parseInt(statistics[i]['max_id']);

            if (min_id > 0 && max_id > 0) {
                ranges.push([min_id, max_id]);
            }
        }

        min_id = false;
        max_id = false;

        for (i=0; i < ranges.length; i++) {
            if (i == 0) {
                if (ranges.length == 1) {
                    excludes.push([ranges[i][0], ranges[i][1]]);
                }

                min_id = ranges[i][0];
                max_id = ranges[i][1];

                continue;
            }

            if (ranges[i][0] != max_id + 1) {
                excludes.push([min_id, max_id]);

                min_id = ranges[i][0];
                max_id = ranges[i][1];
            }

            max_id = ranges[i][1];

            if (ranges.length == i + 1) {
                excludes.push([min_id, max_id]);
            }
        }

        return excludes;
    },

    /**
     * Gets a number of need to load events by exclude ranges and min, max id.
     */
    getEventsTotalLeft: function(minId, maxId, ranges) {
        var i;
        var total = maxId - minId;

        for (i=0; i < ranges.length; i++) {
            total = total - (ranges[i][1] - ranges[i][0]);
        }

        return total;
    },

    /**
     * Gets ~ time to finish synchronization.
     */
    getEventsTimeLeft: function(total, time, limit) {
        var offset = total % limit;
        var requests = 0;

        if (offset > 0) {
            requests = (total + (limit - offset)) / limit;
        } else {
            requests = total / limit;
        }

        var timeleft = requests * time;

        return timeleft;
    },

    /**
     * Gets a percent of loaded events.
     */
    getEventsLoadPercent: function(total, left) {
        // TODO: Use another final logic here.
        var pointer = total / 100;
        var percent = Math.ceil(Math.abs(left - total) / pointer);


        return percent;
    },

    /**
     * Fires when user want's to terminate a synchronization.
     */
    syncTerminate: function() {
        this.terminated = true;
    },

    // Misc methods

    getBackendMinId: function() {
        return this.backendMinId;
    },
    getBackendMaxId: function() {
        return this.backendMaxId;
    },
    getBackendMinDate: function() {
        return this.backendMinDate;
    },
    getBackendMaxDate: function() {
        return this.backendMaxDate;
    },

    /**
     * Fires when connection problem occur.
     */
    onConnectionError: function() {
        this.connProblems = true;
    },

    /**
     * Verifies response and executes a Successs, Retry or nothing when response is skipped or invalid.
     */
    process: function(text, callbackSuccess, callbackRetry, scope) {
        var result;
        var timeout;
        var exception = _t('Unknown error');

        scope = scope || this;

        try {
            result = Ext.decode(text);
        } catch (e) {
            // Server returns an unsupported format
            this.fireEvent('exceptionError', text);
            return;
        }

        timeout = parseInt(result['timeout']);

        if (result.hasOwnProperty('skipped') && parseInt(result['skipped']) === 1) {
            // Request was skipped by a server
            if (result.hasOwnProperty('exception')) {
                exception = result['exception'];
            }

            if (exception == 'TRY_AGAIN_LATER') {
                // Request was correctly skipped by a server and we should wait for a timeout and fire Retry callback
                if (this.attempts === undefined) {
                    this.attempts = 0;
                }

                if (this.attempts >= 5) {
                    setTimeout(function() { callbackRetry.call(scope, true); }, timeout * 1000);
                } else {
                    setTimeout(function() { callbackRetry.call(scope, false); }, timeout * 1000);
                }

                this.attempts++;
                return;
            }

            // Unknow exception we should stop any execution and show an error
            this.fireEvent('exceptionError', exception);
            return;
        }

        if (this.connProblems) {
            this.connProblems = false;
            this.fireEvent('connectionRestored');
        }

        // Normal request
        callbackSuccess.call(this, result, text);
    }
} );
