Ext.define('App.utility.File', {
    singleton: true,
    downloadAsXLSX: function (struct, fileName) {
        this.self._downloadAsXLSX(struct, fileName);
    },
    downloadAsCSV: function (struct, fileName) {
        this.self._downloadAsCSV(struct, fileName);
    },
    faxDataLink: function(record){
        return this.self._faxDataLink(record);
    },
    userDataLink: function(record){
        return this.self._userDataLink(record);
    },
    parseRecordings: function (value) {
        return this.self._parseRecordings(value);
    },
    statics: (function () {

        function datenum(v, date1904) {
            if (date1904)
                v += 1462;
            var epoch = Date.parse(v);
            return (epoch - new Date(Date.UTC(1899, 11, 30))) / (24 * 60 * 60 * 1000);
        }
        function sheet_from_array_of_arrays(data, opts) {
            var ws = {};
            var range = {s: {c: 10000000, r: 10000000}, e: {c: 0, r: 0}};
            for (var R = 0; R != data.length; ++R) {
                for (var C = 0; C != data[R].length; ++C) {
                    if (range.s.r > R)
                        range.s.r = R;
                    if (range.s.c > C)
                        range.s.c = C;
                    if (range.e.r < R)
                        range.e.r = R;
                    if (range.e.c < C)
                        range.e.c = C;
                    var cell = {v: data[R][C]};
                    if (cell.v == null)
                        continue;
                    var cell_ref = XLSX.utils.encode_cell({c: C, r: R});

                    if (typeof cell.v === 'number')
                        cell.t = 'n';
                    else if (typeof cell.v === 'boolean')
                        cell.t = 'b';
                    else if (cell.v instanceof Date) {
                        cell.t = 'n';
                        cell.z = XLSX.SSF._table[14];
                        cell.v = datenum(cell.v);
                    } else
                        cell.t = 's';

                    ws[cell_ref] = cell;
                }
            }
            data = null;
            if (range.s.c < 10000000)
                ws['!ref'] = XLSX.utils.encode_range(range);
            return ws;
        }
        function Workbook() {
            if (!(this instanceof Workbook))
                return new Workbook();
            this.SheetNames = [];
            this.Sheets = {};
        }
        function s2ab(s) {
            var buf = new ArrayBuffer(s.length);
            var view = new Uint8Array(buf);
            for (var i = 0; i != s.length; ++i)
                view[i] = s.charCodeAt(i) & 0xFF;
            return buf;
        }
        function _ConvertToCSV(obj) {
            var separator = ";";
            var div = document.createElement('div');
            var array = [];
            if (obj.headers) {
                array.push(obj.headers);
            }
            if (obj.rows) {
                array = array.concat(obj.rows);
            }
            if (obj.summaries) {
                array.push(obj.summaries);
            }
            var str = '';
            var line = '';

            for (var i = 0; i < array.length; i++) {
                var line = '';
                for (var index in array[i]) {
                    var value = array[i][index] + "";
                    line += '"' + value.replace(/"/g, '""') + '"' + separator;
                }

                line = line.slice(0, -1);
                str += line + '\r\n';
            }
            return str;
        }

        //2015.06.05-10.45.16-111-333-128011243-{n}.wav|1;
        var recordRegExp = /^(\d+)\.(\d+)\.(\d+)-(\d+)\.(\d+)\.(\d+)-([^-]+)-([^-]+)-.*?\|(\d+)$/;
        //2015.06.05/13/49/2015.06.05-13.49.04-111-333-205284820-{n}.wav|1;
        var newRecordRegExp = /^.*?\/(\d+)\.(\d+)\.(\d+)-(\d+)\.(\d+)\.(\d+)-([^-]+)-([^-]+)-.*?\|(\d+)$/;


        return {
            _downloadAsCSV: function (data, fileName) {

                var a = document.createElement('a');
                var mimeType = 'text/csv';
                var content = _ConvertToCSV(data);

                App.utility.File.self.download(content, mimeType, fileName);
            },
            _downloadAsXLSX: function (struct, fileName) {
                var data = [struct.headers].concat(struct.rows).concat([struct.summaries]);
                struct = null;
                var ws_name = "TabSheet1",
                        wb = new Workbook(),
                        ws = sheet_from_array_of_arrays(data);

                /* add worksheet to workbook */
                wb.SheetNames.push(ws_name);
                wb.Sheets[ws_name] = ws;
                var wbout = XLSX.write(wb, {bookType: 'xlsx', bookSST: false, type: 'binary'});
                var content = s2ab(wbout);
                var mimeType = 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet';

                App.utility.File.self.download(content, mimeType, fileName);
            },
            download: function (content, mimeType, fileName) {
                var a = document.createElement('a');

                //var mimeType = 'attachment/file';


                if (navigator.msSaveBlob) { // IE10
                    console.debug('+++++++++++1');
                    return navigator.msSaveBlob(new Blob([content], {type: mimeType}), fileName);
                } else if (URL && URL.createObjectURL && Ext.isSafari) {
                    console.debug('+++++++++++2');
                    var reader = new FileReader();
                    reader.onloadend = function () {
                        var base64Data = reader.result;
                        window.location.href = "data:attachment/file" + base64Data.slice(base64Data.search(/[,;]/));
                    };
                    reader.readAsDataURL(new Blob([content]));
                    return;//For now file name want work in Safari
                    a.href = URL.createObjectURL(new Blob([content], {type: mimeType}));
                    ;
                    a.setAttribute('download', fileName);
                    a.setAttribute('target', '_blank');
                    document.body.appendChild(a);
                    if (window.MouseEvent) {
                        var clk = document.createEvent("MouseEvent");
                        clk.initEvent("click", true, true);
                        a.dispatchEvent(clk);
                    } else {
                        a.click();
                    }
                    setTimeout(function () {
                        document.body.removeChild(a);
                    }, 66);
                    return true;
                } else if (URL && URL.createObjectURL) {
                    console.debug('+++++++++++3');
                    a.href = URL.createObjectURL(new Blob([content], {type: mimeType}));
                    a.setAttribute('download', fileName);
                    document.body.appendChild(a);
                    if (window.MouseEvent) {
                        var clk = document.createEvent("MouseEvent");
                        clk.initEvent("click", true, true);
                        a.dispatchEvent(clk);
                    } else {
                        a.click();
                    }
                    setTimeout(function () {
                        document.body.removeChild(a);
                    }, 66);
                    return true;
                } else if ('download' in a) { //html5 A[download]
                    console.debug('+++++++++++4');
                    a.href = 'data:' + mimeType + ',' + encodeURIComponent(content);
                    a.setAttribute('download', fileName);
                    document.body.appendChild(a);
                    setTimeout(function () {
                        a.click();
                        document.body.removeChild(a);
                    }, 66);
                    return true;
                } else { //do iframe dataURL download (old ch+FF):
                    console.debug('+++++++++++5');
                    var f = document.createElement('iframe');
                    document.body.appendChild(f);
                    f.src = 'data:' + mimeType + ',' + encodeURIComponent(content);

                    setTimeout(function () {
                        document.body.removeChild(f);
                    }, 333);
                    return true;
                }
            },

            _parseRecordings: function(recordStr){
                var records = [];
                var values = (''+recordStr).split(";");
                for (var i=0; i < values.length; i++) {
                    if(values[i]!==''){
                        var filename = values[i].replace(/\|\d+/, '');
                        var m = recordRegExp.exec(values[i]);

                        if (!m){
                            m = newRecordRegExp.exec(values[i]);
                        }

                        if (m && m[7]){
                            records.push({
                                filename: filename,
                                date: new Date(m[1], m[2]-1, m[3], m[4], m[5], m[6]),
                                source: m[7],
                                destination: m[8],
                                files:  m[9]
                            });
                        }
                    }
                }
                return records;
            },
            _userDataLink: function(record){
                var records = [],
                    value = record.get('USERFIELD') ;
                var pbxid = record.get('PBX');
                if (pbxid){
                    pbxid = pbxid.replace(/.*\((\w+)\)/,'$1').trim();
                }

                if (value === null) {
                    value = '';
                }

                if(value.substr(0, 2) == 'R:'){
                    value = value.replace('R:', '');
                    records = App.utility.File.self._parseRecordings(value);
                    if(records && records.length > 0){
                        if(records.length > 1 || records[0].files > 1){
                            return '<a class="w-download-attach w-recording morerecords fa fa-file-audio-o" href="javascript:void(0)" title="'+_t('Download recording')+'" records="'+value+'"></a>';
                        }else if(records.length == 1 && records[0].files == 1){
                            var url = null;
                            if(records[0].filename.indexOf('-{n}') > -1){
                                url = '/spoolview/recordings/'+records[0].filename.replace('{n}', 0);
                            }else{
                                url = '/spoolview/recordings/'+records[0].filename+'-0.wav';
                            }
                            if (pbxid && Serial != pbxid){
                                //Download data from another pbx
                                url = '/collaboration/lua/fileproxy.lua?serial='+pbxid+'&path='+url;
                            }
                            return '<a class="w-download-attach w-recording fa fa-file-audio-o" href="../..' + url + '" target="_blank" title="'+_t('Download recording')+'"></a>';
                        }
                    }
                }
                return '';
            },
            _faxDataLink: function(record){
                var faxdata = record.get('FAXDATA');
                var incoming = record.get('DIRECTION') == 2;
                var pbxid = record.get('PBX');
                if (pbxid){
                    pbxid = pbxid.replace(/.*\((\w+)\)/,'$1').trim();
                }
                if (User.faxAvailable != true) {
                    return '';
                }
                //faxdata = 'OK|202/fax-222-+390733565560-1302881328034504470-1-4.tiff';
                //faxdata = "ERR|F:345/report-345-3453453454-1407328941444897990-14.pdf|3453453454|CHANUNAVA"

                if (faxdata === null) {
                    faxdata = '';
                }

                if (typeof(faxdata) !=' undefined' && faxdata!='') {

                    var pathToFax = '';
                    var folder = incoming ? 'faxin':'faxout';
                    var fax = faxdata.split("|");
                    if(!fax[1] || fax[1].trim() == ''){
                        return '';
                    }
                    if(fax[1].substr(0,2) == 'F:'){
                        pathToFax = '/spoolview/'+folder+'/'+fax[1].substr(2);
                    } else if (fax[0] == 'OK' || fax[0] == 'ERR'){
                        var fax_url = fax[1];

                        var matching = faxdata.match(/\|fax-([0-9]+)-/);

                        if (folder == 'faxout' && matching != null && matching[1] != undefined && matching[1].length > 0) {
                            pathToFax = '/spoolview/'+folder+'/' + matching[1] + '/' + fax_url;
                        } else {
                            matching = fax_url.match(/(tmp\/)/);
                            if (folder == 'faxin' && matching != undefined && matching != null && matching[1] != undefined) {
                                fax_url = fax_url.replace('/tmp/', this.userPhone + '/');
                            }
                            pathToFax = '/spoolview/'+folder+'/'+fax_url;
                        }
                    }
                    if (pathToFax){
                        if (pbxid && Serial != pbxid){
                            //Download fax from another pbx
                            pathToFax = '/collaboration/lua/fileproxy.lua?serial='+pbxid+'&path='+pathToFax;
                        }

                        var urls = '';
                        if(folder == 'faxout'){
                    		var pathToReport = pathToFax.replace("/fax-", '/report-')
                    		urls += '<a class="w-download-attach fa fa-file-pdf-o" href="../..'+ pathToReport +'" target="_blank" title="'+_t('Download fax report')+'"></a>';
                    	}
                        urls += '<a class="w-download-attach fa fa-file-pdf-o" href="../..'+ pathToFax +'" target="_blank" title="'+_t('Download fax')+'"></a>';

                        return urls
                    }

                }

                return '';
            }
        };
    }())

});
