Ext.define('App.utility.NamesFormatter', {
    singleton: true,

    formatSeconds: function(timeleft) {
        var hh = Math.floor(timeleft / 3600);
        if (hh > 0) {
            var mm = Math.floor((timeleft % 3600) / 60);
            var ss = mm > 0 ? (timeleft % 3600) % 60 : timeleft % 3600;
            timeleft = sprintf(_t('%(hours)sh %(minutes)smin %(seconds)ssec '), { hours : hh, minutes : mm, seconds : ss });
        } else {
            mm = Math.floor(timeleft / 60);
            if (mm > 0){
                ss = timeleft % 60;
                timeleft = sprintf(_t('%(minutes)smin %(seconds)ssec '), { minutes : mm, seconds : ss });
            }else{
                timeleft = sprintf(_t('%(seconds)ssec '), { seconds : timeleft });
            }
        }
        return timeleft;
    },

    /**
     * Used in chart
     */
    formatSecondsShort: function(timeleft) {
        var mm = Math.floor(timeleft / 60);
        if (mm > 0){
            return sprintf("%(minutes)sm", { minutes : mm });
        }else{
            return sprintf("%(seconds)ss", { seconds : timeleft });
        }
    },

    formatNumber: function(number) {
        if (!number) {
            number = 0;
        }
        return Ext.util.Format.number(number, '0,000');
    },

    formatFloat: function(number) {
        if (!number) {
            number = 0;
        }
        return Ext.util.Format.number(number, '0,000.00');
    },

    formatWithAccuracyGrade: function(number, grade) {
        if (!number) {
            number = 0;
        }
        grade = parseInt(grade) ? parseInt(grade) : 2;
        return number.toFixed(grade);
    },

    formatGroupDirection: function(value) {
        switch(parseInt(value)) {
            case 1:
                return _t('Unknown');
            case 2:
                return _t('Incoming');
            case 3:
                return _t('Outgoing');
            case 4:
                return _t('Local');
            default:
                return _t('Other');
        }
    },

    formatBill: function(value) {
        if (!value || value === 'other') {
            var secs = 0;
        } else {
            secs = parseInt(value);
        }
        return sprintf('%(time)s', {time: secs});
    },

    formatDirection: function(value, skipTags) {
        switch(parseInt(value)) {
            case 1:
                return _t('Other');
            case 2:
                return skipTags? _t('Inc.') :sprintf('<span style="color:orange;">%(inc)s</span>', { inc : _t('Inc.') });
            case 3:
                return skipTags? _t('Out.') :sprintf('<span style="color:navy;">%(out)s</span>', { out : _t('Out.') });
            case 4:
                return skipTags? _t('Local') :sprintf('<span style="color:green;">%(local)s</span>', { local : _t('Local') });
            default:
                return _t('Other');
        }
    },

    formatService: function(value) {
        switch (parseInt(value)) {
            case 1:
                return _t('Call');
            case 2:
                return _t('Fax');
            case 3:
                return _t('Voicemail');
            default:
                return _t('Unknown');
        }
    },

    formatStatus: function(value, skipTags) {
        if (!value || value === 'other') {
            return skipTags? _t('Unansw.') : sprintf('<span style="color:red;">%(unansw)s</span>', { unansw : _t('Unansw.') });
        } else {
            value = parseInt(value);
            if (value > 0) {
                return skipTags? _t('Answ.') : sprintf('<span style="color:green;">%(answ)s</span>', { answ : _t('Answ.') });
            } else {
                return skipTags? _t('Unansw.') : sprintf('<span style="color:red;">%(unansw)s</span>', { unansw : _t('Unansw.') });
            }
        }
    },

    formatHoursTime: function(secs) {
        if (secs) {
            secs = parseInt(secs);
        } else {
            secs = 0;
        }
        var hours = Math.floor(secs / (60 * 60));
        var divisor_for_minutes = secs % (60 * 60);
        var minutes = Math.floor(divisor_for_minutes / 60);
        var divisor_for_seconds = divisor_for_minutes % 60;
        var seconds = Math.ceil(divisor_for_seconds);

        if (hours < 10) {
            hours = "0" + hours;
        }
        if (minutes < 10) {
            minutes = "0" + minutes;
        }
        if (seconds < 10) {
            seconds = "0" + seconds;
        }
        return hours +':'+ minutes +':'+ seconds;
    },

    formatGroupingByHour: function(value) {
        if (!value || value === 'other') {
            return _t('Other');
        }

        if (!this.timeOffset) {
            this.timeOffset = parseInt(Ext.Date.format(new Date(), "Z"), 10);
        }

        var date = Ext.Date.parse(parseInt(value) * 3600 - this.timeOffset, 'U');

        return Ext.Date.format(date, 'd/m/Y H:i');
    },

    formatGroupingByDay: function(value) {
        if (!value || value === 'other') {
            return _t('Other');
        }
        value = parseInt(value, 10);
        if (!this.timeOffset) {
            this.timeOffset = parseInt(Ext.Date.format(new Date(), "Z"));
        }

        var date = Ext.Date.parse(value * 86400 - this.timeOffset, 'U');

        return Ext.Date.format(date, 'd/m/Y');
    },

    formatHourOfDay: function(value) {
        if (!value || value === 'other') {
            return _t('Other');
        }
        value = parseInt(value, 10);
        if (value < 10) {
            value = "0" + value;
        }
        return value + ":00";
    },

    formatDayOfWeek: function(value) {
        value = parseInt(value, 10);
        var result = _t('Other');
        switch(value) {
            case 0:
                result = _t('Sunday');
                break;
            case 1:
                result = _t('Monday');
                break;
            case 2:
                result = _t('Tuesday');
                break;
            case 3:
                result = _t('Wednesday');
                break;
            case 4:
                result = _t('Thursday');
                break;
            case 5:
                result = _t('Friday');
                break;
            case 6:
                result = _t('Saturday');
                break;
        }
        return result;
    },

    formatGroupingByWeek: function(value) {
        if (!value || value === 'other') {
            return _t('Other');
        }
        value = parseInt(value, 10);
        if (!this.timeOffset) {
            this.timeOffset = parseInt(Ext.Date.format(new Date(), "Z"));
        }

        var dateFrom = Ext.Date.parse(value * 86400 - this.timeOffset, 'U'),
            dateTo = Ext.Date.add(dateFrom, Ext.Date.DAY, 6);

        return Ext.Date.format(dateFrom, 'd/m/Y')+' - '+Ext.Date.format(dateTo, 'd/m/Y');
    },

    formatByMonth: function(value) {
        try {
            value = value.toString();
            if (!value || value === 'other') {
                return _t('Other');
            }
            var year = value.substr(0, 4);
            var month = value.substr(4, 6);
            return month +'/'+ year;
        } catch (e){
            console.error(e, 'for value',value);
        }
    },

    formatMonthOfYear: function(value) {
        value = parseInt(value, 10);
        var result = _t('Other');
        switch(value) {
            case 0:
                result = _t('January');
                break;
            case 1:
                result = _t('February');
                break;
            case 2:
                result = _t('March');
                break;
            case 3:
                result = _t('April');
                break;
            case 4:
                result = _t('May');
                break;
            case 5:
                result = _t('June');
                break;
            case 6:
                result = _t('July');
                break;
            case 7:
                result = _t('August');
                break;
            case 8:
                result = _t('September');
                break;
            case 9:
                result = _t('October');
                break;
            case 10:
                result = _t('November');
                break;
            case 11:
                result = _t('December');
                break;
        }
        return result;
    },

    formatCallClass: function(value) {
        switch (parseInt(value)) {
            case 1:
                return _t('Internal');
            case 2:
                return _t('Local');
            case 3:
                return _t('National');
            case 4:
                return _t('Mobile');
            case 5:
                return _t('Emergency');
            case 6:
                return _t('Free');
            case 7:
                return _t('Premium1');
            case 8:
                return _t('Premium2');
            case 9:
                return _t('Premium3');
            case 10:
                return _t('Premium4');
            case 11:
                return _t('Northamerica');
            case 12:
                return _t('Africa');
            case 13:
                return _t('Europe1');
            case 14:
                return _t('Europe2');
            case 15:
                return _t('Southamerica');
            case 16:
                return _t('Oceania');
            case 17:
                return _t('Russia');
            case 18:
                return _t('Asia1');
            case 19:
                return _t('Asia2');
            default:
                return _t('Other');
        }
    },

    formatText: function(value) {
        var result = value;
        switch (value) {
            case 'TOTAL_EVENTS_CHART_VAL':
                result = _t('Total events');
                break;
            case 'TOTAL_COST_CHART_VAL':
                result = _t('Total cost');
                break;
            case 'TOTAL_BILL_CHART_VAL':
                result = _t('Total talk time');
                break;
            case 'TOTAL_LOCAL_BILL_CHART_VAL':
                result = _t('Total local talk time');
                break;
            case 'TOTAL_INCOMING_BILL_CHART_VAL':
                result = _t('Total incoming talk time');
                break;
            case 'TOTAL_OUTGOING_BILL_CHART_VAL':
                result = _t('Total outgoing talk time');
                break;
            case 'TOTAL_RING_CHART_VAL':
                result = _t('Total ring time');
                break;
            case 'TOTAL_ANSWERED_EVENTS_CHART_VAL':
                result = _t('Total answered');
                break;
            case 'TOTAL_ANSWERED_WITHIN_XX_EVENTS_CHART_VAL':
                result = sprintf(_t('Total answered within %(seconds)s'),{seconds:User.sla_period});
                break;
            case 'TOTAL_ANSWERED_OVER_XX_EVENTS_CHART_VAL':
                result = sprintf(_t('Total answered over %(seconds)s'),{seconds:User.sla_period});
                break;
            case 'TOTAL_NOT_ANSWERED_EVENTS_CHART_VAL':
                result = _t('Total not answered');
                break;
            case 'TOTAL_LOCAL_EVENTS_CHART_VAL':
                result = _t('Total local');
                break;
            case 'TOTAL_INCOMING_EVENTS_CHART_VAL':
                result = _t('Total incoming');
                break;
            case 'TOTAL_OUTGOING_EVENTS_CHART_VAL':
                result = _t('Total outgoing');
                break;
            case 'AVG_COST_CHART_VAL':
                result = _t('Avg. Cost');
                break;
            case 'AVG_BILL_CHART_VAL':
                result = _t('Avg. Talk time');
                break;
            case 'AVG_LOCAL_BILL_CHART_VAL':
                result = _t('Avg. Local Talk time');
                break;
            case 'AVG_INCOMING_BILL_CHART_VAL':
                result = _t('Avg. Inc. Talk time');
                break;
            case 'AVG_OUTGOING_BILL_CHART_VAL':
                result = _t('Avg. Out. Talk time');
                break;
            case 'AVG_RING_CHART_VAL':
                result = _t('Avg. Ring Time');
                break;

            case 'COUNT':
                result = _t('Count');
                break;
            case 'COUNT_BY_SERVICE':
                result = _t('Count by service');
                break;
            case 'COUNT_BY_STATUS':
                result = _t('Count by status');
                break;
            case 'COUNT_BY_DIRECTION':
                result = _t('Count by direction');
                break;
            case 'COUNT_BY_SLA':
                result = _t('Count by SLA');
                break;
            case 'COST':
                result = _t('Cost');
                break;
            case 'COST_BY_SERVICE':
                result = _t('Cost by service');
                break;
            case 'COST_AVG_BY_SERVICE':
                result = _t('Avg. Cost by service');
                break;
            case 'BILL':
                result = _t('Talk time');
                break;
            case 'BILL_BY_DIRECTION':
                result = _t('Talk time by direction');
                break;
            case 'BILL_AVG_BY_DIRECTION':
                result = _t('Avg. talk time by direction');
                break;

            case 'TOTAL_CALL_EVENTS_CHART_VAL':
                result = _t('Call events');
                break;
            case 'TOTAL_FAX_EVENTS_CHART_VAL':
                result = _t('Fax events');
                break;
            case 'TOTAL_VOICEMAIL_EVENTS_CHART_VAL':
                result = _t('Voicemail events');
                break;
            case 'TOTAL_CALL_COST_CHART_VAL':
                result = _t('Call cost');
                break;
            case 'TOTAL_FAX_COST_CHART_VAL':
                result = _t('Fax cost');
                break;
            case 'AVG_CALL_COST_CHART_VAL':
                result = _t('Avg. Call Cost');
                break;
            case 'AVG_FAX_COST_CHART_VAL':
                result = _t('Avg. Fax Cost');
                break;
        }
        return result;
    }

});
