Ext.define('App.view.filter.Editor', {
    extend: 'Ext.Container',
    hidden: true,
    hideMode: 'display',
    floating: true,
    cls: 'w-editor',
    initComponent: function() {
        this.renderTo = Ext.getBody();
        this.renderTpl = [
            '<div class="w-editor-arrow"></div>',
            '<div class="w-editor-body">',
                '<div class="x-editor-top-right"><div class="x-editor-top-left"><div class="x-editor-top"></div></div></div>',
                '<div class="x-editor-center-right"><div class="x-editor-center-left"><div class="x-editor-center">',
                    '<div class="x-editor-collapse"></div>',
                    '<div class="x-editor-title">{title}</div>',
                    '<tpl if="description">',
                        '<div class="x-editor-description">{description}</div>',
                    '</tpl>',
                    '<div class="x-clear"></div>',
                    '<div class="x-editor-form"></div>',
                    sprintf('<div class="x-editor-condition" style="display: none;"><span class="label">%(condRes)s </span><span class="x-editor-condition-result"><span class="any">%(any)s</span></span></div>', { condRes : '', any : _t('Any')}),
                '</div></div></div>',
                '<div class="x-editor-bottom-right"><div class="x-editor-bottom-left"><div class="x-editor-bottom"></div></div></div>',
            '</div>'
        ];
        
        this.renderSelectors = {
            arrowEl: 'div.w-editor-arrow',
            collapseEl: 'div.x-editor-collapse',
            formEl: 'div.x-editor-form',
            conditionEl: 'span.x-editor-condition-result'
        }

        this.renderData = {
            title: this.title,
            description: this.description
        }
        
        this.callParent(arguments);
        
        // Events
        this.addEvents('change');
        this.on('change', this.onChange, this);
        
        // ...
        this.observer = App.view.filter.Observer;
        this.observer.push(this);
        this.state = this.getDefaultState();


    },


    onRender: function() {
        this.callParent(arguments);
    },
    onChange: function(state) {
        this.conditionEl.update(this.getDescription(state));
    },
    afterRender : function() {
        this.callParent(arguments);
        this.collapseEl.on('click', this.onCollapse, this);
    },
    onRenderContent: function() {
        this.contentRendered = true;
    },
    onCollapse: function() {
        this.hide();
    },
    focus: function() {
        this.observer.focus(this);
        this.show();
    },
    show: function() {
        if (this.isVisible()) {
            return false;
        }
        if (!this.targetParent) {
            this.targetParent = this.target.findParentByType('form');
        }
        
        Ext.EventManager.onWindowResize(this.alignEditor, this);
        
        this.alignEditor();
        this.callParent(arguments);
        
        if (!this.contentRendered) {
            this.onRenderContent(this.getState());
        }

        if(!this.KeyMapOnEsc) {
            this.KeyMapOnEsc = new Ext.util.KeyMap(Ext.getBody(), [{
               key: Ext.EventObject.ESC,
               defaultEventAction: 'preventDefault',
               scope: this,
               fn: this.hide
            }]);
        } else {
            this.KeyMapOnEsc.enable();
        }

        return true;
    },
    hide: function() {
        Ext.EventManager.removeResizeListener(this.alignEditor, this);
        
        this.callParent(arguments);

        this.KeyMapOnEsc.disable();
        this.fireEvent('afterhide');
    },
    alignEditor: function() {
        var trgPos = this.target.getPosition();
        var trgLeft = trgPos[0];
        var trgTop = trgPos[1];
        var trgWidth = this.target.getWidth();
        var trgHeight = this.target.getHeight();
        var trgBottom = trgTop + trgHeight;
        var trgCenter = trgLeft + (trgWidth / 2);
        var prnPos = this.targetParent.getPosition();
        var prnLeft = prnPos[0];
        var prnTop = prnPos[1];
        var prnWidth = this.targetParent.getEl().getWidth();
        var prnOffset = 20;
        var scopeLeft = prnLeft + (prnOffset / 2);
        var scopeRight = scopeLeft + prnWidth - prnOffset;
        var selfTop = trgBottom;
        var selfWidth = this.width;
        var selfLeft = trgCenter - (selfWidth / 2 );

        if( selfLeft + selfWidth > scopeRight ) {
            selfLeft = trgLeft + trgWidth - this.width;
        }
        if( selfLeft < scopeLeft ) {
            selfLeft = scopeLeft;
        }
        
        var arrWidth = 19;
        var arrLeft = parseInt(trgCenter - selfLeft - (arrWidth / 2));
        
        this.arrowEl.setLeft(arrLeft);
        this.setPosition(selfLeft, selfTop, false);
    },
    getTarget: function() {
        return this.target;
    },
    getState: function() {
        if (!this.state) {
            this.state = this.getDefaultState();
        }
        
        return this.state;
    },
    getAnyDescription: function() {
        return '';
    },
    getAnyFieldDescription: function() {
        return _t('Any');
    }
} );
