Ext.define('App.view.filter.MinMaxEditor', {
    extend: 'App.view.filter.Editor',
    //title: 'Filter by Duration',
    width: 400,
    mark: 'Duration',
    //description: 'Please select a range of event durations from a moment when extension was dialed not depending on was call answered or not.<br /> Example: 5 < duration <= 10 means calls which are longer than 5 seconds and less or equal to 10 seconds (10 is not included).',
    defaultFieldConfig: {
        allowDecimals: false,
        step: 1
    },
    onRenderContent: function(state) {
        var self = this;

        this.leftField = Ext.create('App.view.filter.minMaxEditor.Spinner', Ext.applyIf({
            width: 90,
            value: state['leftValue']
        }, this.defaultFieldConfig));

        this.rightField = Ext.create('App.view.filter.minMaxEditor.Spinner', Ext.applyIf({
            width: 90,
            value: state['rightValue']
        }, this.defaultFieldConfig));

        this.containerEl = Ext.create('Ext.container.Container', {
            renderTo: this.formEl,
            layout: 'column',
            items: [
                {
                    xtype: 'container',
                    columnWidth: 0.5,
                    layout: 'column',
                    items: [
                        this.leftField,
                        {
                            xtype: 'container',
                            columnWidth: 1,
                            style: 'line-height: 22px; text-align: center; font-size: 14px;',
                            html: '<'
                        }
                    ]
                },
                {
                    xtype: 'container',
                    width: 90,
                    items: [
                        {
                            xtype: 'container',
                            cls: 'w-dst-mark-left',
                            renderTpl: [
                                '<div class="w-dst-mark-right">',
                                    '<div class="w-dst-mark">{title}</div>',
                                '</div>'
                            ],
                            renderData: {
                                title: this.mark
                            }
                        }
                    ]
                },
                {
                    xtype: 'container',
                    columnWidth: 0.5,
                    layout: 'column',
                    items: [
                        {
                            xtype: 'container',
                            columnWidth: 1,
                            style: 'line-height: 22px; text-align: center; font-size: 14px;',
                            html: '<'
                        },
                        this.rightField
                    ]
                }
            ]
        });

        this.leftField.addListener('change', this.onLeftFieldChange, this);
        this.rightField.addListener('change', this.onRightFieldChange, this);

        this.callParent(arguments);
    },
    onLeftFieldChange: function(field, value) {
        console.log("onLeftFieldChange", value);
        if (this.ignoreStateEvents) {
            return;
        }
        if (value == null) {
            this.state['leftEnabled'] = false;
            this.state['leftValue'] = null;
        } else {
            this.state['leftEnabled'] = true;
            this.state['leftValue'] = value;
        }

        this.fireEvent('change', this.state);
    },
    onRightFieldChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }
        if (value == null) {
            this.state['rightEnabled'] = false;
            this.state['rightValue'] = null;
        } else {
            this.state['rightEnabled'] = true;
            this.state['rightValue'] = value;
        }

        this.fireEvent('change', this.state);
    },
    setState: function(s) {
        console.log("MinMaxEditor set state: ", s);

        this.state = s;
        this.conditionEl.update(this.getDescription(s));

        if (!this.contentRendered) {
            return;
        }

        this.ignoreStateEvents = true;

        this.leftField.setValue(s['leftValue']);
        this.rightField.setValue(s['rightValue']);

        this.ignoreStateEvents = false;
    },
    getDefaultState: function() {
        var s = {
            'leftEnabled': false,
            'leftValue': null,
            'rightValue': null,
            'rightEnabled': false
        }

        return s;
    }
} );