Ext.define('App.view.filter.field.combo.list.SelectionView', {
    extend: 'Ext.grid.Panel',
    displayField: 'name',
    style: 'border: 0px;',
    scroll: 'vertical',
    verticalScrollerType: 'filterselectionviewscroller',
    invalidateScrollerOnRefresh: false,
    hideHeaders: true,
    loadMask: true,
    border: 0,
    bodyBorder: 0,
    enableColumnHide: false,
    enableColumnMove: false,
    enableColumnResize: false,
    initComponent: function() {
        this.addEvents('search', 'afterSearch');
        this.on('search', this.onSearch, this);
        this.callParent(arguments);
    },
    afterRender: function() {
        this.callParent(arguments);
        this.emptyEl = Ext.create('Ext.container.Container', {
            hidden: true,
            hideMode: 'display',
            height: 50,
            dock: 'top',
            style: 'padding: 5px 10px; text-align: justify; background: #FFE57C;'
        });
        this.addDocked(this.emptyEl);
    },
    onSearch: function(query) {
        console.log('dest selection view', 'onSearch');
        
        // mask
        this.store.mask();

        // clear store
        this.clearStore();
        
        // clear view
        this.clearView();
        
        // create filters
        var filters = new Array();
        
        if (query) {
            filters.push({property: 'name', value: query});
        }
        
        var decoded = this.store.decodeFilters(filters);
        
        // setup filters
        var i;
        for(i = 0; i < decoded.length; i++) {
            this.store.filters.replace(decoded[i]);
        }
        
        // fetch data
        this.store.on('guaranteedrange', this.onSearched, this, {single: true, query: query});
        this.store.guaranteeRange(0, this.store.pageSize - 1);
    },
    
    /**
     * Fires when data fetched from database
     */
    onSearched: function(range, start, end, options) {
        console.log('dest selection view', 'onSearched');
        
        var scroller = this.verticalScroller;
        var scrollEl = scroller.scrollEl;
        
        scrollEl.removeAllListeners();
        scroller.mon(scrollEl, 'scroll', scroller.onElScroll, scroller);
        
        if (this.store.getTotalCount() == 0) {
            this.emptyEl.update('');
            //this.emptyEl.show();
            this.hideVerticalScroller();
        } else {
            this.emptyEl.hide();
        }
        
        this.invalidateScroller();
        this.fireEvent('afterSearch');
    },
    
    /**
     * 
     */
    clearStore: function() {
        console.log('dest selection view', 'clearStore');
        
        this.store.removeAll();
        this.store.filters.clear();
        this.store.prefetchData.clear();
        this.store.guaranteedStart = undefined;
        this.store.guaranteedEnd = undefined;
        this.store.requestEnd = undefined;
        this.store.requestSeed = undefined;
        this.store.requestStart = undefined;
        this.store.totalCount = this.store.pageSize;
    },
    /**
     * 
     */
    clearView: function() {
        console.log('dest selection view', 'clearView');
        
        var view = this.getTableView();
        
        if (view.el) {
            view.el.dom.scrollTop = 0;
        }

        if (!this.verticalScroller.panel) {
            this.verticalScroller.panel = this.verticalScroller.getPanel();
        }
        
        window.asdfsadf = this.verticalScroller;
        
        if (this.verticalScroller.scrollEl) {
            this.verticalScroller.scrollEl.dom.scrollTop = 0;
        }
        
        return true;
    },
    
    clearEmptyView: function() {
        if (this.emptyEl) {
            this.emptyEl.hide();
        }
    },
    
    getTableView: function() {
        return this.down('tableview');
    }
} );
