Ext.define('App.view.metric.Picker', {
    extend: 'Ext.view.View',
    hidden: true,
    hideMode: 'display',
    floating: true,
    cls: 'w-viewport-picker',
    dirty: false,
    allowList: false,
    initComponent: function() {
        var me = this;
        this.groupColumns = [
            { id: '1', name: 'COUNT', text: _t('Count'), group: 'count', active: true },
            { id: '2', name: 'COUNT_BY_SERVICE', text: _t('Count by service'), group: 'count' },
            { id: '3', name: 'COUNT_BY_STATUS', text: _t('Count by status'), group: 'count' },
            { id: '4', name: 'COUNT_BY_DIRECTION', text: _t('Count by direction'), group: 'count' },
            { id: '11', name: 'COUNT_BY_SLA', text: _t('Count by SLA'), group: 'count' },
            { id: '5', name: 'COST', text: _t('Cost'), group: 'cost', active: true },
            { id: '6', name: 'COST_BY_SERVICE', text: _t('Cost by service'), group: 'cost' },
            { id: '7', name: 'COST_AVG_BY_SERVICE', text: _t('Avg. Cost by service') },
            { id: '8', name: 'BILL', text: _t('Talk time'), group: 'bill', active: true },
            { id: '9', name: 'BILL_BY_DIRECTION', text: _t('Talk time by direction'), group: 'bill' },
            { id: '10', name: 'BILL_AVG_BY_DIRECTION', text: _t('Avg. talk time by direction') }
        ];

        this.addEvents('expand', 'collapse', 'change');

        this.store = Ext.create('Ext.data.JsonStore', {
            fields: [
                { name: 'id' },
                { name: 'name' },
                { name: 'text' },
                { name: 'group' },
                { name: 'active', defaultValue: false }
            ],
            data: this.groupColumns
        });

        this.tpl = new Ext.XTemplate(
            '<div class="w-viewport-picker-tl"><div class="w-viewport-picker-tr"><div class="w-viewport-picker-tt">',
                '<div class="w-viewport-picker-arrow">', 
                    _t('Metric'), 
                '</div>',
            '</div></div></div>',
            '<div class="w-viewport-picker-cl"><div class="w-viewport-picker-cr">',
                '<div class="x-box-scroller-top w-viewport-picker-list-scroll-top">',
                    '<div class="x-menu-scroll-top"></div>',
                '</div>',
                '<div class="w-viewport-picker-list">',

                /* Error state */
                '<div style="padding: 1px 5px 7px 8px;" <tpl if="this.checkGrouping()">class="helper-hide"</tpl>>',
                    '<span>', 
                    '</span>',
                '</div>',

                /* Original state */
                '<div <tpl if="!this.checkGrouping()">class="helper-hide"</tpl> >',
                    '<div class="w-viewport-picker-list-scroll">',
                    '<div class="w-viewport-picker-list-scroll-wrap">',
                    '<tpl for=".">',
                        '<tpl if="id == 1"><div class="items-splitter">',
                            _t('Count of events'),
                        '</div></tpl>',
                        '<tpl if="id == 5"><div class="items-splitter">',
                            _t('Cost of events'),
                        '</div></tpl>',
                        '<tpl if="id == 8"><div class="items-splitter">',
                            _t('Talk time of events'),
                        '</div></tpl>',
                        '<div class="item <tpl if="active == true">item-active</tpl> <tpl if="permanent == true">item-permanent</tpl>">',
                            '<div class="item-left"><div class="item-right">',
                                '<span class="item-name">{text}</span>',
                            '</div></div>',
                        '</div>',
                    '</tpl>',
                    '</div>',
                    '</div>',
                '</div>',

                '</div>',
                '<div class="x-box-scroller-bottom w-viewport-picker-list-scroll-bottom">',
                    '<div class="x-menu-scroll-bottom"></div>',
                '</div>',
                '<tpl if="this.checkGrouping()">',
                    '<div class="w-viewer-table-columns-clear"><a class="w-viewer-table-columns-clear-text">', 
                        _t('Reset'), 
                    '</a></div>',
                '</tpl>',
            '</div></div>',
            '<div class="w-viewport-picker-bl"><div class="w-viewport-picker-br"><div class="w-viewport-picker-bb"></div></div></div>',
            {
                checkGrouping: function() {
                    return me.allowList;
                }
            }
        );

        this.itemSelector = 'div.item';
        this.callParent(arguments);
        this.on('refresh', this.onRefresh, this);

        this.bind = true;
    },

    onRefresh: function(){
        this.bindView();
    },

    setState: function(state) {
        this.store.each(function(rec) {
            if(state.indexOf(rec.get('name')) != -1 ){
                rec.set('active', true);
                rec.set('enable', true);
                if(this.getMode() == 'SINGLE') {
                    this.setLastActiveRecord(rec)
                }
            } else {
                rec.set('active', false);
                rec.set('enable', true);
            }
        }, this);
    },

    getState: function() {
        var results = new Array(), columns;

        columns = this.store.queryBy(function(rec) {
            return rec.get('active') == true;
        });
        columns.each(function(rec) {
            results.push(rec.get('name'));
        });

        return results;
    },

    /**
     *
     * @param mode String ('SINGLE' | 'MULTI')
     */
    setMode: function(mode){
        this.mode = mode;
    },

    getMode: function(){
        return this.mode;
    },

    setLastActiveRecord: function(rec){
        this.lastActiveRecord = rec;
    },

    getLastActiveRecordName: function() {
        if(this.lastActiveRecord) {
            return this.lastActiveRecord.get('name');
        } else {
            return this.groupColumns[0].name;
        }
    },

    onHeadElClick: function() {
        this.hide();
    },

    onResetElClick: function() {
        var i, r, rx, first = false;
        var len = this.store.getCount();

        for (i=0; i < len; i++) {
            r = this.store.getAt(i);
            rx = this.groupColumns[i];

            if (rx['active']) {
                if(this.getMode() == 'SINGLE' && first) {
                    r.set('active', false);
                    continue;
                }
                first = true;
                r.set('active', true);
            } else {
                r.set('active', false);
            }
        }

        this.hide();
    },

    onGroupElClick: function() {
        this.hide();
        this.fireEvent('grpElClick');
    },

    bindView: function() {
        this.headEl = this.getEl().down('div.w-viewport-picker-arrow');
        this.headEl.on('click', this.onHeadElClick, this);

        this.resetEl = this.getEl().down('a.w-viewer-table-columns-clear-text');
        if (this.resetEl) {
            this.resetEl.on('click', this.onResetElClick, this);
        }

        this.groupEl = this.getEl().down('a.w-viewer-grouping-on');
        if (this.groupEl) {
            this.groupEl.on('click', this.onGroupElClick, this);
        }

        this.listEl = this.getEl().down('div.w-viewport-picker-list');
        this.listScrollTopEl = this.getEl().down('div.w-viewport-picker-list-scroll-top');
        this.listScrollTopEl.setVisibilityMode(Ext.core.Element.DISPLAY);
        this.listScrollTopEl.on('click', this.onListScrollTopElClick, this);
        this.listScrollBottomEl = this.getEl().down('div.w-viewport-picker-list-scroll-bottom');
        this.listScrollBottomEl.setVisibilityMode(Ext.core.Element.DISPLAY);
        this.listScrollBottomEl.on('click', this.onListScrollBottomElClick, this);
        this.listScrollEl = this.getEl().down('div.w-viewport-picker-list-scroll');
        this.listScrollWrapEl = this.getEl().down('div.w-viewport-picker-list-scroll-wrap');
    },

    onListScrollTopElClick: function() {
        this.listScrollEl.scroll('top', 26, true)
    },

    onListScrollBottomElClick: function() {
        this.listScrollEl.scroll('bottom', 26, true)
    },


    show: function() {
        // Show component
        if (this.isVisible()) {
            return;
        }

        Ext.getDoc().on('mousedown', this.collapseWheel, this);
        Ext.EventManager.onWindowResize(this.align, this);

        this.callParent(arguments);

        if (this.bind)  {
            this.bindView();
            this.bind = false;
        }

        this.shownState = this.getState();
        this.align();

        if(!this.KeyMapOnEsc) {
            this.KeyMapOnEsc = new Ext.util.KeyMap(Ext.getBody(), [{
               key: Ext.EventObject.ESC,
               defaultEventAction: 'preventDefault',
               scope: this,
               fn: this.hide
            }]);
        } else {
            this.KeyMapOnEsc.enable();
        }
    },
    hide: function() {
        Ext.getDoc().un('mousedown', this.collapseWheel, this);
        Ext.EventManager.removeResizeListener(this.align, this);

        this.callParent(arguments);

        // Calculating changes
        var i, missmatch = false;

        this.currentState = this.getState();

        if (this.shownState.length != this.currentState.length) {
            missmatch = true;
        }

        if (!missmatch) {
            for (i=0; i < this.currentState.length; i++) {
                if (this.shownState[i] != this.currentState[i]) {
                    missmatch = true;
                    break;
                }
            }
        }

        if (missmatch) {
            this.fireEvent('change', this.currentState);
        }
        this.KeyMapOnEsc.disable();
    },
    align: function() {
        var trgLeft = this.target.getLeft();
            trgLeft = trgLeft - 10;
        var trgTop = this.target.getTop();
            trgTop = trgTop - 5;
        var trgWidth = this.target.getWidth();

        if (trgWidth < 160) {
            trgWidth = 160;
        }

        if(this.allowList) {
            this.listEl.setHeight('auto');
            this.listScrollEl.setHeight('auto');
        }

        var offset = Ext.getBody().getHeight() - (trgTop + this.getHeight());
        var height = Ext.getBody().getHeight() - trgTop;

        if (offset < 0) {
            var offsetBt = this.getHeight() - this.listEl.getHeight();
            var heightList = height - offsetBt;

            this.listScrollTopEl.show();
            this.listScrollBottomEl.show();

            this.listEl.setHeight(heightList - this.listScrollTopEl.getHeight() - this.listScrollBottomEl.getHeight());
            this.listScrollEl.setHeight(this.listEl.getHeight());
            this.listScrollEl.setWidth(trgWidth - 4 + 20);
            this.listScrollWrapEl.setWidth(trgWidth - 4);
        } else {
            this.listScrollTopEl.hide();
            this.listScrollBottomEl.hide();
        }

        this.setWidth(trgWidth);
        this.setPosition(trgLeft, trgTop, false);
    },

    collapseWheel: function(event, element) {
        if (!event.within(this.el, false, true)) {
            this.hide();
        }
    },
    onItemClick: function(record, item, index, event) {
        // Handling item
        var eventTarget = Ext.get(event.target);
        var group = record.get('group');
        var i, r, isActive;
        var len = this.store.getCount();

        if(this.getMode() == 'SINGLE') {
            if (record.get('active')) {
                return;
            } else {
                for (var j=0; j < this.store.getCount(); j++) {
                    var r = this.store.getAt(j);
                    if ( r.get('active')) {
                        r.set('active', false);
                    }
                }
                record.set('active', true);
                this.setLastActiveRecord(record);
                this.hide();
                return;
            }
        }

        // Special behavior for groups and users groupings
        if (group) {
            if (record.get('active')) {
                for (i=0; i < len; i++) {
                    r = this.store.getAt(i);
                    if (r.get('active') && r != record) {
                        isActive = true;
                        break;
                    }
                }
                if (isActive) {
                    record.set('active', false);
                }
                return;
            }

            for (i=0; i < len; i++) {
                r = this.store.getAt(i);

                if (r.get('group') == group && r.get('active')) {
                    r.set('active', false);
                    break;
                }
            }

            record.set('active', true);
            this.setLastActiveRecord(record);
            return;
        }

        // Default behavior (items that can be singly)
        if (record.get('active')) {
            for (i=0; i < len; i++) {
                r = this.store.getAt(i);
                if (r.get('active') && r != record) {
                    isActive = true;
                    break;
                }
            }
            if (isActive) {
                record.set('active', false);
            }
            return;
        }

        record.set('active', true);
        this.setLastActiveRecord(record);
    }
});
