Ext.define('App.view.viewer.ExportDialog', {
    extend: 'Ext.window.Window',
    title: _t('Export datagrid'),
    width: 360,
    modal: true,
    resizable: false,
    closable: false,
    bodyStyle: 'background: transparent; border: 0;',
    constructor: function() {
        this.addEvents('complete', 'terminated');
        this.callParent(arguments);
    },
    initComponent: function() {

        this.addEvents('export', 'open');

        this.xlsRadioField = Ext.create('Ext.form.field.Radio', {
            name: 'exportFormatRadio',
            value: 'xls',
            boxLabel: _t('.xls (Microsoft Excel)')
        });

        this.csvRadioField = Ext.create('Ext.form.field.Radio', {
            name: 'exportFormatRadio',
            value: 'csv',
            checked: true,
            boxLabel: _t('.csv (Comma-separated values)')
        });

        this.formatCt = Ext.create('Ext.container.Container', {
            cls: 'w-sync-description',
            style: 'padding-bottom: 5px;',
            items: [
                {
                    xtype: 'container',
                    padding: '0 0 5 0',
                    html: _t('Please select a format for export:')
                },
                this.xlsRadioField,
                this.csvRadioField
            ]
        });


        this.croppedDescriptionCt = Ext.create('Ext.container.Container', {
            padding: '0 0 5 0'
        });

        this.croppedCt = Ext.create('Ext.container.Container', {
            cls: 'w-sync-description',
            style: 'padding-bottom: 5px;',
            hidden: true,
            items: [
                {
                    xtype: 'container'
                }
            ]
        });

        this.generatedDescriptionCt = Ext.create('Ext.container.Container', {
            padding: '0 0 5 0'
        });

        this.generatedCt = Ext.create('Ext.container.Container', {
            cls: 'w-sync-description',
            style: 'padding-bottom: 5px;',
            hidden: true,
            items: [
                this.generatedDescriptionCt
            ]
        });

        this.exportButton = Ext.create('Ext.button.Button', {
            text: _t('Export')
        });
        this.exportButton.on("click", this.onExportButtonClick, this)

        this.nextButton = Ext.create('Ext.button.Button', {
            text: _t('Next'),
            hidden: true
        });
        this.nextButton.on("click", this.onNextButtonClick, this);

        this.cancelButton = Ext.create('Ext.button.Button', {
            text: _t('Cancel')
        });
        this.cancelButton.on("click", this.onCloseOrCancelButtonClick, this);

        this.openButton = Ext.create('Ext.button.Button', {
            text: _t('Open'),
            hidden: true
        });
        this.openButton.on("click", this.onOpenButtonClick, this);

        this.closeButton = Ext.create('Ext.button.Button', {
            text: _t('Close'),
            hidden: true
        });
        this.closeButton.on("click", this.onCloseOrCancelButtonClick, this);

        this.items = [
            this.croppedCt,
            this.formatCt,
            this.generatedCt
        ];

        this.buttons = [
            this.nextButton,
            this.exportButton,
            this.cancelButton,
            this.openButton,
            this.closeButton
        ]

        this.callParent(arguments);
    },

    showCroppedNotification: function() {
        this.croppedCt.show();
        this.formatCt.hide();
        this.generatedCt.hide();
        this.nextButton.show();
        this.exportButton.hide();
        this.cancelButton.hide();
        this.openButton.hide();
        this.closeButton.show();
        this.show();
    },

    switchToDownloadState: function(path) {
        var separator = "/";

        if (Ext.isWindows) {
            separator = "\\";
        }

        var paths = path.split(separator);
        var file = paths.pop();
        var directory = paths.join(separator);
        var description = sprintf('File <b>"%(file)s"</b> successfully generated and saved in <b>"%(directory)s"</b>, press "Open" to open it immediately', {
            file: file,
            directory: directory
        });

        this.generatedDescriptionCt.getEl().update(description);

        this.exportButton.hide();
        this.cancelButton.hide();
        this.closeButton.show();
        this.openButton.show();

        this.formatCt.hide();
        this.generatedCt.show();
    },
    onExportButtonClick: function() {
        var format = 'xls';

        console.log("xlsRadioField", this.xlsRadioField.getValue());
        console.log("csvRadioField", this.csvRadioField.getValue());
        if (this.csvRadioField.getValue()) {
            format = 'csv';
        }

        this.fireEvent("export", format);
    },
    onNextButtonClick: function() {
        this.croppedCt.hide();
        this.formatCt.show();
        this.generatedCt.hide();
        this.nextButton.hide();
        this.exportButton.show();
        this.cancelButton.show();
        this.openButton.hide();
        this.closeButton.hide();
    },
    onOpenButtonClick: function() {
        this.fireEvent("open");
    },
    onCloseOrCancelButtonClick: function() {
        Ext.destroy(this);
    }
});
